@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.3
+#!/usr/local/bin/python2.4
 
 ##############################################################################
 #
@@ -6,7 +6,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -113,7 +113,7 @@ Usage: loadmail2 [options]
 
        are equivalent
 
-$Id: zodbload.py,v 1.2.2.7 2004/06/09 18:42:04 tim_one Exp $
+$Id: zodbload.py 29533 2005-03-17 19:11:11Z tim_one $
 """
 
 import mailbox
@@ -124,6 +124,7 @@ import re
 import sys
 import threading
 import time
+import transaction
 
 class JobProducer:
 
@@ -172,7 +173,11 @@ class MBox:
             self._max = 0
             min = start = 0
 
-        f = open(filename)
+        if filename.endswith('.bz2'):
+            f = os.popen("bunzip2 <"+filename, 'r')
+            filename = filename[-4:]
+        else:
+            f = open(filename)
 
         self._mbox = mb = mailbox.UnixMailbox(f)
 
@@ -183,6 +188,7 @@ class MBox:
 
         self._lock = threading.Lock()
         self.__name__ = os.path.splitext(os.path.split(filename)[1])[0]
+        self._max = max
 
     def next(self):
         self._lock.acquire()
@@ -232,10 +238,10 @@ def setup(lib_python):
         os.remove(os.path.join(lib_python, '..', '..', 'var', 'Data.fs'))
     except:
         pass
-    import Zope
+    import Zope2
     import Products
     import AccessControl.SecurityManagement
-    app=Zope.app()
+    app=Zope2.app()
 
     Products.ZCatalog.ZCatalog.manage_addZCatalog(app, 'cat', '')
 
@@ -253,7 +259,7 @@ def setup(lib_python):
 
     app.cat.addIndex('PrincipiaSearchSource', 'ZCTextIndex', extra)
 
-    get_transaction().commit()
+    transaction.commit()
 
     system = AccessControl.SpecialUsers.system
     AccessControl.SecurityManagement.newSecurityManager(None, system)
@@ -273,7 +279,7 @@ def do(db, f, args):
     while 1:
         connection = db.open()
         try:
-            get_transaction().begin()
+            transaction.begin()
             t=time.time()
             c=time.clock()
             try:
@@ -281,7 +287,7 @@ def do(db, f, args):
                     r = f(connection, *args)
                 except ConflictError:
                     rconflicts += 1
-                    get_transaction().abort()
+                    transaction.abort()
                     continue
             finally:
                 wcomp += time.time() - t
@@ -291,11 +297,11 @@ def do(db, f, args):
             c=time.clock()
             try:
                 try:
-                    get_transaction().commit()
+                    transaction.commit()
                     break
                 except ConflictError:
                     wconflicts += 1
-                    get_transaction().abort()
+                    transaction.abort()
                     continue
             finally:
                 wcommit += time.time() - t
@@ -314,16 +320,16 @@ def run1(tid, db, factory, job, args):
         factory.__name__, r)
 
 def run(jobs, tid=''):
-    import Zope
+    import Zope2
     while 1:
         factory, job, args, repeatp = jobs.next()
-        run1(tid, Zope.DB, factory, job, args)
+        run1(tid, Zope2.DB, factory, job, args)
         if repeatp:
             while 1:
                 i = random.randint(0,100)
                 if i > repeatp:
                     break
-                run1(tid, Zope.DB, factory, job, args)
+                run1(tid, Zope2.DB, factory, job, args)
 
 
 def index(connection, messages, catalog, max):
@@ -471,7 +477,11 @@ def search(connection, terms, number):
         results = cat(PrincipiaSearchSource=term)
         n += len(results)
         for result in results:
-            did = result.getObject().getId()
+            obj = result.getObject()
+            # Apparently, there is a bug in Zope that leads obj to be None
+            # on occasion.
+            if obj is not None:
+                obj.getId()
 
     return n
 
@@ -777,8 +787,8 @@ def main(args=None):
     if options.has_key('setup'):
         setup(lib_python)
     else:
-        import Zope
-        Zope.startup()
+        import Zope2
+        Zope2.startup()
 
     #from ThreadedAsync.LoopCallback import loop
     #threading.Thread(target=loop, args=(), name='asyncore').start()
