@@ -1,28 +1,31 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
 #
 ##############################################################################
-"""Access control support"""
-
-__version__='$Revision: 1.56.14.4 $'[11:-2]
+"""Access control support
 
+$Id: Role.py 77298 2007-07-02 10:16:05Z shh $
+"""
+from cgi import escape
 
 from Globals import DTMLFile, MessageDialog, Dictionary
 from Acquisition import Implicit, Acquired, aq_get
 import Globals, ExtensionClass, PermissionMapping, Products
-from Permission import Permission
 from App.Common import aq_base
-from cgi import escape
+from zope.interface import implements
+
+from interfaces import IRoleManager
+from Permission import Permission
+from requestmethod import postonly
 
-ListType=type([])
 
 DEFAULTMAXLISTUSERS=250
 
@@ -32,9 +35,13 @@ def _isBeingUsedAsAMethod(self):
 def _isNotBeingUsedAsAMethod(self):
     return not aq_get(self, '_isBeingUsedAsAMethod_', 0)
 
+
 class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
+
     """An object that has configurable permissions"""
 
+    implements(IRoleManager)
+
     __ac_permissions__=(
         ('Change permissions',
          ('manage_access', 'permission_settings',
@@ -91,9 +98,10 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         return tuple(r)
 
     def permission_settings(self, permission=None):
-        """Return user-role permission settings. If 'permission' 
-           is passed to the method then only the settings for 'permission'
-           is returned.
+        """Return user-role permission settings.
+
+        If 'permission' is passed to the method then only the settings for
+        'permission' is returned.
         """
         result=[]
         valid=self.valid_roles()
@@ -110,7 +118,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             p=Permission(name,value,self)
             roles=p.getRoles(default=[])
             d={'name': name,
-               'acquire': type(roles) is ListType and 'CHECKED' or '',
+               'acquire': isinstance(roles, list) and 'CHECKED' or '',
                'roles': map(
                    lambda ir, roles=roles, valid=valid, ip=ip:
                    {
@@ -128,8 +136,10 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                              help_topic='Security_Manage-Role.stx',
                              help_product='OFSP')
 
+    @postonly
     def manage_role(self, role_to_manage, permissions=[], REQUEST=None):
-        "Change the permissions given to the given role"
+        """Change the permissions given to the given role.
+        """
         self._isBeingUsedAsAMethod(REQUEST, 0)
         for p in self.ac_inherited_permissions(1):
             name, value = p[:2]
@@ -143,8 +153,10 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                                  help_topic='Security_Manage-Acquisition.stx',
                                  help_product='OFSP')
 
+    @postonly
     def manage_acquiredPermissions(self, permissions=[], REQUEST=None):
-        "Change the permissions that acquire"
+        """Change the permissions that acquire.
+        """
         self._isBeingUsedAsAMethod(REQUEST, 0)
         for p in self.ac_inherited_permissions(1):
             name, value = p[:2]
@@ -161,13 +173,15 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                                    help_topic='Security_Manage-Permission.stx',
                                    help_product='OFSP')
 
+    @postonly
     def manage_permission(self, permission_to_manage,
                           roles=[], acquire=0, REQUEST=None):
-        """Change the settings for the given permission
+        """Change the settings for the given permission.
 
         If optional arg acquire is true, then the roles for the permission
         are acquired, in addition to the ones specified, otherwise the
-        permissions are restricted to only the designated roles."""
+        permissions are restricted to only the designated roles.
+        """
         self._isBeingUsedAsAMethod(REQUEST, 0)
         for p in self.ac_inherited_permissions(1):
             name, value = p[:2]
@@ -188,15 +202,18 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
     _method_manage_access=DTMLFile('dtml/methodAccess', globals())
 
     def manage_access(self, REQUEST, **kw):
-        "Return an interface for making permissions settings"
+        """Return an interface for making permissions settings.
+        """
         if hasattr(self, '_isBeingUsedAsAMethod') and \
            self._isBeingUsedAsAMethod():
             return apply(self._method_manage_access,(), kw)
         else:
             return apply(self._normal_manage_access,(), kw)
 
+    @postonly
     def manage_changePermissions(self, REQUEST):
-        "Change all permissions settings, called by management screen"
+        """Change all permissions settings, called by management screen.
+        """
         self._isBeingUsedAsAMethod(REQUEST, 0)
         valid_roles=self.valid_roles()
         indexes=range(len(valid_roles))
@@ -225,9 +242,9 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             message='Your changes have been saved',
             action ='manage_access')
 
-
     def permissionsOfRole(self, role):
-        "used by management screen"
+        """Used by management screen.
+        """
         r=[]
         for p in self.ac_inherited_permissions(1):
             name, value = p[:2]
@@ -239,7 +256,8 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         return r
 
     def rolesOfPermission(self, permission):
-        "used by management screen"
+        """Used by management screen.
+        """
         valid_roles=self.valid_roles()
         for p in self.ac_inherited_permissions(1):
             name, value = p[:2]
@@ -257,13 +275,14 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             "The permission <em>%s</em> is invalid." % escape(permission))
 
     def acquiredRolesAreUsedBy(self, permission):
-        "used by management screen"
+        """Used by management screen.
+        """
         for p in self.ac_inherited_permissions(1):
             name, value = p[:2]
             if name==permission:
                 p=Permission(name,value,self)
                 roles=p.getRoles()
-                return type(roles) is ListType and 'CHECKED' or ''
+                return isinstance(roles, list) and 'CHECKED' or ''
 
         raise ValueError, (
             "The permission <em>%s</em> is invalid." % escape(permission))
@@ -318,15 +337,16 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             aclu = getattr(aq_base(item), '__allow_groups__', _notfound)
             if aclu is not _notfound:
                 mlu = getattr(aclu, 'maxlistusers', _notfound)
-                if type(mlu) != type(1): mlu = DEFAULTMAXLISTUSERS
+                if not isinstance(mlu, int): mlu = DEFAULTMAXLISTUSERS
                 if mlu < 0: raise OverflowError
                 un = getattr(aclu, 'user_names', _notfound)
                 if un is not _notfound:
+                    un = aclu.__of__(item).user_names # rewrap
                     unl = un()
                     # maxlistusers of 0 is list all
                     if len(unl) > mlu and mlu != 0:
                         raise OverflowError
-                    for name in un():
+                    for name in unl:
                         dict[name]=1
             item = getattr(item, 'aq_parent', _notfound)
             if item is _notfound:
@@ -339,51 +359,58 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         dict=self.__ac_local_roles__ or {}
         return tuple(dict.get(userid, []))
 
+    @postonly
     def manage_addLocalRoles(self, userid, roles, REQUEST=None):
         """Set local roles for a user."""
         if not roles:
             raise ValueError, 'One or more roles must be given!'
-        dict=self.__ac_local_roles__ or {}
+        dict=self.__ac_local_roles__
+        if dict is None:
+            self.__ac_local_roles__ = dict = {}
         local_roles = list(dict.get(userid, []))
         for r in roles:
             if r not in local_roles:
                 local_roles.append(r)
         dict[userid] = local_roles
-        self.__ac_local_roles__=dict
+        self._p_changed=True
         if REQUEST is not None:
             stat='Your changes have been saved.'
             return self.manage_listLocalRoles(self, REQUEST, stat=stat)
 
+    @postonly
     def manage_setLocalRoles(self, userid, roles, REQUEST=None):
         """Set local roles for a user."""
         if not roles:
             raise ValueError, 'One or more roles must be given!'
-        dict=self.__ac_local_roles__ or {}
+        dict=self.__ac_local_roles__
+        if dict is None:
+            self.__ac_local_roles__ = dict = {}
         dict[userid]=roles
-        self.__ac_local_roles__=dict
+        self._p_changed=True
         if REQUEST is not None:
             stat='Your changes have been saved.'
             return self.manage_listLocalRoles(self, REQUEST, stat=stat)
 
+    @postonly
     def manage_delLocalRoles(self, userids, REQUEST=None):
         """Remove all local roles for a user."""
-        dict=self.__ac_local_roles__ or {}
+        dict=self.__ac_local_roles__
+        if dict is None:
+            self.__ac_local_roles__ = dict = {}
         for userid in userids:
             if dict.has_key(userid):
                 del dict[userid]
-        self.__ac_local_roles__=dict
+        self._p_changed=True
         if REQUEST is not None:
             stat='Your changes have been saved.'
             return self.manage_listLocalRoles(self, REQUEST, stat=stat)
 
-
-
-
     #------------------------------------------------------------
 
     access_debug_info__roles__=()
     def access_debug_info(self):
-        "Return debug info"
+        """Return debug info.
+        """
         clas=class_attrs(self)
         inst=instance_attrs(self)
         data=[]
@@ -403,7 +430,8 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         return data
 
     def valid_roles(self):
-        "Return list of valid roles"
+        """Return list of valid roles.
+        """
         obj=self
         dict={}
         dup =dict.has_key
@@ -423,7 +451,8 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         return tuple(roles)
 
     def validate_roles(self, roles):
-        "Return true if all given roles are valid"
+        """Return true if all given roles are valid.
+        """
         valid=self.valid_roles()
         for role in roles:
             if role not in valid:
@@ -431,16 +460,17 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         return 1
 
     def userdefined_roles(self):
-        "Return list of user-defined roles"
+        """Return list of user-defined roles.
+        """
         roles=list(self.__ac_roles__)
         for role in classattr(self.__class__,'__ac_roles__'):
             try:    roles.remove(role)
             except: pass
         return tuple(roles)
 
-
-    def manage_defined_roles(self,submit=None,REQUEST=None):
-        """Called by management screen."""
+    def manage_defined_roles(self, submit=None, REQUEST=None):
+        """Called by management screen.
+        """
 
         if submit=='Add Role':
             role=reqattr(REQUEST, 'role')
@@ -452,6 +482,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
         return self.manage_access(REQUEST)
 
+    @postonly
     def _addRole(self, role, REQUEST=None):
         if not role:
             return MessageDialog(
@@ -469,7 +500,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         if REQUEST is not None:
             return self.manage_access(REQUEST)
 
-
+    @postonly
     def _delRoles(self, roles, REQUEST=None):
         if not roles:
             return MessageDialog(
@@ -484,21 +515,19 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         if REQUEST is not None:
             return self.manage_access(REQUEST)
 
-
     def _has_user_defined_role(self, role):
         return role in self.__ac_roles__
 
-
     # Compatibility names only!!
 
     smallRolesWidget=selectedRoles=aclAChecked=aclPChecked=aclEChecked=''
     validRoles=valid_roles
     #manage_rolesForm=manage_access
 
-    def manage_editRoles(self,REQUEST,acl_type='A',acl_roles=[]):
+    def manage_editRoles(self, REQUEST, acl_type='A', acl_roles=[]):
         pass
 
-    def _setRoles(self,acl_type,acl_roles):
+    def _setRoles(self, acl_type, acl_roles):
         pass
 
     def possible_permissions(self):
@@ -516,7 +545,6 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
         return d
 
-
 Globals.default__class_init__(RoleManager)
 
 
