@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -39,12 +39,12 @@
 """
 
 import sys
+import logging
+logger = logging.getLogger('SecurityInfo')
 
 import Acquisition
 
 from AccessControl.ImplPython import _what_not_even_god_should_do
-from zLOG import LOG, WARNING
-
 
 # Security constants - these are imported into the AccessControl
 # namespace and can be referenced as AccessControl.PUBLIC etc.
@@ -69,8 +69,8 @@ class SecurityInfo(Acquisition.Implicit):
     def _setaccess(self, names, access):
         for name in names:
             if self.names.get(name, access) != access:
-                LOG('SecurityInfo', WARNING, 'Conflicting security '
-                    'declarations for "%s"' % name)
+                logger.warn('Conflicting security declarations for "%s"'
+                             % name)
                 self._warnings = 1
             self.names[name] = access
 
@@ -111,8 +111,8 @@ class SecurityInfo(Acquisition.Implicit):
         for role in roles:
             rdict[role] = 1
         if self.roles.get(permission_name, rdict) != rdict:
-            LOG('SecurityInfo', WARNING, 'Conflicting default role'
-                'declarations for permission "%s"' % permission_name)
+            logger.warn('Conflicting default role '
+                        'declarations for permission "%s"' % permission_name)
             self._warnings = 1
         self.roles[permission_name] = rdict
 
@@ -193,8 +193,8 @@ class ClassSecurityInfo(SecurityInfo):
                     access)
 
         if getattr(self, '_warnings', None):
-            LOG('SecurityInfo', WARNING, 'Class "%s" had conflicting '
-                'security declarations' % classobj.__name__)
+            logger.warn('Class "%s" had conflicting security declarations'
+                        % classobj.__name__)
 
 class ClassSecurityInformation(ClassSecurityInfo):
     # Default policy is disallow
@@ -273,8 +273,8 @@ class _ModuleSecurityInfo(SecurityInfo):
             dict['__allow_access_to_unprotected_subobjects__'] = self
 
         if getattr(self, '_warnings', None):
-            LOG('SecurityInfo', WARNING, 'Module "%s" had conflicting '
-                'security declarations' % dict['__name__'])
+            logger.warn('Module "%s" had conflicting security declarations'
+                        % dict['__name__'])
 
     declareProtected__roles__=ACCESS_PRIVATE
     def declareProtected(self, permission_name, *names):
