@@ -1,30 +1,30 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""User folder tests
+"""User folder tests.
+
+$Id: testUserFolder.py 40222 2005-11-18 15:46:28Z andreasjung $
 """
 
-__rcs_id__='$Id: testUserFolder.py 39545 2005-10-21 06:50:48Z tseaver $'
-__version__='$Revision: 1.10 $'[11:-2]
-
-import os, sys, base64, unittest
-
-from Testing.makerequest import makerequest
-
-import transaction
-
+import unittest
+import Testing
 import Zope2
 Zope2.startup()
 
+import os, sys, base64
+
+import transaction
+from Testing.makerequest import makerequest
+
 from AccessControl import Unauthorized
 from AccessControl.SecurityManagement import newSecurityManager
 from AccessControl.SecurityManagement import noSecurityManager
@@ -65,6 +65,13 @@ class UserFolderTests(unittest.TestCase):
         user = user.__of__(self.uf)
         newSecurityManager(None, user)
 
+    def test_z3interfaces(self):
+        from AccessControl.interfaces import IStandardUserFolder
+        from AccessControl.User import UserFolder
+        from zope.interface.verify import verifyClass
+
+        verifyClass(IStandardUserFolder, UserFolder)
+
     def testGetUser(self):
         self.failIfEqual(self.uf.getUser('user1'), None)
 
@@ -239,6 +246,7 @@ class UserFolderTests(unittest.TestCase):
         self.failUnless(uf._isPasswordEncrypted(user.__))
         self.failUnless(pw_validate(user.__, PASSWORD))
 
+
 class UserTests(unittest.TestCase):
 
     def testGetUserName(self):
@@ -267,6 +275,21 @@ class UserTests(unittest.TestCase):
         f = User('chris', '123', ['Manager'], [])
         self.assertEqual(f.getDomains(), ())
 
+    def testRepr(self):
+        f = User('flo', '123', ['Manager'], [])
+        self.assertEqual(repr(f), "<User 'flo'>")
+
+    def testReprSpecial(self):
+        from AccessControl.User import NullUnrestrictedUser
+        from AccessControl.User import nobody
+        from AccessControl.User import system
+        # NullUnrestrictedUser is used when there is no emergency user
+        self.assertEqual(repr(NullUnrestrictedUser()),
+                         "<NullUnrestrictedUser (None, None)>")
+        self.assertEqual(repr(nobody),
+                         "<SpecialUser 'Anonymous User'>")
+        self.assertEqual(repr(system),
+                         "<UnrestrictedUser 'System Processes'>")
 
 def test_suite():
     suite = unittest.TestSuite()
