@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -38,11 +38,14 @@ import os, re, zlib, marshal, cPickle
 from cgi import escape
 from urllib import quote
 
+import transaction
+
 import Globals, OFS.Folder, OFS.SimpleItem,  Acquisition, Products
 import ZClasses, AccessControl.Owned
 from OFS.Folder import Folder
 from HelpSys.HelpSys import ProductHelp
 from AccessControl import Unauthorized
+from AccessControl.Permissions import manage_zclasses
 
 from Factory import Factory
 from Permission import PermissionManager
@@ -58,7 +61,8 @@ class ProductFolder(Folder):
     meta_type ='Product Management'
     icon='p_/ProductFolder_icon'
 
-    all_meta_types={'name': 'Product', 'action': 'manage_addProductForm'},
+    all_meta_types={'name': 'Product', 'action': 'manage_addProductForm',
+                    'permission': manage_zclasses},
     meta_types=all_meta_types
 
     # This prevents subobjects from being owned!
@@ -386,7 +390,7 @@ class Product(Folder, PermissionManager):
         message = None
         if RefreshFuncs.performFullRefresh(self._p_jar, self.id):
             from ZODB import Connection
-            Connection.updateCodeTimestamp() # Clears cache in next connection.
+            Connection.resetCaches() # Clears cache in future connections.
             message = 'Product refreshed.'
         else:
             message = 'An exception occurred.'
@@ -580,7 +584,7 @@ def initializeProduct(productp, name, home, app):
              'help': ('OFSP','Product_Refresh.stx')},)
 
     if not doInstall():
-        get_transaction().abort()
+        transaction.abort()
         return product
 
     # Give the ZClass fixup code in Application
