@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -1503,6 +1503,7 @@ class DateTime:
                microseconds).strftime(format) 
         return format_is_unicode and unicode(ds, 'utf-8') or ds
 
+
     # General formats from previous DateTime
     def Date(self):
         """Return the date string for the object."""
@@ -1804,6 +1805,14 @@ class DateTime:
         d1 = (( d4 - L) % 365) + L
         return d1/7 + 1
 
+    def encode(self, out):
+        """
+        Encode value for XML-RPC
+        """
+        out.write('<value><dateTime.iso8601>')
+        out.write(self.ISO8601())
+        out.write('</dateTime.iso8601></value>\n')
+
 
 class strftimeFormatter:
 
