@@ -1,23 +1,27 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
 #
 ##############################################################################
-'''$Id: DT_Util.py,v 1.89.12.2 2004/04/29 20:34:29 caseman Exp $'''
-__version__='$Revision: 1.89.12.2 $'[11:-2]
+"""DTML Utilities
+
+$Id: DT_Util.py 40319 2005-11-22 15:49:55Z tseaver $"""
 
 import re
+
 from html_quote import html_quote, ustr # for import by other modules, dont remove!
 from RestrictedPython.Guards import safe_builtins
 from RestrictedPython.Utilities import utility_builtins
 from RestrictedPython.Eval import RestrictionCapableEval
+from cDocumentTemplate import InstanceDict, TemplateDict, \
+         render_blocks, safe_callable, join_unicode
 
 test = utility_builtins['test'] # for backwards compatibility, dont remove!
 
@@ -29,40 +33,34 @@ ParseError='Document Template Parse Error'
 from zExceptions import Unauthorized as ValidationError
 
 def int_param(params,md,name,default=0, st=type('')):
-    try: v=params[name]
-    except: v=default
+    v = params.get(name, default)
     if v:
-        try: v=int(v)
+        try:
+            v = int(v)
         except:
-            v=md[v]
-            if type(v) is st: v=int(v)
+            v = md[v]
+            if isinstance(v, str):
+                v = int(v)
     return v or 0
 
-import ExtensionClass
-from cDocumentTemplate import InstanceDict, TemplateDict, \
-     render_blocks, safe_callable, join_unicode
-
 functype = type(int_param)
 class NotBindable:
     # Used to prevent TemplateDict from trying to bind to functions.
     def __init__(self, f):
         self.__call__ = f
 
-d = TemplateDict.__dict__
 for name, f in safe_builtins.items() + utility_builtins.items():
     if type(f) is functype:
-        d[name] = NotBindable(f)
-    else:
-        d[name] = f
+        f = NotBindable(f)
+    setattr(TemplateDict, name, f)
 
 if LIMITED_BUILTINS:
     # Replace certain builtins with limited versions.
     from RestrictedPython.Limits import limited_builtins
     for name, f in limited_builtins.items():
         if type(f) is functype:
-            d[name] = NotBindable(f)
-        else:
-            d[name] = f
+            f = NotBindable(f)
+        setattr(TemplateDict, name, f)
 
 try:
     # Wrap the string module so it can deal with TaintedString strings.
@@ -101,7 +99,7 @@ try:
                 retval = TaintedString(retval)
             return retval
 
-    d['string'] = StringModuleWrapper()
+    TemplateDict.string = StringModuleWrapper()
 
 except ImportError:
     # Use the string module already defined in RestrictedPython.Utilities
@@ -135,8 +133,8 @@ def careful_hasattr(md, inst, name):
     else:
         return 1
 
-d['getattr']=careful_getattr
-d['hasattr']=careful_hasattr
+TemplateDict.getattr = careful_getattr
+TemplateDict.hasattr = careful_hasattr
 
 def namespace(self, **kw):
     """Create a tuple consisting of a single instance whose attributes are
@@ -149,7 +147,7 @@ def namespace(self, **kw):
         information may contain more details.)'''
     return self(**kw)
 
-d['namespace']=namespace
+TemplateDict.namespace = namespace
 
 def render(self, v):
     "Render an object in the way done by the 'name' attribute"
@@ -164,7 +162,7 @@ def render(self, v):
                 v = v()
     return v
 
-d['render']=render
+TemplateDict.render = render
 
 
 class Eval(RestrictionCapableEval):
@@ -317,7 +315,10 @@ Python expression support
 
   - Special security-aware versions of 'getattr' and 'hasattr',
 
-  - The Python 'string', 'math', and 'whrandom' modules, and
+  - The Python 'string', 'math', modules,
+
+  - BBB alias for the Python 'whrandom' module (will be removed in
+    Zope 2.10),
 
   - A special function, 'test', that supports if-then expressions.
     The 'test' function accepts any number of arguments.  If the
