@@ -1,39 +1,62 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-__doc__="""Copy interface"""
-__version__='$Revision: 1.85.2.9 $'[11:-2]
+"""Copy interface
 
-import re, sys, Globals, Moniker, tempfile, ExtensionClass
+$Id: CopySupport.py 40928 2005-12-21 00:04:08Z efge $
+"""
+
+import re, sys, tempfile
+import warnings
+from cgi import escape
 from marshal import loads, dumps
 from urllib import quote, unquote
 from zlib import compress, decompress
 
-from App.Dialogs import MessageDialog
+import Globals, Moniker, ExtensionClass
+import transaction
 from AccessControl import getSecurityManager
 from AccessControl.Permissions import delete_objects as DeleteObjects
 from Acquisition import aq_base, aq_inner, aq_parent
-from zExceptions import Unauthorized, BadRequest
+from App.Dialogs import MessageDialog
 from webdav.Lockable import ResourceLockedError
-from cgi import escape
+from zExceptions import Unauthorized, BadRequest
+from ZODB.POSException import ConflictError
+from zope.interface import implements
+from zope.event import notify
+from zope.app.event.objectevent import ObjectCopiedEvent
+from zope.app.container.contained import ObjectMovedEvent
+from zope.app.container.contained import notifyContainerModified
+from OFS.event import ObjectWillBeMovedEvent
+from OFS.event import ObjectClonedEvent
+import OFS.subscribers
+
+from OFS.interfaces import ICopyContainer
+from OFS.interfaces import ICopySource
+
 
 CopyError='Copy Error'
 
 copy_re = re.compile('^copy([0-9]*)_of_(.*)')
 
 _marker=[]
+
+
 class CopyContainer(ExtensionClass.Base):
+
     """Interface for containerish objects which allow cut/copy/paste"""
 
+    implements(ICopyContainer)
+
     __ac_permissions__=(
         ('View management screens',
          ('manage_copyObjects', 'manage_pasteObjects',
@@ -42,17 +65,20 @@ class CopyContainer(ExtensionClass.Base):
          ('manage_cutObjects',)),
         )
 
-
     # The following three methods should be overridden to store sub-objects
     # as non-attributes.
-    def _setOb(self, id, object): setattr(self, id, object)
-    def _delOb(self, id): delattr(self, id)
-    def _getOb(self, id, default=_marker):
-        self = aq_base(self)
-        if default is _marker: return getattr(self, id)
-        try: return getattr(self, id)
-        except: return default
+    def _setOb(self, id, object):
+        setattr(self, id, object)
 
+    def _delOb(self, id):
+        delattr(self, id)
+
+    def _getOb(self, id, default=_marker):
+        if hasattr(aq_base(self), id):
+            return getattr(self, id)
+        if default is _marker:
+            raise AttributeError, id
+        return default
 
     def manage_CopyContainerFirstItem(self, REQUEST):
         return self._getOb(REQUEST['ids'][0])
@@ -133,86 +159,145 @@ class CopyContainer(ExtensionClass.Base):
 
     def manage_pasteObjects(self, cb_copy_data=None, REQUEST=None):
         """Paste previously copied objects into the current object.
-           If calling manage_pasteObjects from python code, pass
-           the result of a previous call to manage_cutObjects or
-           manage_copyObjects as the first argument."""
-        cp=None
+
+        If calling manage_pasteObjects from python code, pass the result of a
+        previous call to manage_cutObjects or manage_copyObjects as the first
+        argument.
+
+        Also sends IObjectCopiedEvent and IObjectClonedEvent
+        or IObjectWillBeMovedEvent and IObjectMovedEvent.
+        """
         if cb_copy_data is not None:
-            cp=cb_copy_data
+            cp = cb_copy_data
+        elif REQUEST is not None and REQUEST.has_key('__cp'):
+            cp = REQUEST['__cp']
         else:
-            if REQUEST and REQUEST.has_key('__cp'):
-                cp=REQUEST['__cp']
+            cp = None
         if cp is None:
             raise CopyError, eNoData
 
-        try:    cp=_cb_decode(cp)
-        except: raise CopyError, eInvalid
+        try:
+            op, mdatas = _cb_decode(cp)
+        except:
+            raise CopyError, eInvalid
 
-        oblist=[]
-        op=cp[0]
+        oblist = []
         app = self.getPhysicalRoot()
-        result = []
-
-        for mdata in cp[1]:
+        for mdata in mdatas:
             m = Moniker.loadMoniker(mdata)
-            try: ob = m.bind(app)
-            except: raise CopyError, eNotFound
+            try:
+                ob = m.bind(app)
+            except ConflictError:
+                raise
+            except:
+                raise CopyError, eNotFound
             self._verifyObjectPaste(ob, validate_src=op+1)
             oblist.append(ob)
 
-        if op==0:
+        result = []
+        if op == 0:
             # Copy operation
             for ob in oblist:
+                orig_id = ob.getId()
                 if not ob.cb_isCopyable():
-                    raise CopyError, eNotSupported % escape(ob.getId())
-                try:    ob._notifyOfCopyTo(self, op=0)
-                except: raise CopyError, MessageDialog(
-                    title='Copy Error',
-                    message=sys.exc_info()[1],
-                    action ='manage_main')
-                ob=ob._getCopy(self)
-                orig_id=ob.getId()
-                id=self._get_id(ob.getId())
-                result.append({'id':orig_id, 'new_id':id})
+                    raise CopyError, eNotSupported % escape(orig_id)
+
+                try:
+                    ob._notifyOfCopyTo(self, op=0)
+                except ConflictError:
+                    raise
+                except:
+                    raise CopyError, MessageDialog(
+                        title="Copy Error",
+                        message=sys.exc_info()[1],
+                        action='manage_main')
+
+                id = self._get_id(orig_id)
+                result.append({'id': orig_id, 'new_id': id})
+
+                orig_ob = ob
+                ob = ob._getCopy(self)
                 ob._setId(id)
+                notify(ObjectCopiedEvent(ob, orig_ob))
+
                 self._setObject(id, ob)
                 ob = self._getOb(id)
-                ob.manage_afterClone(ob)
                 ob.wl_clearLocks()
 
+                ob._postCopy(self, op=0)
+
+                OFS.subscribers.compatibilityCall('manage_afterClone', ob, ob)
+
+                notify(ObjectClonedEvent(ob))
+
             if REQUEST is not None:
                 return self.manage_main(self, REQUEST, update_menu=1,
                                         cb_dataValid=1)
 
-        if op==1:
+        elif op == 1:
             # Move operation
             for ob in oblist:
-                id=ob.getId()
+                orig_id = ob.getId()
                 if not ob.cb_isMoveable():
-                    raise CopyError, eNotSupported % escape(id)
-                try:    ob._notifyOfCopyTo(self, op=1)
-                except: raise CopyError, MessageDialog(
-                    title='Move Error',
-                    message=sys.exc_info()[1],
-                    action ='manage_main')
+                    raise CopyError, eNotSupported % escape(orig_id)
+
+                try:
+                    ob._notifyOfCopyTo(self, op=1)
+                except ConflictError:
+                    raise
+                except:
+                    raise CopyError, MessageDialog(
+                        title="Move Error",
+                        message=sys.exc_info()[1],
+                        action='manage_main')
+
                 if not sanity_check(self, ob):
-                    raise CopyError, 'This object cannot be pasted into itself'
+                    raise CopyError, "This object cannot be pasted into itself"
+
+                orig_container = aq_parent(aq_inner(ob))
+                if aq_base(orig_container) is aq_base(self):
+                    id = orig_id
+                else:
+                    id = self._get_id(orig_id)
+                result.append({'id': orig_id, 'new_id': id})
+
+                notify(ObjectWillBeMovedEvent(ob, orig_container, orig_id,
+                                              self, id))
 
                 # try to make ownership explicit so that it gets carried
                 # along to the new location if needed.
                 ob.manage_changeOwnershipType(explicit=1)
 
-                aq_parent(aq_inner(ob))._delObject(id)
+                try:
+                    orig_container._delObject(orig_id, suppress_events=True)
+                except TypeError:
+                    # BBB: removed in Zope 2.11
+                    orig_container._delObject(orig_id)
+                    warnings.warn(
+                        "%s._delObject without suppress_events is deprecated "
+                        "and will be removed in Zope 2.11." %
+                        orig_container.__class__.__name__, DeprecationWarning)
                 ob = aq_base(ob)
-                orig_id=id
-                id=self._get_id(id)
-                result.append({'id':orig_id, 'new_id':id })
                 ob._setId(id)
 
-                self._setObject(id, ob, set_owner=0)
+                try:
+                    self._setObject(id, ob, set_owner=0, suppress_events=True)
+                except TypeError:
+                    # BBB: removed in Zope 2.11
+                    self._setObject(id, ob, set_owner=0)
+                    warnings.warn(
+                        "%s._setObject without suppress_events is deprecated "
+                        "and will be removed in Zope 2.11." %
+                        self.__class__.__name__, DeprecationWarning)
+                ob = self._getOb(id)
 
+                notify(ObjectMovedEvent(ob, orig_container, orig_id, self, id))
+                notifyContainerModified(orig_container)
+                if aq_base(orig_container) is not aq_base(self):
+                    notifyContainerModified(self)
+
+                ob._postCopy(self, op=1)
                 # try to make ownership implicit if possible
-                ob=self._getOb(id)
                 ob.manage_changeOwnershipType(explicit=0)
 
             if REQUEST is not None:
@@ -222,8 +307,8 @@ class CopyContainer(ExtensionClass.Base):
                 REQUEST['__cp'] = None
                 return self.manage_main(self, REQUEST, update_menu=1,
                                         cb_dataValid=0)
-        return result
 
+        return result
 
     manage_renameForm=Globals.DTMLFile('dtml/renameForm', globals())
 
@@ -239,30 +324,66 @@ class CopyContainer(ExtensionClass.Base):
         return None
 
     def manage_renameObject(self, id, new_id, REQUEST=None):
-        """Rename a particular sub-object"""
-        try: self._checkId(new_id)
-        except: raise CopyError, MessageDialog(
-                      title='Invalid Id',
-                      message=sys.exc_info()[1],
-                      action ='manage_main')
-        ob=self._getOb(id)
+        """Rename a particular sub-object.
+        """
+        try:
+            self._checkId(new_id)
+        except:
+            raise CopyError, MessageDialog(
+                title='Invalid Id',
+                message=sys.exc_info()[1],
+                action ='manage_main')
+
+        ob = self._getOb(id)
+
         if ob.wl_isLocked():
-            raise ResourceLockedError, 'Object "%s" is locked via WebDAV' % ob.getId()
+            raise ResourceLockedError, ('Object "%s" is locked via WebDAV'
+                                        % ob.getId())
         if not ob.cb_isMoveable():
             raise CopyError, eNotSupported % escape(id)
         self._verifyObjectPaste(ob)
-        try:    ob._notifyOfCopyTo(self, op=1)
-        except: raise CopyError, MessageDialog(
-                      title='Rename Error',
-                      message=sys.exc_info()[1],
-                      action ='manage_main')
-        self._delObject(id)
+
+        try:
+            ob._notifyOfCopyTo(self, op=1)
+        except ConflictError:
+            raise
+        except:
+            raise CopyError, MessageDialog(
+                title="Rename Error",
+                message=sys.exc_info()[1],
+                action ='manage_main')
+
+        notify(ObjectWillBeMovedEvent(ob, self, id, self, new_id))
+
+        try:
+            self._delObject(id, suppress_events=True)
+        except TypeError:
+            # BBB: removed in Zope 2.11
+            self._delObject(id)
+            warnings.warn(
+                "%s._delObject without suppress_events is deprecated "
+                "and will be removed in Zope 2.11." %
+                self.__class__.__name__, DeprecationWarning)
         ob = aq_base(ob)
         ob._setId(new_id)
 
         # Note - because a rename always keeps the same context, we
         # can just leave the ownership info unchanged.
-        self._setObject(new_id, ob, set_owner=0)
+        try:
+            self._setObject(new_id, ob, set_owner=0, suppress_events=True)
+        except TypeError:
+            # BBB: removed in Zope 2.11
+            self._setObject(new_id, ob, set_owner=0)
+            warnings.warn(
+                "%s._setObject without suppress_events is deprecated "
+                "and will be removed in Zope 2.11." %
+                self.__class__.__name__, DeprecationWarning)
+        ob = self._getOb(new_id)
+
+        notify(ObjectMovedEvent(ob, self, id, self, new_id))
+        notifyContainerModified(self)
+
+        ob._postCopy(self, op=1)
 
         if REQUEST is not None:
             return self.manage_main(self, REQUEST, update_menu=1)
@@ -274,25 +395,44 @@ class CopyContainer(ExtensionClass.Base):
     # Because it's still a "management" function.
     manage_clone__roles__=None
     def manage_clone(self, ob, id, REQUEST=None):
-        # Clone an object, creating a new object with the given id.
+        """Clone an object, creating a new object with the given id.
+        """
         if not ob.cb_isCopyable():
             raise CopyError, eNotSupported % escape(ob.getId())
-        try: self._checkId(id)
-        except: raise CopyError, MessageDialog(
-                      title='Invalid Id',
-                      message=sys.exc_info()[1],
-                      action ='manage_main')
+        try:
+            self._checkId(id)
+        except:
+            raise CopyError, MessageDialog(
+                title='Invalid Id',
+                message=sys.exc_info()[1],
+                action ='manage_main')
+
         self._verifyObjectPaste(ob)
-        try:    ob._notifyOfCopyTo(self, op=0)
-        except: raise CopyError, MessageDialog(
-                      title='Clone Error',
-                      message=sys.exc_info()[1],
-                      action ='manage_main')
-        ob=ob._getCopy(self)
+
+        try:
+            ob._notifyOfCopyTo(self, op=0)
+        except ConflictError:
+            raise
+        except:
+            raise CopyError, MessageDialog(
+                title="Clone Error",
+                message=sys.exc_info()[1],
+                action='manage_main')
+
+        orig_ob = ob
+        ob = ob._getCopy(self)
         ob._setId(id)
+        notify(ObjectCopiedEvent(ob, orig_ob))
+
         self._setObject(id, ob)
-        ob=ob.__of__(self)
-        ob.manage_afterClone(ob)
+        ob = self._getOb(id)
+
+        ob._postCopy(self, op=0)
+
+        OFS.subscribers.compatibilityCall('manage_afterClone', ob, ob)
+
+        notify(ObjectClonedEvent(ob))
+
         return ob
 
     def cb_dataValid(self):
@@ -333,7 +473,7 @@ class CopyContainer(ExtensionClass.Base):
         if not hasattr(object, 'meta_type'):
             raise CopyError, MessageDialog(
                   title   = 'Not Supported',
-                  message = ('The object <EM>%s</EM> does not support this' \
+                  message = ('The object <em>%s</em> does not support this' \
                              ' operation' % escape(absattr(object.id))),
                   action  = 'manage_main')
 
@@ -353,68 +493,48 @@ class CopyContainer(ExtensionClass.Base):
                 mt_permission = d.get('permission')
                 break
 
-        if method_name:
-            try:
-                method = self.restrictedTraverse(method_name)
-                # method_name is e.g.
-                # "manage_addProduct/PageTemplates/manage_addPageTemplateForm".
-                # restrictedTraverse will raise Unauthorized if it
-                # can't obtain the factory method by name due to a
-                # security restriction.  We depend on this side effect
-                # here!  Note that we use restrictedTraverse as
-                # opposed to checkPermission to take into account the
-                # special security circumstances related to proxy
-                # roles.  See collector #78.
+        if mt_permission is not None:
+            sm = getSecurityManager()
 
-            except Unauthorized:
-                if mt_permission:
+            if sm.checkPermission(mt_permission, self):
+                if validate_src:
+                    # Ensure the user is allowed to access the object on the
+                    # clipboard.
+                    try:
+                        parent = aq_parent(aq_inner(object))
+                    except:
+                        parent = None
+
+                    if not sm.validate(None, parent, None, object):
+                        raise Unauthorized(absattr(object.id))
+
+                    if validate_src == 2: # moving
+                        if not sm.checkPermission(DeleteObjects, parent):
+                            raise Unauthorized('Delete not allowed.')
+            else:
+                raise CopyError, MessageDialog(
+                    title = 'Insufficient Privileges',
                     message = ('You do not possess the %s permission in the '
                                'context of the container into which you are '
                                'pasting, thus you are not able to perform '
-                               'this operation.' % mt_permission)
-                else:
-                    message = ('You do not possess the permission required '
-                               'to call %s in the context of the container '
-                               'into which you are pasting, thus you are not '
-                               'able to perform this operation.' % method_name)
-
-                raise CopyError, MessageDialog(
-                  title = 'Insufficient Privileges',
-                  message = message,
-                  action = 'manage_main')
-
-            if validate_src:
-
-                sm = getSecurityManager()
-
-                # Ensure the user is allowed to access the object on the
-                # clipboard.
-                try:
-                    parent = aq_parent(aq_inner(object))
-                except:
-                    parent = None
-
-                if not sm.validate(None,parent,None,object):
-                    raise Unauthorized, absattr(object.id)
-
-                if validate_src == 2: # moving
-                    if not sm.checkPermission(DeleteObjects, parent):
-                        raise Unauthorized, 'Delete not allowed.'
-
-        else: # /if method_name
+                               'this operation.' % mt_permission),
+                    action = 'manage_main')
+        else:
             raise CopyError, MessageDialog(
-                  title   = 'Not Supported',
-                  message = ('The object <EM>%s</EM> does not support this '
-                             'operation.' % escape(absattr(object.id))),
-                  action  = 'manage_main')
+                title = 'Not Supported',
+                message = ('The object <em>%s</em> does not support this '
+                           'operation.' % escape(absattr(object.id))),
+                action = 'manage_main')
 
 Globals.default__class_init__(CopyContainer)
 
 
-
 class CopySource(ExtensionClass.Base):
+
     """Interface for objects which allow themselves to be copied."""
 
+    implements(ICopySource)
+
     # declare a dummy permission for Copy or Move here that we check
     # in cb_isCopyable.
     __ac_permissions__=(
@@ -422,14 +542,18 @@ class CopySource(ExtensionClass.Base):
         )
 
     def _canCopy(self, op=0):
-        """Called to make sure this object is copyable. The op var
-        is 0 for a copy, 1 for a move."""
+        """Called to make sure this object is copyable.
+
+        The op var is 0 for a copy, 1 for a move.
+        """
         return 1
 
     def _notifyOfCopyTo(self, container, op=0):
-        """Overide this to be pickly about where you go! If you dont
-        want to go there, raise an exception. The op variable is
-        0 for a copy, 1 for a move."""
+        """Overide this to be pickly about where you go!
+
+        If you dont want to go there, raise an exception. The op variable is 0
+        for a copy, 1 for a move.
+        """
         pass
 
     def _getCopy(self, container):
@@ -437,7 +561,7 @@ class CopySource(ExtensionClass.Base):
         # 1) Make sure the data about to be exported is current
         # 2) Ensure self._p_jar and container._p_jar are set even if
         #    either one is a new object
-        get_transaction().commit(1)
+        transaction.savepoint(optimistic=True)
 
         if self._p_jar is None:
             raise CopyError, (
