@@ -1,21 +1,21 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
 #
 ##############################################################################
-"""Image object"""
-
-__version__='$Revision: 1.145.2.10 $'[11:-2]
+"""Image object
 
+$Id: Image.py 69770 2006-08-24 16:46:48Z shh $
+"""
 import Globals, struct
-from OFS.content_types import guess_content_type
+from zope.app.content_types import guess_content_type
 from Globals import DTMLFile
 from PropertyManager import PropertyManager
 from AccessControl.Role import RoleManager
@@ -34,6 +34,7 @@ from ZPublisher.HTTPRequest import FileUpload
 from ZPublisher.Iterators import filestream_iterator
 from zExceptions import Redirect
 from cgi import escape
+import transaction
 
 StringType=type('')
 manage_addFileForm=DTMLFile('dtml/imageAdd', globals(),Kind='File',kind='file')
@@ -76,7 +77,6 @@ class File(Persistent, Implicit, PropertyManager,
 
     precondition=''
     size=None
-    alt=''
 
     manage_editForm  =DTMLFile('dtml/fileEdit',globals(),
                                Kind='File',kind='file')
@@ -105,7 +105,7 @@ class File(Persistent, Implicit, PropertyManager,
          ('manage_edit','manage_upload','PUT')),
         ('View',
          ('index_html', 'view_image_or_file', 'get_size',
-          'getContentType', '')),
+          'getContentType', 'PrincipiaSearchSource', '')),
         ('FTP access',
          ('manage_FTPstat','manage_FTPget','manage_FTPlist')),
         ('Delete objects',
@@ -114,7 +114,6 @@ class File(Persistent, Implicit, PropertyManager,
 
 
     _properties=({'id':'title', 'type': 'string'},
-                 {'id':'alt', 'type':'string'},
                  {'id':'content_type', 'type':'string'},
                  )
 
@@ -218,7 +217,7 @@ class File(Persistent, Implicit, PropertyManager,
                     return True
 
                 ranges = HTTPRangeSupport.expandRanges(ranges, self.size)
-                                
+
                 if len(ranges) == 1:
                     # Easy case, set extra header and return partial set.
                     start, end = ranges[0]
@@ -364,6 +363,12 @@ class File(Persistent, Implicit, PropertyManager,
 
         if self._if_modified_since_request_handler(REQUEST, RESPONSE):
             # we were able to handle this by returning a 304
+            # unfortunately, because the HTTP cache manager uses the cache
+            # API, and because 304 responses are required to carry the Expires
+            # header for HTTP/1.1, we need to call ZCacheable_set here.
+            # This is nonsensical for caches other than the HTTP cache manager
+            # unfortunately.
+            self.ZCacheable_set(None)
             return ''
 
         if self.precondition and hasattr(self, str(self.precondition)):
@@ -395,10 +400,10 @@ class File(Persistent, Implicit, PropertyManager,
                 return result
 
         self.ZCacheable_set(None)
-            
+
         data=self.data
         if type(data) is type(''):
-            RESPONSE.setBase(None) 
+            RESPONSE.setBase(None)
             return data
 
         while data is not None:
@@ -413,6 +418,13 @@ class File(Persistent, Implicit, PropertyManager,
         """
         raise Redirect, URL1
 
+    def PrincipiaSearchSource(self):
+        """ Allow file objects to be searched.
+        """
+        if self.content_type.startswith('text/'):
+            return str(self.data)
+        return ''
+
     # private
     update_data__roles__=()
     def update_data(self, data, content_type=None, size=None):
@@ -502,11 +514,9 @@ class File(Persistent, Implicit, PropertyManager,
 
         # Make sure we have an _p_jar, even if we are a new object, by
         # doing a sub-transaction commit.
-        get_transaction().commit(1)
+        transaction.savepoint(optimistic=True)
 
-        jar=self._p_jar
-
-        if jar is None:
+        if self._p_jar is None:
             # Ugh
             seek(0)
             return Pdata(read(size)), size
@@ -515,31 +525,29 @@ class File(Persistent, Implicit, PropertyManager,
         # to front to minimize the number of database updates
         # and to allow us to get things out of memory as soon as
         # possible.
-        next=None
+        next = None
         while end > 0:
-            pos=end-n
-            if pos < n: pos=0 # we always want at least n bytes
+            pos = end-n
+            if pos < n:
+                pos = 0 # we always want at least n bytes
             seek(pos)
-            data=Pdata(read(end-pos))
 
-            # Woooop Woooop Woooop! This is a trick.
-            # We stuff the data directly into our jar to reduce the
-            # number of updates necessary.
-            data._p_jar=jar
+            # Create the object and assign it a next pointer
+            # in the same transaction, so that there is only
+            # a single database update for it.
+            data = Pdata(read(end-pos))
+            self._p_jar.add(data)
+            data.next = next
 
-            # This is needed and has side benefit of getting
-            # the thing registered:
-            data.next=next
+            # Save the object so that we can release its memory.
+            transaction.savepoint(optimistic=True)
+            data._p_deactivate()
+            # The object should be assigned an oid and be a ghost.
+            assert data._p_oid is not None
+            assert data._p_state == -1
 
-            # Now make it get saved in a sub-transaction!
-            get_transaction().commit(1)
-
-            # Now make it a ghost to free the memory.  We
-            # don't need it anymore!
-            data._p_changed=None
-
-            next=data
-            end=pos
+            next = data
+            end = pos
 
         return next, size
 
@@ -584,6 +592,8 @@ class File(Persistent, Implicit, PropertyManager,
 
     def manage_FTPget(self):
         """Return body for ftp."""
+        RESPONSE = self.REQUEST.RESPONSE
+
         if self.ZCacheable_isCachingEnabled():
             result = self.ZCacheable_get(default=None)
             if result is not None:
@@ -592,9 +602,19 @@ class File(Persistent, Implicit, PropertyManager,
                 # from FileCacheManager.
                 # the content-length is required here by HTTPResponse, even
                 # though FTP doesn't use it.
-                self.REQUEST.RESPONSE.setHeader('Content-Length', self.size)
+                RESPONSE.setHeader('Content-Length', self.size)
                 return result
-        return str(self.data)
+
+        data = self.data
+        if type(data) is type(''):
+            RESPONSE.setBase(None)
+            return data
+
+        while data is not None:
+            RESPONSE.write(data.data)
+            data = data.next
+
+        return ''
 
 manage_addImageForm=DTMLFile('dtml/imageAdd',globals(),
                              Kind='Image',kind='image')
@@ -697,7 +717,7 @@ class Image(File):
     __implements__ = (WriteLockInterface,)
     meta_type='Image'
 
-
+    alt=''
     height=''
     width=''
 
@@ -716,6 +736,7 @@ class Image(File):
         )
 
     _properties=({'id':'title', 'type': 'string'},
+                 {'id':'alt', 'type':'string'},
                  {'id':'content_type', 'type':'string','mode':'w'},
                  {'id':'height', 'type':'string'},
                  {'id':'width', 'type':'string'},
@@ -759,6 +780,7 @@ class Image(File):
 
         self.ZCacheable_invalidate()
         self.ZCacheable_set(None)
+        self.http__refreshEtag()
 
     def __str__(self):
         return self.tag()
@@ -808,7 +830,7 @@ class Image(File):
         if width:
             result = '%s width="%s"' % (result, width)
 
-         # Collector #1557
+        # Omitting 'border' attribute (Collector #1557)
 #        if not 'border' in [ x.lower() for x in  args.keys()]:
 #            result = '%s border="0"' % result
 
@@ -859,4 +881,3 @@ class Pdata(Persistent, Implicit):
             next=self.next
 
         return ''.join(r)
-
