@@ -1,45 +1,56 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """Object Manager
 
-$Id: ObjectManager.py 38738 2005-10-04 11:02:52Z efge $
+$Id: ObjectManager.py 71154 2006-11-17 15:50:39Z andreasjung $
 """
 
-import sys, fnmatch, copy, os, re
+import warnings
 import marshal
+import sys, fnmatch, copy, os, re
 from cgi import escape
 from cStringIO import StringIO
+import logging
+logger = logging.getLogger('FTP')
 from types import StringType, UnicodeType
 
-import App.Management, Acquisition, Globals, Products
+import App.Common
 import App.FactoryDispatcher, Products
+import App.Management, Acquisition, Globals, Products
+from AccessControl import getSecurityManager
+from AccessControl.ZopeSecurityPolicy import getRoles
+from Acquisition import aq_base
+from App.config import getConfiguration
 from Globals import DTMLFile, Persistent
 from Globals import MessageDialog, default__class_init__
 from Globals import REPLACEABLE, NOT_REPLACEABLE, UNIQUE
-from webdav.NullResource import NullResource
 from webdav.Collection import Collection
-from Acquisition import aq_base
 from webdav.Lockable import ResourceLockedError
-from ZODB.POSException import ConflictError
-import App.Common
-from App.config import getConfiguration
-from AccessControl import getSecurityManager
-from AccessControl.ZopeSecurityPolicy import getRoles
-from zLOG import LOG, ERROR
+from webdav.NullResource import NullResource
 from zExceptions import BadRequest
+from ZODB.POSException import ConflictError
+from zope.interface import implements
 
-from OFS.Traversable import Traversable
 import CopySupport
+from interfaces import IObjectManager
+from Traversable import Traversable
+from zope.event import notify
+from zope.app.container.contained import ObjectAddedEvent
+from zope.app.container.contained import ObjectRemovedEvent
+from zope.app.container.contained import notifyContainerModified
+from OFS.event import ObjectWillBeAddedEvent
+from OFS.event import ObjectWillBeRemovedEvent
+import OFS.subscribers
 
 # the name BadRequestException is relied upon by 3rd-party code
 BadRequestException = BadRequest
@@ -122,6 +133,8 @@ class ObjectManager(
     This class provides core behavior for collections of heterogeneous objects.
     """
 
+    implements(IObjectManager)
+
     __ac_permissions__=(
         ('View management screens', ('manage_main',)),
         ('Access contents information',
@@ -155,6 +168,8 @@ class ObjectManager(
 
     isPrincipiaFolderish=1
 
+    has_order_support = 0 # See OrderSupport.py
+
     def __class_init__(self):
         try:    mt=list(self.meta_types)
         except: mt=[]
@@ -261,11 +276,31 @@ class ObjectManager(
             raise AttributeError, id
         return default
 
-    def _setObject(self, id, object, roles=None, user=None, set_owner=1):
-        v=self._checkId(id)
-        if v is not None: id=v
-        try:    t=object.meta_type
-        except: t=None
+    def hasObject(self, id):
+        """Indicate whether the folder has an item by ID.
+
+        This doesn't try to be more intelligent than _getOb, and doesn't
+        consult _objects (for performance reasons). The common use case
+        is to check that an object does *not* exist.
+        """
+        if (id in ('.', '..') or
+            id.startswith('_') or
+            id.startswith('aq_') or
+            id.endswith('__')):
+            return False
+        return getattr(aq_base(self), id, None) is not None
+
+    def _setObject(self, id, object, roles=None, user=None, set_owner=1,
+                   suppress_events=False):
+        """Set an object into this container.
+
+        Also sends IObjectWillBeAddedEvent and IObjectAddedEvent.
+        """
+        ob = object # better name, keep original function signature
+        v = self._checkId(id)
+        if v is not None:
+            id = v
+        t = getattr(ob, 'meta_type', None)
 
         # If an object by the given id already exists, remove it.
         for object_info in self._objects:
@@ -273,78 +308,64 @@ class ObjectManager(
                 self._delObject(id)
                 break
 
-        self._objects=self._objects+({'id':id,'meta_type':t},)
-        self._setOb(id,object)
-        object=self._getOb(id)
+        if not suppress_events:
+            notify(ObjectWillBeAddedEvent(ob, self, id))
+
+        self._objects = self._objects + ({'id': id, 'meta_type': t},)
+        self._setOb(id, ob)
+        ob = self._getOb(id)
 
         if set_owner:
-            object.manage_fixupOwnershipAfterAdd()
+            # TODO: eventify manage_fixupOwnershipAfterAdd
+            # This will be called for a copy/clone, or a normal _setObject.
+            ob.manage_fixupOwnershipAfterAdd()
 
             # Try to give user the local role "Owner", but only if
             # no local roles have been set on the object yet.
-            if hasattr(object, '__ac_local_roles__'):
-                if object.__ac_local_roles__ is None:
-                    user=getSecurityManager().getUser()
-                    if user is not None:
-                        userid=user.getId()
-                        if userid is not None:
-                            object.manage_setLocalRoles(userid, ['Owner'])
+            if getattr(ob, '__ac_local_roles__', _marker) is None:
+                user = getSecurityManager().getUser()
+                if user is not None:
+                    userid = user.getId()
+                    if userid is not None:
+                        ob.manage_setLocalRoles(userid, ['Owner'])
+
+        if not suppress_events:
+            notify(ObjectAddedEvent(ob, self, id))
+            notifyContainerModified(self)
+
+        OFS.subscribers.compatibilityCall('manage_afterAdd', ob, ob, self)
 
-        object.manage_afterAdd(object, self)
         return id
 
     def manage_afterAdd(self, item, container):
-        for object in self.objectValues():
-            try: s=object._p_changed
-            except: s=0
-            if hasattr(aq_base(object), 'manage_afterAdd'):
-                object.manage_afterAdd(item, container)
-            if s is None: object._p_deactivate()
+        # Don't do recursion anymore, a subscriber does that.
+        pass
+    manage_afterAdd.__five_method__ = True
 
     def manage_afterClone(self, item):
-        for object in self.objectValues():
-            try: s=object._p_changed
-            except: s=0
-            if hasattr(aq_base(object), 'manage_afterClone'):
-                object.manage_afterClone(item)
-            if s is None: object._p_deactivate()
+        # Don't do recursion anymore, a subscriber does that.
+        pass
+    manage_afterClone.__five_method__ = True
 
     def manage_beforeDelete(self, item, container):
-        for object in self.objectValues():
-            try: s=object._p_changed
-            except: s=0
-            try:
-                if hasattr(aq_base(object), 'manage_beforeDelete'):
-                    object.manage_beforeDelete(item, container)
-            except BeforeDeleteException, ob:
-                raise
-            except ConflictError:
-                raise
-            except:
-                LOG('Zope',ERROR,'manage_beforeDelete() threw',
-                    error=sys.exc_info())
-                # In debug mode when non-Manager, let exceptions propagate.
-                if getConfiguration().debug_mode:
-                    if not getSecurityManager().getUser().has_role('Manager'):
-                        raise
-            if s is None: object._p_deactivate()
+        # Don't do recursion anymore, a subscriber does that.
+        pass
+    manage_beforeDelete.__five_method__ = True
 
-    def _delObject(self, id, dp=1):
-        object=self._getOb(id)
-        try:
-            object.manage_beforeDelete(object, self)
-        except BeforeDeleteException, ob:
-            raise
-        except ConflictError:
-            raise
-        except:
-            LOG('Zope', ERROR, '_delObject() threw',
-                error=sys.exc_info())
-            # In debug mode when non-Manager, let exceptions propagate.
-            if getConfiguration().debug_mode:
-                if not getSecurityManager().getUser().has_role('Manager'):
-                    raise
-        self._objects=tuple(filter(lambda i,n=id: i['id']!=n, self._objects))
+    def _delObject(self, id, dp=1, suppress_events=False):
+        """Delete an object from this container.
+
+        Also sends IObjectWillBeRemovedEvent and IObjectRemovedEvent.
+        """
+        ob = self._getOb(id)
+
+        OFS.subscribers.compatibilityCall('manage_beforeDelete', ob, ob, self)
+
+        if not suppress_events:
+            notify(ObjectWillBeRemovedEvent(ob, self, id))
+
+        self._objects = tuple([i for i in self._objects
+                               if i['id'] != id])
         self._delOb(id)
 
         # Indicate to the object that it has been deleted. This is
@@ -352,8 +373,14 @@ class ObjectManager(
         # tolerate failure here because the object being deleted could
         # be a Broken object, and it is not possible to set attributes
         # on Broken objects.
-        try:    object._v__object_deleted__ = 1
-        except: pass
+        try:
+            ob._v__object_deleted__ = 1
+        except:
+            pass
+
+        if not suppress_events:
+            notify(ObjectRemovedEvent(ob, self, id))
+            notifyContainerModified(self)
 
     def objectIds(self, spec=None):
         # Returns a list of subobject ids of the current object.
@@ -500,7 +527,7 @@ class ObjectManager(
             obj_ids.sort()
             for id in obj_ids:
                 o=self._getOb(id)
-                if hasattr(o, 'isPrincipiaFolderish') and \
+                if hasattr(aq_base(o), 'isPrincipiaFolderish') and \
                    o.isPrincipiaFolderish:
                     r.append(o)
         return r
@@ -595,7 +622,9 @@ class ObjectManager(
             paths.append(cfg.instancehome)
         for impath in paths:
             directory = os.path.join(impath, 'import')
-            listing += [f for f in os.listdir(directory) 
+            if not os.path.isdir(directory):
+                continue
+            listing += [f for f in os.listdir(directory)
                         if f.endswith('.zexp') or f.endswith('.xml')]
         return listing
 
@@ -615,7 +644,7 @@ class ObjectManager(
                 break
             ob=ob.aq_parent
 
-        files=self.objectItems()
+        files = list(self.objectItems())
 
         # recursive ride through all subfolders (ls -R) (ajung)
 
@@ -623,15 +652,10 @@ class ObjectManager(
 
             all_files = copy.copy(files)
             for f in files:
-                if f[1].meta_type == "Folder":
+                if hasattr(aq_base(f[1]), 'isPrincipiaFolderish') and f[1].isPrincipiaFolderish:
                     all_files.extend(findChildren(f[1]))
-                else:
-                    all_files.append(f)
-
             files = all_files
 
-        files = list(files)
-
         # Perform globbing on list of files (ajung)
 
         globbing = REQUEST.environ.get('GLOBBING','')
@@ -652,8 +676,8 @@ class ObjectManager(
             try:
                 stat=marshal.loads(v.manage_FTPstat(REQUEST))
             except:
-                LOG("FTP", ERROR, "Failed to stat file '%s'" % k,
-                    error=sys.exc_info())
+                logger.error("Failed to stat file '%s'" % k,
+                             exc_info=True)
                 stat=None
             if stat is not None:
                 out=out+((k,stat),)
@@ -709,12 +733,12 @@ def findChildren(obj,dirname=''):
     find all children of an object (ajung)
     """
 
-    lst =[]
-    for name,child in obj.objectItems():
-        if child.meta_type=="Folder":
-            lst.extend(findChildren(child,dirname+ obj.id + '/'))
+    lst = []
+    for name, child in obj.objectItems():
+        if hasattr(aq_base(child), 'isPrincipiaFolderish') and child.isPrincipiaFolderish:
+            lst.extend(findChildren(child, dirname + obj.id + '/'))
         else:
-            lst.append( (dirname + obj.id + "/" + name,child) )
+            lst.append((dirname + obj.id + "/" + name, child))
 
     return lst
 
