@@ -1,13 +1,13 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """This module implements a simple item mix-in for objects that have a
@@ -17,30 +17,37 @@ Aqueduct database adapters, etc.
 This module can also be used as a simple template for implementing new
 item types.
 
-$Id: SimpleItem.py 30800 2005-06-15 00:48:38Z shane $
+$Id: SimpleItem.py 40478 2005-12-02 15:52:43Z chrisw $
 """
 
+import warnings
 import marshal, re, sys, time
 
-import Globals, App.Management, Acquisition, App.Undo
 import AccessControl.Role, AccessControl.Owned, App.Common
-from webdav.Resource import Resource
-from ExtensionClass import Base
-from ComputedAttribute import ComputedAttribute
+import Globals, App.Management, Acquisition, App.Undo
 from AccessControl import getSecurityManager, Unauthorized
 from AccessControl.ZopeSecurityPolicy import getRoles
 from Acquisition import aq_base, aq_parent, aq_inner, aq_acquire
+from ComputedAttribute import ComputedAttribute
+from DocumentTemplate.html_quote import html_quote
 from DocumentTemplate.ustr import ustr
-from zExceptions.ExceptionFormatter import format_exception
+from ExtensionClass import Base
+from webdav.Resource import Resource
 from zExceptions import Redirect
-from zLOG import LOG, BLATHER
+from zExceptions.ExceptionFormatter import format_exception
+from zope.interface import implements
 
-from CopySupport import CopySource
-from Traversable import Traversable
 import ZDOM
+from CopySupport import CopySource
+from interfaces import IItem
+from interfaces import IItemWithName
+from interfaces import ISimpleItem
+from Traversable import Traversable
 
 HTML=Globals.HTML
 
+import logging
+logger = logging.getLogger()
 
 class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
            ZDOM.Element,
@@ -50,17 +57,22 @@ class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
 
     """A common base class for simple, non-container objects."""
 
+    implements(IItem)
+
     isPrincipiaFolderish=0
     isTopLevelPrincipiaApplicationObject=0
 
     def manage_afterAdd(self, item, container):
         pass
+    manage_afterAdd.__five_method__ = True
 
     def manage_beforeDelete(self, item, container):
         pass
+    manage_beforeDelete.__five_method__ = True
 
     def manage_afterClone(self, item):
         pass
+    manage_afterClone.__five_method__ = True
 
     # Direct use of the 'id' attribute is deprecated - use getId()
     id=''
@@ -100,6 +112,8 @@ class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
     manage_options=(
         App.Undo.UndoSupport.manage_options
         +AccessControl.Owned.Owned.manage_options
+        +({'label': 'Interfaces',
+           'action': 'manage_interfaces'},)
         )
 
     # Attributes that must be acquired
@@ -210,17 +224,21 @@ class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
                 else:
                     v = HTML.__call__(s, client, REQUEST, **kwargs)
             except:
-                LOG('OFS', BLATHER,
+                logger.error(
                     'Exception while rendering an error message',
-                    error=sys.exc_info())
+                    exc_info=True
+                    )
                 try:
                     strv = str(error_value)
                 except:
-                    strv = ('<unprintable %s object>' % 
+                    strv = ('<unprintable %s object>' %
                             str(type(error_value).__name__))
                 v = strv + (
-                    " (Also, an error occurred while attempting "
-                    "to render the standard error message.)")
+                    (" (Also, the following error occurred while attempting "
+                     "to render the standard error message, please see the "
+                     "event log for full details: %s)")%(
+                    html_quote(sys.exc_info()[1]),
+                    ))
             raise error_type, v, tb
         finally:
             if hasattr(self, '_v_eek'): del self._v_eek
@@ -342,8 +360,11 @@ Globals.default__class_init__(Item)
 
 
 class Item_w__name__(Item):
+
     """Mixin class to support common name/id functions"""
 
+    implements(IItemWithName)
+
     def getId(self):
         """Return the id of the object as a string.
         """
@@ -397,6 +418,8 @@ class SimpleItem(Item, Globals.Persistent,
     """Mix-in class combining the most common set of basic mix-ins
     """
 
+    implements(ISimpleItem)
+
     manage_options=Item.manage_options+(
         {'label':'Security',
          'action':'manage_access',
