@@ -1,5 +1,10 @@
-import os, sys
 import unittest
+
+import Testing
+import Zope2
+Zope2.startup()
+
+import os, sys
 import time
 from cStringIO import StringIO
 
@@ -12,6 +17,7 @@ from ZPublisher.HTTPResponse import HTTPResponse
 from App.Common import rfc1123_date
 from Testing.makerequest import makerequest
 from zExceptions import Redirect
+import transaction
 
 try:
     here = os.path.dirname(os.path.abspath(__file__))
@@ -42,11 +48,11 @@ def aputrequest(file, content_type):
 class DummyCache:
     def __init__(self):
         self.clear()
-        
+
     def ZCache_set(self, ob, data, view_name='', keywords=None,
                    mtime_func=None):
         self.set = (ob, data)
-    
+
     def ZCache_get(self, ob, data, view_name='', keywords=None,
                    mtime_func=None):
         self.get = ob
@@ -64,14 +70,14 @@ class DummyCache:
 
     def setStreamIterator(self, si):
         self.si = si
-    
+
 
 ADummyCache=DummyCache()
 
 class DummyCacheManager(SimpleItem):
     def ZCacheManager_getCache(self):
         return ADummyCache
-    
+
 class FileTests(unittest.TestCase):
     data = open(filedata, 'rb').read()
     content_type = 'application/octet-stream'
@@ -95,16 +101,16 @@ class FileTests(unittest.TestCase):
             setattr(self.app, ZCM_MANAGERS, ('dcm',))
             # Hack, we need a _p_mtime for the file, so we make sure that it
             # has one.
-            get_transaction().commit()
+            transaction.commit()
         except:
             self.connection.close()
             raise
-        get_transaction().begin()
+        transaction.begin()
         self.file = getattr( self.app, 'file' )
 
     def tearDown( self ):
         del self.file
-        get_transaction().abort()
+        transaction.abort()
         self.connection.close()
         del self.app
         del self.responseOut
@@ -147,7 +153,7 @@ class FileTests(unittest.TestCase):
         self.assertEqual(self.file.content_type, 'text/plain')
         self.failUnless(ADummyCache.invalidated)
         self.failUnless(ADummyCache.set)
-        
+
     def testManageEditWithoutFileData(self):
         self.file.manage_edit('foobar', 'text/plain')
         self.assertEqual(self.file.title, 'foobar')
@@ -218,6 +224,35 @@ class FileTests(unittest.TestCase):
     def testStr(self):
         self.assertEqual(str(self.file), self.data)
 
+    def testFindSupport_not_text(self):
+        self.file.manage_edit('foobar', 'application/octet-stream',
+                              filedata=''.join([chr(x) for x in range(256)]))
+        self.assertEqual(self.file.PrincipiaSearchSource(), '')
+
+    def testFindSupport_text(self):
+        self.file.manage_edit('foobar', 'text/plain',
+                              filedata='Now is the time for all good men to '
+                                       'come to the aid of the Party.')
+        self.failUnless('Party' in self.file.PrincipiaSearchSource())
+
+    def testFindFile(self):
+        self.file.manage_edit('foobar', 'text/plain',
+                              filedata='Now is the time for all good men to '
+                                       'come to the aid of the Party.')
+        results = self.app.ZopeFind(self.app, obj_searchterm='Party')
+        self.assertEqual(len(results), 1)
+        self.assertEqual(results[0][1], self.file)
+
+    def test_z2interfaces(self):
+        from Interface.Verify import verifyClass
+        from OFS.Image import File
+        from webdav.WriteLockInterface import WriteLockInterface
+        from ZPublisher.HTTPRangeSupport import HTTPRangeInterface
+
+        verifyClass(HTTPRangeInterface, File)
+        verifyClass(WriteLockInterface, File)
+
+
 class ImageTests(FileTests):
     data = open(filedata, 'rb').read()
     content_type = 'image/gif'
@@ -231,24 +266,35 @@ class ImageTests(FileTests):
         self.assertEqual(self.file.height, 16)
         self.failUnless(ADummyCache.invalidated)
         self.failUnless(ADummyCache.set)
-        
+
     def testStr(self):
         self.assertEqual(str(self.file),
           ('<img src="http://foo/file" alt="" title="" height="16" width="16" />'))
-           
 
     def testTag(self):
-        self.assertEqual(self.file.tag(),
-          ('<img src="http://foo/file" alt="" title="" height="16" width="16" />'))
+        tag_fmt = '<img src="http://foo/file" alt="%s" title="%s" height="16" width="16" />'
+        self.assertEqual(self.file.tag(), (tag_fmt % ('','')))
+        self.file.manage_changeProperties(title='foo')
+        self.assertEqual(self.file.tag(), (tag_fmt % ('','foo')))
+        self.file.manage_changeProperties(alt='bar')
+        self.assertEqual(self.file.tag(), (tag_fmt % ('bar','foo')))
 
     def testViewImageOrFile(self):
         pass # dtml method,screw it
-    
-def test_suite():
-    suite = unittest.TestSuite()
-    suite.addTest( unittest.makeSuite( FileTests ) )
-    suite.addTest( unittest.makeSuite( ImageTests ))
-    return suite
 
-if __name__ == "__main__":
-    unittest.main()
+    def test_z2interfaces(self):
+        from Interface.Verify import verifyClass
+        from OFS.Image import Image
+        from webdav.WriteLockInterface import WriteLockInterface
+
+        verifyClass(WriteLockInterface, Image)
+
+
+def test_suite():
+    return unittest.TestSuite((
+        unittest.makeSuite(FileTests),
+        unittest.makeSuite(ImageTests),
+        ))
+
+if __name__ == '__main__':
+    unittest.main(defaultTest='test_suite')
