@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,7 +12,7 @@
 ##############################################################################
 """Keyword index.
 
-$Id: KeywordIndex.py 30994 2005-07-04 16:59:17Z yuppie $
+$Id: KeywordIndex.py 68138 2006-05-15 20:21:03Z chrisw $
 """
 
 from types import StringType, UnicodeType
@@ -127,11 +127,11 @@ class KeywordIndex(UnIndex):
         try:
             del self._unindex[documentId]
         except KeyError:
-            LOG.error('Attempt to unindex nonexistent'
+            LOG.debug('Attempt to unindex nonexistent'
                       ' document id %s' % documentId)
 
-    index_html = DTMLFile('dtml/index', globals())
-    manage_workspace = DTMLFile('dtml/manageKeywordIndex', globals())
+    manage = manage_main = DTMLFile('dtml/manageKeywordIndex', globals())
+    manage_main._setName('manage_main')
     manage_browse = DTMLFile('../dtml/browseIndex', globals())
 
 
