@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,16 +12,17 @@
 ##############################################################################
 """Deprecated text index. Please use ZCTextIndex instead.
 
-$Id: TextIndex.py 31005 2005-07-05 13:38:18Z yuppie $
+$Id: TextIndex.py 68835 2006-06-24 22:45:20Z tseaver $
 """
 
 import operator, warnings
 import re
 from cgi import escape
+import logging
+logger = logging.getLogger('TextIndex')
 from types import *
 
 from Globals import Persistent, DTMLFile
-from zLOG import LOG, ERROR
 from Acquisition import Implicit
 from OFS.SimpleItem import SimpleItem
 from BTrees.IOBTree import IOBTree
@@ -376,9 +377,8 @@ class TextIndex(Persistent, Implicit, SimpleItem):
         for wid in wids:
             widScores = get(wid, None)
             if widScores is None:
-                LOG('TextIndex', ERROR,
-                    'unindex_object tried to unindex nonexistent'
-                    ' document, wid  %s, %s' % (i,wid))
+                logger.error('unindex_object tried to unindex nonexistent'
+                             ' document, wid  %s, %s' % (i,wid))
                 continue
             if type(widScores) is TupleType:
                 del index[wid]
@@ -394,9 +394,8 @@ class TextIndex(Persistent, Implicit, SimpleItem):
                     else:
                         del index[wid]
                 except (KeyError, IndexError, TypeError):
-                    LOG('TextIndex', ERROR,
-                        'unindex_object tried to unindex nonexistent'
-                        ' document %s' % str(i))
+                    logger.error('unindex_object tried to unindex nonexistent'
+                                 ' document %s' % str(i))
 
     def __getitem__(self, word):
         """Return an InvertedIndex-style result "list"
@@ -652,7 +651,8 @@ class TextIndex(Persistent, Implicit, SimpleItem):
         if RESPONSE:
             RESPONSE.redirect(URL2 + '/manage_main?manage_tabs_message=Preferences%20saved')
 
-    manage_workspace  = DTMLFile("dtml/manageTextIndex",globals())
+    manage = manage_main = DTMLFile("dtml/manageTextIndex",globals())
+    manage_main._setName('manage_main')
     manage_vocabulary = DTMLFile("dtml/manageVocabulary",globals())
 
 
