@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -20,6 +20,7 @@ Python code.
 __version__='$Revision: 1.56 $'[11:-2]
 
 import sys, os, traceback, re, marshal, new
+import logging
 from Globals import DTMLFile, MessageDialog, package_home
 import AccessControl, OFS, RestrictedPython
 from Acquisition import aq_parent
@@ -34,10 +35,11 @@ from OFS.History import Historical, html_diff
 from OFS.Cache import Cacheable
 from AccessControl.ZopeGuards import get_safe_globals, guarded_getattr
 from AccessControl.requestmethod import postonly
-from zLOG import LOG, ERROR, INFO, PROBLEM
 from zExceptions import Forbidden
 import Globals
 
+LOG = logging.getLogger('Script (Python)')
+
 # Track the Python bytecode version
 import imp
 Python_magic = imp.get_magic()
@@ -210,7 +212,7 @@ class PythonScript(Script, Historical, Cacheable):
             getattr(self, 'Script_magic', None) != Script_magic):
             global _log_complaint
             if _log_complaint:
-                LOG(self.meta_type, INFO, _log_complaint)
+                LOG.info(_log_complaint)
                 _log_complaint = 0
             # Changes here won't get saved, unless this Script is edited.
             body = self._body.rstrip()
@@ -331,6 +333,14 @@ class PythonScript(Script, Historical, Cacheable):
         if item is self:
             self._filepath = self.get_filepath()
 
+    def manage_beforeDelete(self, item, container):
+        # shut up deprecation warnings
+        pass
+
+    def manage_afterClone(self, item):
+        # shut up deprecation warnings
+        pass
+
     def get_filepath(self):
         return self.meta_type + ':' + '/'.join(self.getPhysicalPath())
 
@@ -350,6 +360,7 @@ class PythonScript(Script, Historical, Cacheable):
       'manage_proxyForm', 'manage_proxy')
 
     manage_proxyForm = DTMLFile('www/pyScriptProxy', globals())
+    @postonly
     def manage_proxy(self, roles=(), REQUEST=None):
         "Change Proxy Roles"
         self._validateProxy(roles)
@@ -360,7 +371,6 @@ class PythonScript(Script, Historical, Cacheable):
                     title  ='Success!',
                     message='Your changes have been saved',
                     action ='manage_main')
-    manage_proxy = postonly(manage_proxy)
 
     security.declareProtected('Change Python Scripts',
       'PUT', 'manage_FTPput', 'write',
@@ -408,7 +418,7 @@ class PythonScript(Script, Historical, Cacheable):
                 k = k.strip().lower()
                 v = v.strip()
                 if not mdata.has_key(k):
-                    SyntaxError, 'Unrecognized header line "%s"' % line
+                    raise SyntaxError, 'Unrecognized header line "%s"' % line
                 if v == mdata[k]:
                     # Unchanged value
                     continue
@@ -432,7 +442,7 @@ class PythonScript(Script, Historical, Cacheable):
             else:
                 self._makeFunction()
         except:
-            LOG(self.meta_type, ERROR, 'write failed', error=sys.exc_info())
+            LOG.error('write failed', exc_info=True)
             raise
 
     def manage_FTPget(self):
