@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -128,6 +128,9 @@ class TestPythonScriptNoAq(PythonScriptTestBase):
         res = self._newPS('return 1 * 5 + 4 / 2 - 6')()
         self.assertEqual(res, 1)
 
+    def testCollector2295(self):
+        res = self._newPS('if False:\n  pass\n#hi')
+
     def testReduce(self):
         res = self._newPS('return reduce(lambda x, y: x + y, [1,3,5,7])')()
         self.assertEqual(res, 16)
@@ -197,6 +200,9 @@ class TestPythonScriptNoAq(PythonScriptTestBase):
         bound = f.__render_with_namespace__({'yes': 1, 'no': self.fail})
         self.assertEqual(bound, 1)
 
+    def testNSBindInvalidHeader(self):
+        self.assertRaises(SyntaxError, self._filePS, 'ns_bind_invalid')
+
     def testBooleanMap(self):
         res = self._filePS('boolean_map')()
         self.failUnless(res)
