@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -11,24 +11,26 @@
 #
 ##############################################################################
 
-from Persistence import Persistent
+import types
+import logging
+from bisect import bisect
+from random import randint
+
 import Acquisition
 import ExtensionClass
-from MultiMapping import MultiMapping
-import Record
 from Missing import MV
-from zLOG import LOG, ERROR
+from Persistence import Persistent
 
-from Lazy import LazyMap, LazyFilter, LazyCat, LazyValues
-from CatalogBrains import AbstractCatalogBrain, NoBrainer
+import BTrees.Length
 from BTrees.IIBTree import intersection, weightedIntersection, IISet
 from BTrees.OIBTree import OIBTree
 from BTrees.IOBTree import IOBTree
-import BTrees.Length
+from Lazy import LazyMap, LazyCat, LazyValues
+from CatalogBrains import AbstractCatalogBrain, NoBrainer
+
+
+LOG = logging.getLogger('Zope.ZCatalog')
 
-import time, sys, types
-from bisect import bisect
-from random import randint
 
 try:
     from DocumentTemplate.cDocumentTemplate import safe_callable
@@ -76,7 +78,6 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
         # object unique identifier to the rid, and self.paths is a
         # mapping of the rid to the unique identifier.
 
-        self.__len__=BTrees.Length.Length()
         self.clear()
 
         if brains is not None:
@@ -84,49 +85,28 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
 
         self.updateBrains()
 
+    
+    def __len__(self):
+        return self._length()
+
+    def migrate__len__(self):
+        """ migration of old __len__ magic for Zope 2.8 """
+        if not hasattr(self, '_length'):
+            n = self.__dict__['__len__']()
+            del self.__dict__['__len__'] 
+            self._length = BTrees.Length.Length(n)
+
     def clear(self):
         """ clear catalog """
 
         self.data  = IOBTree()  # mapping of rid to meta_data
         self.uids  = OIBTree()  # mapping of uid to rid
         self.paths = IOBTree()  # mapping of rid to uid
-
-        # convert old-style Catalog object to new in-place
-        try: self.__len__.set(0)
-        except AttributeError: self.__len__=BTrees.Length.Length()
+        self._length = BTrees.Length.Length()
 
         for index in self.indexes.keys():
             self.getIndex(index).clear()
 
-    def _convertBTrees(self, threshold=200):
-
-        from BTrees.convert import convert
-
-        if type(self.data) is not IOBTree:
-            data=self.data
-            self.data=IOBTree()
-            convert(data, self.data, threshold)
-
-            self.__len__=BTrees.Length.Length(len(data))
-
-            uids=self.uids
-            self.uids=OIBTree()
-            convert(uids, self.uids, threshold)
-
-            paths=self.paths
-            self.paths=IOBTree()
-            convert(paths, self.paths, threshold)
-
-
-        for index in self.indexes.values():
-            if hasattr(index, '__of__'): index=index.__of__(self)
-            index._convertBTrees(threshold)
-
-    def __len__(self):
-        # NOTE, this is never called for new catalogs, since
-        # each instance overrides this.
-        return len(self.data)
-
     def updateBrains(self):
         self.useBrains(self._v_brains)
 
@@ -222,11 +202,10 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
         _index = names.index(name)
 
         if not self.schema.has_key(name):
-            LOG('Catalog', ERROR, ('delColumn attempted to delete '
-                                   'nonexistent column %s.' % str(name)))
+            LOG.error('delColumn attempted to delete nonexistent column %s.' % str(name))
             return
 
-        names.remove(name)
+        del names[_index]
 
         # rebuild the schema
         i=0; schema = {}
@@ -243,7 +222,7 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
         # remove the column value from each record
         for key in self.data.keys():
             rec = list(self.data[key])
-            rec.remove(rec[_index])
+            del rec[_index]
             self.data[key] = tuple(rec)
 
     def addIndex(self, name, index_type):
@@ -268,7 +247,7 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
 
         indexes = self.indexes
 
-        if isinstance(index_type, types.StringType):
+        if isinstance(index_type, str):
             raise TypeError,"""Catalog addIndex now requires the index type to
             be resolved prior to adding; create the proper index in the caller."""
 
@@ -359,9 +338,9 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
         if index is None:  # we are inserting new data
             index = self.updateMetadata(object, uid)
 
-            try: self.__len__.change(1)
-            except AttributeError: pass # No managed length (old-style)
-
+            if not hasattr(self, '_length'):
+                self.migrate__len__()
+            self._length.change(1)
             self.uids[uid] = index
             self.paths[index] = uid
 
@@ -381,8 +360,7 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
                 blah = x.index_object(index, object, threshold)
                 total = total + blah
             else:
-                LOG('Catalog', ERROR, ('catalogObject was passed '
-                                       'bad index object %s.' % str(x)))
+                LOG.error('catalogObject was passed bad index object %s.' % str(x))
 
         return total
 
@@ -412,12 +390,14 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
             del data[rid]
             del paths[rid]
             del uids[uid]
-            try: self.__len__.change(-1)
-            except AttributeError: pass # No managed length
+            if not hasattr(self, '_length'):
+                self.migrate__len__()
+            self._length.change(-1)
+            
         else:
-            LOG('Catalog', ERROR, ('uncatalogObject unsuccessfully '
-                                   'attempted to uncatalog an object '
-                                   'with a uid of %s. ' % str(uid)))
+            LOG.error('uncatalogObject unsuccessfully '
+                      'attempted to uncatalog an object '
+                      'with a uid of %s. ' % str(uid))
 
 
     def uniqueValuesFor(self, name):
@@ -511,6 +491,8 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
         elif rs:
             # We got some results from the indexes.
             # Sort and convert to sequences.
+            # XXX: The check for 'values' is really stupid since we call
+            # items() and *not* values()
             if sort_index is None and hasattr(rs, 'values'):
                 # having a 'values' means we have a data structure with
                 # scores.  Build a new result set, sort it by score, reverse
@@ -746,7 +728,7 @@ class Catalog(Persistent, Acquisition.Implicit, ExtensionClass.Base):
         reverse = 0
         if sort_index is not None:
             order = self._get_sort_attr("order", args)
-            if (isinstance(order, types.StringType) and
+            if (isinstance(order, str) and
                 order.lower() in ('reverse', 'descending')):
                 reverse = 1
         # Perform searches with indexes and sort_index
