@@ -1,19 +1,19 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-__doc__='''Generic Database Adapter Package Registration
+"""Generic Database Adapter Package Registration.
 
-$Id: __init__.py 24763 2004-05-17 05:59:28Z philikon $'''
-__version__='$Revision: 1.16 $'[11:-2]
+$Id: __init__.py 40222 2005-11-18 15:46:28Z andreasjung $
+"""
 
 import Globals, os
 
@@ -34,12 +34,6 @@ for icon in ('table', 'view', 'stable', 'what',
              'date','time','datetime'):
     misc_[icon]=Globals.ImageFile('icons/%s.gif' % icon, globals())
 
-meta_types=(
-    {'name':'Z %s Database Connection' % database_type,
-     'action':'manage_addZ%sConnectionForm' % database_type,
-     },
-    )
-
 DA=None
 def getDA():
     global DA
@@ -70,20 +64,17 @@ def manage_addZGadflyConnection(
     return getDA().manage_addZGadflyConnection(
         self, id, title, connection, check, REQUEST)
 
-methods={
-    'manage_addZGadflyConnection':
-    manage_addZGadflyConnection,
-    'manage_addZGadflyConnectionForm':
-    manage_addZGadflyConnectionForm,
-    }
+def initialize(context):
 
-__ac_permissions__=(
-    ('Add Z Gadfly Database Connections',
-     ('manage_addZGadflyConnectionForm',
-      'manage_addZGadflyConnection')),
+    context.registerClass(
+        DA.Connection,
+        permission='Add Z Gadfly Database Connections',
+        constructors=(manage_addZGadflyConnectionForm,
+                      manage_addZGadflyConnection),
+        legacy=(manage_addZGadflyConnectionForm,
+                manage_addZGadflyConnection),
     )
 
-
 # from App.config import getConfiguration
 # j=os.path.join
 # d=j(getConfiguration().clienthome,'gadfly')
