@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -414,14 +414,14 @@ class ToXMLUnpickler(Unpickler):
     def load_binput(self):
         i = mloads('i' + self.read(1) + '\000\000\000')
         last = self.stack[-1]
-        if getattr(last, 'id', last) is not last:
+        if getattr(last, 'id', last) is last:
             last.id = self.idprefix + `i`
     dispatch[BINPUT] = load_binput
 
     def load_long_binput(self):
         i = mloads('i' + self.read(4))
         last = self.stack[-1]
-        if getattr(last, 'id', last) is not last:
+        if getattr(last, 'id', last) is last:
             last.id = self.idprefix + `i`
     dispatch[LONG_BINPUT] = load_long_binput
 
@@ -643,10 +643,10 @@ class xmlPickler(NoBlanks, xyap):
         'pickle': lambda self, tag, attrs: [tag, attrs],
         }
     end_handlers={
-        'pickle': lambda self, tag, data: data[2]+'.',
+        'pickle': lambda self, tag, data: str(data[2])+'.',
         'none': lambda self, tag, data: 'N',
         'int': save_int,
-        'long': lambda self, tag, data: 'L'+data[2]+'L\012',
+        'long': lambda self, tag, data: 'L'+str(data[2])+'L\012',
         'float': save_float,
         'string': save_string,
         'reference': save_reference,
@@ -750,7 +750,7 @@ def test1():
     print z, '\012'
 
 def test2():
-    import xml.parsers.pyexpat
+    import xml.parsers.expat
     c=C()
     c.foo=1
     c.bar=2
@@ -778,7 +778,7 @@ def test2():
     file=''
     F=xmlPickler()
     F.binary=0
-    p=xml.parsers.pyexpat.ParserCreate()
+    p=xml.parsers.expat.ParserCreate()
     p.CharacterDataHandler=F.handle_data
     p.StartElementHandler=F.unknown_starttag
     p.EndElementHandler=F.unknown_endtag
@@ -786,13 +786,13 @@ def test2():
     print r, '\012'
 
 def test3():
-    import xml.parsers.pyexpat
+    import xml.parsers.expat
     data=open('Data.xml').read()
     file=open('out','w'+'b')
     F=xmlPickler()
     F.file=file
     F.binary=1
-    p=xml.parsers.pyexpat.ParserCreate()
+    p=xml.parsers.expat.ParserCreate()
     p.CharacterDataHandler=F.handle_data
     p.StartElementHandler=F.unknown_starttag
     p.EndElementHandler=F.unknown_endtag
