@@ -12,7 +12,7 @@
 ##############################################################################
 """Example Zope doctest
 
-$Id: testZopeDocTest.py 30245 2005-05-05 09:50:09Z shh $
+$Id: testZopeDocTest.py 76991 2007-06-23 17:26:25Z shh $
 """
 
 import os, sys
@@ -29,8 +29,12 @@ def setUp(self):
 
     >>> 'object' in folder.objectIds()
     True
+
+    >>> foo
+    1
     '''
     self.folder.manage_addFolder('object', '')
+    self.globs['foo'] = 1
 
 
 def test_suite():
