@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -20,7 +20,7 @@ The Demo storage serves two purposes:
 
   - Provide a volatile storage that is useful for giving demonstrations.
 
-The demo strorage can have a "base" storage that is used in a
+The demo storage can have a "base" storage that is used in a
 read-only fashion. The base storage must not not to contain version
 data.
 
@@ -45,14 +45,12 @@ There are three main data structures:
 
 A record is a tuple:
 
-  oid, serial, pre, vdata, p,
+  oid, pre, vdata, p, tid
 
 where:
 
      oid -- object id
 
-     serial -- object serial number
-
      pre -- The previous record for this object (or None)
 
      vdata -- version data
@@ -62,6 +60,8 @@ where:
 
      p -- the pickle data or None
 
+     tid -- the transaction id that wrote the record
+
 The pickle data will be None for a record for an object created in
 an aborted version.
 
@@ -69,42 +69,60 @@ It is instructive to watch what happens to the internal data structures
 as changes are made.  For example, in Zope, you can create an external
 method::
 
-  import Zope
+  import Zope2
 
   def info(RESPONSE):
       RESPONSE['Content-type']= 'text/plain'
 
-      return Zope.DB._storage._splat()
+      return Zope2.DB._storage._splat()
 
 and call it to monitor the storage.
 
 """
-__version__='$Revision: 1.14 $'[11:-2]
 
-import base64, time, string
-from ZODB import POSException, BaseStorage, utils
-from TimeStamp import TimeStamp
+import base64, time
+from ZODB import POSException
+from ZODB.utils import z64, oid_repr
+from ZODB.BaseStorage import BaseStorage
+from persistent.TimeStamp import TimeStamp
 from cPickle import loads
 from BTrees import OOBTree
 
-class DemoStorage(BaseStorage.BaseStorage):
+class DemoStorage(BaseStorage):
 
     def __init__(self, name='Demo Storage', base=None, quota=None):
-
-        BaseStorage.BaseStorage.__init__(self, name, base)
+        BaseStorage.__init__(self, name, base)
 
         # We use a BTree because the items are sorted!
-        self._data=OOBTree.OOBTree()
-        self._index={}
-        self._vindex={}
-        self._base=base
-        self._size=0
-        self._quota=quota
+        self._data = OOBTree.OOBTree()
+        self._index = {}
+        self._vindex = {}
+        self._base = base
+        self._size = 0
+        self._quota = quota
+        self._ltid = None
         self._clear_temp()
         if base is not None and base.versions():
-            raise POSException.StorageError, (
+            raise POSException.StorageError(
                 "Demo base storage has version data")
 
+    # While we officially don't support wrapping a non-read-only base
+    # storage, it has proved useful for test suites to wrap a ClientStorage
+    # in DemoStorage.  The least we can do to help support that case is
+    # to arrange for invalidations to get delivered to the base storage.
+    def registerDB(self, db, limit):
+        if self._base is not None: # delegate
+            self._base.registerDB(db, limit)
+
+    # When DemoStorage needs to create a new oid, and there is a base
+    # storage, it must use that storage's new_oid() method.  Else
+    # DemoStorage may end up assigning "new" oids that are already in use
+    # by the base storage, leading to a variety of "impossible" problems.
+    def new_oid(self):
+        if self._base is None:
+            return BaseStorage.new_oid(self)
+        else:
+            return self._base.new_oid()
 
     def __len__(self):
         base=self._base
@@ -114,7 +132,7 @@ class DemoStorage(BaseStorage.BaseStorage):
         s=100
         for tid, (p, u, d, e, t) in self._data.items():
             s=s+16+24+12+4+16+len(u)+16+len(d)+16+len(e)+16
-            for oid, serial, pre, vdata, p in t:
+            for oid, pre, vdata, p, tid in t:
                 s=s+16+24+24+4+4+(p and (16+len(p)) or 4)
                 if vdata: s=s+12+16+len(vdata[0])+4
 
@@ -140,16 +158,16 @@ class DemoStorage(BaseStorage.BaseStorage):
 
             oids = []
             for r in v.values():
-                oid, serial, pre, (version, nv), p = r
+                oid, pre, (version, nv), p, tid = r
+                oids.append(oid)
                 if nv:
-                    oids.append(oid)
-                    oid, serial, pre, vdata, p = nv
-                    self._tindex.append([oid, serial, r, None, p])
+                    oid, pre, vdata, p, tid = nv
+                    self._tindex.append([oid, r, None, p, self._tid])
                 else:
                     # effectively, delete the thing
-                    self._tindex.append([oid, None, r, None, None])
+                    self._tindex.append([oid, r, None, None, self._tid])
 
-            return oids
+            return self._tid, oids
 
         finally: self._lock_release()
 
@@ -165,56 +183,64 @@ class DemoStorage(BaseStorage.BaseStorage):
 
         self._lock_acquire()
         try:
-            v=self._vindex.get(src, None)
-            if v is None: return
+            v = self._vindex.get(src)
+            if v is None:
+                return
 
-            newserial = self._serial
-            tindex=self._tindex
-            oids=[]
+            newserial = self._tid
+            tindex = self._tindex
+            oids = []
             for r in v.values():
-                oid, serial, pre, vdata, p = r
+                oid, pre, vdata, p, tid = r
                 assert vdata is not None
                 oids.append(oid)
                 if dest:
                     new_vdata = dest, vdata[1]
                 else:
                     new_vdata = None
-                tindex.append([oid, newserial, r, new_vdata, p])
+                tindex.append([oid, r, new_vdata, p, self._tid])
 
+            return self._tid, oids
 
-            return oids
+        finally:
+            self._lock_release()
 
-        finally: self._lock_release()
-
-    def load(self, oid, version):
+    def loadEx(self, oid, version):
         self._lock_acquire()
         try:
             try:
-                oid, serial, pre, vdata, p = self._index[oid]
+                oid, pre, vdata, p, tid = self._index[oid]
             except KeyError:
                 if self._base:
                     return self._base.load(oid, '')
-                raise KeyError, oid
+                raise KeyError(oid)
 
+            ver = ""
             if vdata:
                 oversion, nv = vdata
                 if oversion != version:
                     if nv:
-                        oid, serial, pre, vdata, p = nv
+                        # Return the current txn's tid with the non-version
+                        # data.
+                        oid, pre, vdata, p, skiptid = nv
                     else:
-                        raise KeyError, oid
+                        raise KeyError(oid)
+                ver = oversion
 
             if p is None:
-                raise KeyError, oid
+                raise KeyError(oid)
 
-            return p, serial
+            return p, tid, ver
         finally: self._lock_release()
 
+    def load(self, oid, version):
+        return self.loadEx(oid, version)[:2]
+
     def modifiedInVersion(self, oid):
         self._lock_acquire()
         try:
             try:
-                oid, serial, pre, vdata, p = self._index[oid]
+                oid, pre, vdata, p, tid = self._index[oid]
                 if vdata: return vdata[0]
                 return ''
             except: return ''
@@ -226,31 +252,34 @@ class DemoStorage(BaseStorage.BaseStorage):
 
         self._lock_acquire()
         try:
-            old=self._index.get(oid, None)
+            old = self._index.get(oid, None)
             if old is None:
                 # Hm, nothing here, check the base version:
-                try: p, oserial = self._base.load(oid, '')
-                except: pass
-                else:
-                    old= oid, oserial, None, None, p
+                if self._base:
+                    try:
+                        p, tid = self._base.load(oid, '')
+                    except KeyError:
+                        pass
+                    else:
+                        old = oid, None, None, p, tid
 
             nv=None
             if old:
-                oid, oserial, pre, vdata, p = old
+                oid, pre, vdata, p, tid = old
 
                 if vdata:
                     if vdata[0] != version:
-                        raise POSException.VersionLockError, oid
+                        raise POSException.VersionLockError(oid)
 
                     nv=vdata[1]
                 else:
                     nv=old
 
-                if serial != oserial:
-                    raise POSException.ConflictError(serials=(oserial, serial))
+                if serial != tid:
+                    raise POSException.ConflictError(
+                        oid=oid, serials=(tid, serial), data=data)
 
-            serial=self._serial
-            r=[oid, serial, old, version and (version, nv) or None, data]
+            r = [oid, old, version and (version, nv) or None, data, self._tid]
             self._tindex.append(r)
 
             s=self._tsize
@@ -258,21 +287,24 @@ class DemoStorage(BaseStorage.BaseStorage):
             if version: s=s+32+len(version)
 
             if self._quota is not None and s > self._quota:
-                raise POSException.StorageError, (
+                raise POSException.StorageError(
                     '''<b>Quota Exceeded</b><br>
                     The maximum quota for this demonstration storage
                     has been exceeded.<br>Have a nice day.''')
 
         finally: self._lock_release()
-        return serial
+        return self._tid
 
-    def supportsUndo(self): return 1
-    def supportsVersions(self): return 1
+    def supportsVersions(self):
+        return 1
 
     def _clear_temp(self):
         self._tindex = []
         self._tsize = self._size + 160
 
+    def lastTransaction(self):
+        return self._ltid
+
     def _begin(self, tid, u, d, e):
         self._tsize = self._size + 120 + len(u) + len(d) + len(e)
 
@@ -281,11 +313,11 @@ class DemoStorage(BaseStorage.BaseStorage):
 
         self._data[tid] = None, user, desc, ext, tuple(self._tindex)
         for r in self._tindex:
-            oid, serial, pre, vdata, p = r
+            oid, pre, vdata, p, tid = r
             old = self._index.get(oid)
             # If the object had version data, remove the version data.
             if old is not None:
-                oldvdata = old[3]
+                oldvdata = old[2]
                 if oldvdata:
                     v = self._vindex[oldvdata[0]]
                     del v[oid]
@@ -302,61 +334,11 @@ class DemoStorage(BaseStorage.BaseStorage):
                 if v is None:
                     v = self._vindex[version] = {}
                 v[oid] = r
-
-    def undo(self, transaction_id):
-        self._lock_acquire()
-        try:
-            transaction_id=base64.decodestring(transaction_id+'\n')
-            try: t=self._data[transaction_id][4]
-            except KeyError:
-                raise UndoError, 'Invalid undo transaction id'
-
-            index=self._index
-            vindex=self._vindex
-            vindex_get=self._vindex.get
-            for r in t:
-                if index[r[0]] is not r:
-                    raise POSException.UndoError, 'non-undoable transaction'
-
-            oids=[]
-            for r in t:
-                oid, serial, pre, vdata, p = r
-                if pre:
-
-                    index[oid] = pre
-                    oids.append(oid)
-
-                    # Delete old version data
-                    if vdata:
-                        version=vdata[0]
-                        v=vindex.get(version, None)
-                        if v: del v[oid]
-
-                    # Add new version data (from pre):
-                    oid, serial, prepre, vdata, p = pre
-                    if vdata:
-                        version=vdata[0]
-                        v=vindex.get(version, None)
-                        if v is None: v=vindex[version]={}
-                        v[oid]=pre
-
-                else:
-                    del index[oid]
-                    if vdata:
-                        version=vdata[0]
-                        v=vindex.get(version, None)
-                        if v: del v[oid]
-                        if not v: del vindex[version]
-
-            del self._data[transaction_id]
-
-            return oids
-
-        finally: self._lock_release()
+        self._ltid = self._tid
 
     def undoLog(self, first, last, filter=None):
-        # I think this is wrong given the handling of first and last
-        # in FileStorage.
+        if last < 0:  # abs(last) is an upper bound on the # to return
+            last = first - last
         self._lock_acquire()
         try:
             transactions = self._data.items()
@@ -364,11 +346,10 @@ class DemoStorage(BaseStorage.BaseStorage):
             r = []
             i = 0
             while i < last and pos:
-                pos = pos - 1
-                if i < first:
-                    i = i + 1
-                    continue
+                pos -= 1
                 tid, (p, u, d, e, t) = transactions[pos]
+                # Bug alert:  why do we skip this if the transaction
+                # has been packed?
                 if p:
                     continue
                 d = {'id': base64.encodestring(tid)[:-1],
@@ -376,10 +357,10 @@ class DemoStorage(BaseStorage.BaseStorage):
                      'user_name': u, 'description': d}
                 if e:
                     d.update(loads(e))
-
                 if filter is None or filter(d):
-                    r.append(d)
-                    i = i + 1
+                    if i >= first:
+                        r.append(d)
+                    i += 1
             return r
         finally:
             self._lock_release()
@@ -399,17 +380,17 @@ class DemoStorage(BaseStorage.BaseStorage):
 
     def _build_indexes(self, stop='\377\377\377\377\377\377\377\377'):
         # Rebuild index structures from transaction data
-        index={}
-        vindex={}
-        _data=self._data
-        for tid, (p, u, d, e, t) in _data.items():
-            if tid >= stop: break
+        index = {}
+        vindex = {}
+        for tid, (p, u, d, e, t) in self._data.items():
+            if tid >= stop:
+                break
             for r in t:
-                oid, serial, pre, vdata, p = r
+                oid, pre, vdata, p, tid = r
                 old=index.get(oid, None)
 
                 if old is not None:
-                    oldvdata=old[3]
+                    oldvdata=old[2]
                     if oldvdata:
                         v=vindex[oldvdata[0]]
                         del v[oid]
@@ -433,55 +414,62 @@ class DemoStorage(BaseStorage.BaseStorage):
         self._lock_acquire()
         try:
 
-            stop=`apply(TimeStamp, time.gmtime(t)[:5]+(t%60,))`
-            _data=self._data
+            stop=`TimeStamp(*time.gmtime(t)[:5]+(t%60,))`
 
             # Build indexes up to the pack time:
             index, vindex = self._build_indexes(stop)
 
+
+            # TODO:  This packing algorithm is flawed. It ignores
+            # references from non-current records after the pack
+            # time.
+
             # Now build an index of *only* those objects reachable
             # from the root.
-            rootl=['\0\0\0\0\0\0\0\0']
-            pop=rootl.pop
-            pindex={}
-            referenced=pindex.has_key
+            rootl = [z64]
+            pindex = {}
             while rootl:
-                oid=pop()
-                if referenced(oid): continue
+                oid = rootl.pop()
+                if oid in pindex:
+                    continue
 
                 # Scan non-version pickle for references
-                r=index.get(oid, None)
+                r = index.get(oid, None)
                 if r is None:
-                    # Base storage
-                    p, s = self._base.load(oid, '')
-                    referencesf(p, rootl)
+                    if self._base:
+                        p, s = self._base.load(oid, '')
+                        referencesf(p, rootl)
                 else:
-                    pindex[oid]=r
-                    oid, serial, pre, vdata, p = r
+                    pindex[oid] = r
+                    oid, pre, vdata, p, tid = r
                     referencesf(p, rootl)
                     if vdata:
-                        nv=vdata[1]
+                        nv = vdata[1]
                         if nv:
-                            oid, serial, pre, vdata, p = nv
+                            oid, pre, vdata, p, tid = nv
                             referencesf(p, rootl)
 
             # Now we're ready to do the actual packing.
             # We'll simply edit the transaction data in place.
             # We'll defer deleting transactions till the end
             # to avoid messing up the BTree items.
-            deleted=[]
-            for tid, (p, u, d, e, t) in _data.items():
-                if tid >= stop: break
-                o=[]
-                for r in t:
-                    c=pindex.get(r[0])
+            deleted = []
+            for tid, (p, u, d, e, records) in self._data.items():
+                if tid >= stop:
+                    break
+                o = []
+                for r in records:
+                    c = pindex.get(r[0])
                     if c is None:
                         # GC this record, no longer referenced
                         continue
-                    elif c is not r:
+                    if c == r:
+                        # This is the most recent revision.
+                        o.append(r)
+                    else:
                         # This record is not the indexed record,
                         # so it may not be current. Let's see.
-                        oid, serial, pre, vdata, p = r
+                        vdata = r[3]
                         if vdata:
                             # Version record are current *only* if they
                             # are indexed
@@ -489,7 +477,7 @@ class DemoStorage(BaseStorage.BaseStorage):
                         else:
                             # OK, this isn't a version record, so it may be the
                             # non-version record for the indexed record.
-                            oid, serial, pre, vdata, p = c
+                            vdata = c[3]
                             if vdata:
                                 if vdata[1] != r:
                                     # This record is not the non-version
@@ -500,29 +488,31 @@ class DemoStorage(BaseStorage.BaseStorage):
                                 # so this record can not be the non-version
                                 # record for it.
                                 continue
-                    o.append(r)
+                        o.append(r)
 
                 if o:
-                    if len(o) != len(t):
-                        _data[tid]=1, u, d, e, tuple(o) # Reset data
+                    if len(o) != len(records):
+                        self._data[tid] = 1, u, d, e, tuple(o) # Reset data
                 else:
                     deleted.append(tid)
 
             # Now delete empty transactions
-            for tid in deleted: del _data[tid]
+            for tid in deleted:
+                del self._data[tid]
 
             # Now reset previous pointers for "current" records:
             for r in pindex.values():
-                r[2]=None # Previous record
-                if r[3]: # vdata
-                    r[3][1][2]=None
-
-            pindex=None
+                r[1] = None # Previous record
+                if r[2] and r[2][1]: # vdata
+                    # If this record contains version data and
+                    # non-version data, then clear it out.
+                    r[2][1][2] = None
 
             # Finally, rebuild indexes from transaction data:
             self._index, self._vindex = self._build_indexes()
 
-        finally: self._lock_release()
+        finally:
+            self._lock_release()
         self.getSize()
 
     def _splat(self):
@@ -534,21 +524,22 @@ class DemoStorage(BaseStorage.BaseStorage):
         for tid, (p, u, d, e, t) in self._data.items():
             o.append("  %s %s" % (TimeStamp(tid), p))
             for r in t:
-                oid, serial, pre, vdata, p = r
-                oid=utils.u64(oid)
-                if serial is not None: serial=str(TimeStamp(serial))
+                oid, pre, vdata, p, tid = r
+                oid = oid_repr(oid)
+                tid = oid_repr(tid)
+##                if serial is not None: serial=str(TimeStamp(serial))
                 pre=id(pre)
                 if vdata and vdata[1]: vdata=vdata[0], id(vdata[1])
                 if p: p=''
                 o.append('    %s: %s' %
-                         (id(r), `(oid, serial, pre, vdata, p)`))
+                         (id(r), `(oid, pre, vdata, p, tid)`))
 
         o.append('\nIndex:')
         items=self._index.items()
         items.sort()
         for oid, r in items:
             if r: r=id(r)
-            o.append('  %s: %s' % (utils.u64(oid), r))
+            o.append('  %s: %s' % (oid_repr(oid), r))
 
         o.append('\nVersion Index:')
         items=self._vindex.items()
@@ -559,7 +550,6 @@ class DemoStorage(BaseStorage.BaseStorage):
             vitems.sort()
             for oid, r in vitems:
                 if r: r=id(r)
-                o.append('    %s: %s' % (utils.u64(oid), r))
+                o.append('    %s: %s' % (oid_repr(oid), r))
 
-
-        return string.join(o,'\n')
+        return '\n'.join(o)
