@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -13,17 +13,12 @@
 ##############################################################################
 """Tests for application-level conflict resolution."""
 
-from ZODB.Transaction import Transaction
 from ZODB.POSException import ConflictError, UndoError
-from Persistence import Persistent
+from persistent import Persistent
+from transaction import Transaction
 
 from ZODB.tests.StorageTestBase import zodb_unpickle, zodb_pickle
 
-import sys
-import types
-from cPickle import Pickler, Unpickler
-from cStringIO import StringIO
-
 class PCounter(Persistent):
 
     _value = 0
@@ -42,7 +37,7 @@ class PCounter(Persistent):
 
         return oldState
 
-    # XXX What if _p_resolveConflict _thinks_ it resolved the
+    # Insecurity:  What if _p_resolveConflict _thinks_ it resolved the
     # conflict, but did something wrong?
 
 class PCounter2(PCounter):
@@ -52,11 +47,11 @@ class PCounter2(PCounter):
 
 class PCounter3(PCounter):
     def _p_resolveConflict(self, oldState, savedState, newState):
-        raise AttributeError, "no attribute (testing conflict resolution)"
+        raise AttributeError("no attribute (testing conflict resolution)")
 
 class PCounter4(PCounter):
     def _p_resolveConflict(self, oldState, savedState):
-        raise RuntimeError, "Can't get here; not enough args"
+        raise RuntimeError("Can't get here; not enough args")
 
 class ConflictResolvingStorage:
 
@@ -94,14 +89,17 @@ class ConflictResolvingStorage:
         # pickle is to commit two different transactions relative to
         # revid1 that add two to _value.
         revid2 = self._dostoreNP(oid, revid=revid1, data=zodb_pickle(obj))
-        self.assertRaises(ConflictError,
-                          self._dostoreNP,
-                          oid, revid=revid1, data=zodb_pickle(obj))
+        try:
+            self._dostoreNP(oid, revid=revid1, data=zodb_pickle(obj))
+        except ConflictError, err:
+            self.assert_("PCounter2" in str(err))
+        else:
+            self.fail("Expected ConflictError")
 
     def checkZClassesArentResolved(self):
-        from ZODB.ConflictResolution import bad_class
+        from ZODB.ConflictResolution import find_global, BadClassName
         dummy_class_tuple = ('*foobar', ())
-        assert bad_class(dummy_class_tuple) == 1
+        self.assertRaises(BadClassName, find_global, '*foobar', ())
 
     def checkBuggyResolve1(self):
         obj = PCounter3()
@@ -159,7 +157,7 @@ class ConflictResolvingTransUndoStorage:
         tid = info[1]['id']
         t = Transaction()
         self._storage.tpc_begin(t)
-        self._storage.transactionalUndo(tid, t)
+        self._storage.undo(tid, t)
         self._storage.tpc_finish(t)
 
     def checkUndoUnresolvable(self):
@@ -180,6 +178,6 @@ class ConflictResolvingTransUndoStorage:
         tid = info[1]['id']
         t = Transaction()
         self._storage.tpc_begin(t)
-        self.assertRaises(UndoError, self._storage.transactionalUndo,
+        self.assertRaises(UndoError, self._storage.undo,
                           tid, t)
         self._storage.tpc_abort(t)
