@@ -1,18 +1,33 @@
-"""Check transactionalUndo().
+##############################################################################
+#
+# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
+# All Rights Reserved.
+#
+# This software is subject to the provisions of the Zope Public License,
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
+# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
+# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
+# FOR A PARTICULAR PURPOSE.
+#
+##############################################################################
+"""Check undo().
 
-Any storage that supports transactionalUndo() must pass these tests.
+Any storage that supports undo() must pass these tests.
 """
-from __future__ import nested_scopes
 
 import time
 import types
+
+from persistent import Persistent
+import transaction
+from transaction import Transaction
+
 from ZODB import POSException
-from ZODB.Transaction import Transaction
-from ZODB.referencesf import referencesf
-from ZODB.utils import u64, p64
+from ZODB.serialize import referencesf
+from ZODB.utils import p64
 from ZODB import DB
 
-from Persistence import Persistent
 from ZODB.tests.MinPO import MinPO
 from ZODB.tests.StorageTestBase import zodb_pickle, zodb_unpickle
 
@@ -21,6 +36,23 @@ ZERO = '\0'*8
 class C(Persistent):
     pass
 
+def snooze():
+    # In Windows, it's possible that two successive time.time() calls return
+    # the same value.  Tim guarantees that time never runs backwards.  You
+    # usually want to call this before you pack a storage, or must make other
+    # guarantees about increasing timestamps.
+    now = time.time()
+    while now == time.time():
+        time.sleep(0.1)
+
+def listeq(L1, L2):
+    """Return True if L1.sort() == L2.sort()"""
+    c1 = L1[:]
+    c2 = L2[:]
+    c1.sort()
+    c2.sort()
+    return c1 == c2
+
 class TransactionalUndoStorage:
 
     def _transaction_begin(self):
@@ -69,6 +101,15 @@ class TransactionalUndoStorage:
             for rec in txn:
                 pass
 
+    def undo(self, tid, note):
+        t = Transaction()
+        t.note(note)
+        self._storage.tpc_begin(t)
+        oids = self._storage.undo(tid, t)
+        self._storage.tpc_vote(t)
+        self._storage.tpc_finish(t)
+        return oids
+
     def checkSimpleTransactionalUndo(self):
         eq = self.assertEqual
         oid = self._storage.new_oid()
@@ -77,60 +118,48 @@ class TransactionalUndoStorage:
         revid = self._dostore(oid, revid=revid, data=MinPO(25))
 
         info = self._storage.undoInfo()
-        tid = info[0]['id']
         # Now start an undo transaction
-        t = Transaction()
-        t.note('undo1')
-        self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
-        self._storage.tpc_vote(t)
-        self._storage.tpc_finish(t)
-        eq(len(oids), 1)
-        eq(oids[0], oid)
+        self._undo(info[0]["id"], [oid], note="undo1")
         data, revid = self._storage.load(oid, '')
         eq(zodb_unpickle(data), MinPO(24))
+
         # Do another one
         info = self._storage.undoInfo()
-        tid = info[2]['id']
-        t = Transaction()
-        t.note('undo2')
-        self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
-        self._storage.tpc_vote(t)
-        self._storage.tpc_finish(t)
-        eq(len(oids), 1)
-        eq(oids[0], oid)
+        self._undo(info[2]["id"], [oid], note="undo2")
         data, revid = self._storage.load(oid, '')
         eq(zodb_unpickle(data), MinPO(23))
+
         # Try to undo the first record
         info = self._storage.undoInfo()
-        tid = info[4]['id']
-        t = Transaction()
-        t.note('undo3')
-        self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
-        self._storage.tpc_vote(t)
-        self._storage.tpc_finish(t)
-
-        eq(len(oids), 1)
-        eq(oids[0], oid)
+        self._undo(info[4]["id"], [oid], note="undo3")
         # This should fail since we've undone the object's creation
         self.assertRaises(KeyError,
                           self._storage.load, oid, '')
+
         # And now let's try to redo the object's creation
         info = self._storage.undoInfo()
-        tid = info[0]['id']
-        t = Transaction()
-        self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
-        self._storage.tpc_vote(t)
-        self._storage.tpc_finish(t)
-        eq(len(oids), 1)
-        eq(oids[0], oid)
+        self._undo(info[0]["id"], [oid])
         data, revid = self._storage.load(oid, '')
         eq(zodb_unpickle(data), MinPO(23))
         self._iterate()
 
+    def checkCreationUndoneGetSerial(self):
+        # create an object
+        oid = self._storage.new_oid()
+        self._dostore(oid, data=MinPO(23))
+        # undo its creation
+        info = self._storage.undoInfo()
+        tid = info[0]['id']
+        t = Transaction()
+        t.note('undo1')
+        self._storage.tpc_begin(t)
+        self._storage.undo(tid, t)
+        self._storage.tpc_vote(t)
+        self._storage.tpc_finish(t)
+        # Check that calling getSerial on an uncreated object raises a KeyError
+        # The current version of FileStorage fails this test
+        self.assertRaises(KeyError, self._storage.getSerial, oid)
+
     def checkUndoCreationBranch1(self):
         eq = self.assertEqual
         oid = self._storage.new_oid()
@@ -138,27 +167,14 @@ class TransactionalUndoStorage:
         revid = self._dostore(oid, revid=revid, data=MinPO(12))
         # Undo the last transaction
         info = self._storage.undoInfo()
-        tid = info[0]['id']
-        t = Transaction()
-        self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
-        self._storage.tpc_vote(t)
-        self._storage.tpc_finish(t)
-        eq(len(oids), 1)
-        eq(oids[0], oid)
+        self._undo(info[0]['id'], [oid])
         data, revid = self._storage.load(oid, '')
         eq(zodb_unpickle(data), MinPO(11))
+
         # Now from here, we can either redo the last undo, or undo the object
         # creation.  Let's undo the object creation.
         info = self._storage.undoInfo()
-        tid = info[2]['id']
-        t = Transaction()
-        self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
-        self._storage.tpc_vote(t)
-        self._storage.tpc_finish(t)
-        eq(len(oids), 1)
-        eq(oids[0], oid)
+        self._undo(info[2]['id'], [oid])
         self.assertRaises(KeyError, self._storage.load, oid, '')
         self._iterate()
 
@@ -169,27 +185,13 @@ class TransactionalUndoStorage:
         revid = self._dostore(oid, revid=revid, data=MinPO(12))
         # Undo the last transaction
         info = self._storage.undoInfo()
-        tid = info[0]['id']
-        t = Transaction()
-        self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
-        self._storage.tpc_vote(t)
-        self._storage.tpc_finish(t)
-        eq(len(oids), 1)
-        eq(oids[0], oid)
+        self._undo(info[0]['id'], [oid])
         data, revid = self._storage.load(oid, '')
         eq(zodb_unpickle(data), MinPO(11))
         # Now from here, we can either redo the last undo, or undo the object
         # creation.  Let's redo the last undo
         info = self._storage.undoInfo()
-        tid = info[0]['id']
-        t = Transaction()
-        self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
-        self._storage.tpc_vote(t)
-        self._storage.tpc_finish(t)
-        eq(len(oids), 1)
-        eq(oids[0], oid)
+        self._undo(info[0]['id'], [oid])
         data, revid = self._storage.load(oid, '')
         eq(zodb_unpickle(data), MinPO(12))
         self._iterate()
@@ -231,17 +233,10 @@ class TransactionalUndoStorage:
         eq(zodb_unpickle(data), MinPO(32))
         data, revid2 = self._storage.load(oid2, '')
         eq(zodb_unpickle(data), MinPO(52))
+
         # Now attempt to undo the transaction containing two objects
         info = self._storage.undoInfo()
-        tid = info[0]['id']
-        t = Transaction()
-        self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
-        self._storage.tpc_vote(t)
-        self._storage.tpc_finish(t)
-        eq(len(oids), 2)
-        self.failUnless(oid1 in oids)
-        self.failUnless(oid2 in oids)
+        self._undo(info[0]['id'], [oid1, oid2])
         data, revid1 = self._storage.load(oid1, '')
         eq(zodb_unpickle(data), MinPO(31))
         data, revid2 = self._storage.load(oid2, '')
@@ -287,13 +282,11 @@ class TransactionalUndoStorage:
         tid1 = info[1]['id']
         t = Transaction()
         self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
-        oids1 = self._storage.transactionalUndo(tid1, t)
+        tid, oids = self._storage.undo(tid, t)
+        tid, oids1 = self._storage.undo(tid1, t)
         self._storage.tpc_vote(t)
         self._storage.tpc_finish(t)
         # We get the finalization stuff called an extra time:
-##        self._storage.tpc_vote(t)
-##        self._storage.tpc_finish(t)
         eq(len(oids), 2)
         eq(len(oids1), 2)
         unless(oid1 in oids)
@@ -302,17 +295,10 @@ class TransactionalUndoStorage:
         eq(zodb_unpickle(data), MinPO(30))
         data, revid2 = self._storage.load(oid2, '')
         eq(zodb_unpickle(data), MinPO(50))
+
         # Now try to undo the one we just did to undo, whew
         info = self._storage.undoInfo()
-        tid = info[0]['id']
-        t = Transaction()
-        self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
-        self._storage.tpc_vote(t)
-        self._storage.tpc_finish(t)
-        eq(len(oids), 2)
-        unless(oid1 in oids)
-        unless(oid2 in oids)
+        self._undo(info[0]['id'], [oid1, oid2])
         data, revid1 = self._storage.load(oid1, '')
         eq(zodb_unpickle(data), MinPO(32))
         data, revid2 = self._storage.load(oid2, '')
@@ -344,15 +330,7 @@ class TransactionalUndoStorage:
         eq(revid1, revid2)
         # Now attempt to undo the transaction containing two objects
         info = self._storage.undoInfo()
-        tid = info[0]['id']
-        t = Transaction()
-        self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
-        self._storage.tpc_vote(t)
-        self._storage.tpc_finish(t)
-        eq(len(oids), 2)
-        self.failUnless(oid1 in oids)
-        self.failUnless(oid2 in oids)
+        self._undo(info[0]["id"], [oid1, oid2])
         data, revid1 = self._storage.load(oid1, '')
         eq(zodb_unpickle(data), MinPO(31))
         data, revid2 = self._storage.load(oid2, '')
@@ -378,7 +356,7 @@ class TransactionalUndoStorage:
         tid = info[1]['id']
         t = Transaction()
         self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
+        tid, oids = self._storage.undo(tid, t)
         self._storage.tpc_vote(t)
         self._storage.tpc_finish(t)
         eq(len(oids), 1)
@@ -404,7 +382,7 @@ class TransactionalUndoStorage:
         t = Transaction()
         self._storage.tpc_begin(t)
         self.assertRaises(POSException.UndoError,
-                          self._storage.transactionalUndo,
+                          self._storage.undo,
                           tid, t)
         self._storage.tpc_abort(t)
         # Now have more fun: object1 and object2 are in the same transaction,
@@ -443,7 +421,7 @@ class TransactionalUndoStorage:
         t = Transaction()
         self._storage.tpc_begin(t)
         self.assertRaises(POSException.UndoError,
-                          self._storage.transactionalUndo,
+                          self._storage.undo,
                           tid, t)
         self._storage.tpc_abort(t)
         self._iterate()
@@ -453,16 +431,10 @@ class TransactionalUndoStorage:
         # Add a few object revisions
         oid = self._storage.new_oid()
         revid1 = self._dostore(oid, data=MinPO(51))
-        # For the pack(), we need a timestamp greater than revid1's timestamp.
-        # The semantics of pack()'s `t' argument is that all non-current
-        # revisions with an earlier timestamp will be packed away.  If they
-        # were equal (because the Windows clock resolution is too coarse),
-        # then we won't pack away the first revision.
-        now = packtime = time.time()
-        while packtime <= now:
-            packtime = time.time()
+        packtime = time.time()
+        snooze()                # time.time() now distinct from packtime
         revid2 = self._dostore(oid, revid=revid1, data=MinPO(52))
-        revid3 = self._dostore(oid, revid=revid2, data=MinPO(53))
+        self._dostore(oid, revid=revid2, data=MinPO(53))
         # Now get the undo log
         info = self._storage.undoInfo()
         eq(len(info), 3)
@@ -477,7 +449,7 @@ class TransactionalUndoStorage:
         # And now attempt to undo the last transaction
         t = Transaction()
         self._storage.tpc_begin(t)
-        oids = self._storage.transactionalUndo(tid, t)
+        tid, oids = self._storage.undo(tid, t)
         self._storage.tpc_vote(t)
         self._storage.tpc_finish(t)
         eq(len(oids), 1)
@@ -497,7 +469,7 @@ class TransactionalUndoStorage:
         o2 = C()
         root['obj'] = o1
         o1.obj = o2
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note('o1 -> o2')
         txn.commit()
         now = packtime = time.time()
@@ -506,12 +478,12 @@ class TransactionalUndoStorage:
 
         o3 = C()
         o2.obj = o3
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note('o1 -> o2 -> o3')
         txn.commit()
 
         o1.obj = o3
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note('o1 -> o3')
         txn.commit()
 
@@ -529,7 +501,7 @@ class TransactionalUndoStorage:
 
         tid = log[0]['id']
         db.undo(tid)
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note('undo')
         txn.commit()
         # undo does a txn-undo, but doesn't invalidate
@@ -543,6 +515,108 @@ class TransactionalUndoStorage:
         eq(o1.obj.obj, o3)
         self._iterate()
 
+    def checkPackAfterUndoDeletion(self):
+        db = DB(self._storage)
+        cn = db.open()
+        root = cn.root()
+
+        pack_times = []
+        def set_pack_time():
+            pack_times.append(time.time())
+            snooze()
+
+        root["key0"] = MinPO(0)
+        root["key1"] = MinPO(1)
+        root["key2"] = MinPO(2)
+        txn = transaction.get()
+        txn.note("create 3 keys")
+        txn.commit()
+
+        set_pack_time()
+
+        del root["key1"]
+        txn = transaction.get()
+        txn.note("delete 1 key")
+        txn.commit()
+
+        set_pack_time()
+
+        root._p_deactivate()
+        cn.sync()
+        self.assert_(listeq(root.keys(), ["key0", "key2"]))
+
+        L = db.undoInfo()
+        db.undo(L[0]["id"])
+        txn = transaction.get()
+        txn.note("undo deletion")
+        txn.commit()
+
+        set_pack_time()
+
+        root._p_deactivate()
+        cn.sync()
+        self.assert_(listeq(root.keys(), ["key0", "key1", "key2"]))
+
+        for t in pack_times:
+            self._storage.pack(t, referencesf)
+
+            root._p_deactivate()
+            cn.sync()
+            self.assert_(listeq(root.keys(), ["key0", "key1", "key2"]))
+            for i in range(3):
+                obj = root["key%d" % i]
+                self.assertEqual(obj.value, i)
+            root.items()
+            self._inter_pack_pause()
+
+    def checkPackAfterUndoManyTimes(self):
+        db = DB(self._storage)
+        cn = db.open()
+        rt = cn.root()
+
+        rt["test"] = MinPO(1)
+        transaction.commit()
+        rt["test2"] = MinPO(2)
+        transaction.commit()
+        rt["test"] = MinPO(3)
+        txn = transaction.get()
+        txn.note("root of undo")
+        txn.commit()
+
+        packtimes = []
+        for i in range(10):
+            L = db.undoInfo()
+            db.undo(L[0]["id"])
+            txn = transaction.get()
+            txn.note("undo %d" % i)
+            txn.commit()
+            rt._p_deactivate()
+            cn.sync()
+
+            self.assertEqual(rt["test"].value, i % 2 and 3 or 1)
+            self.assertEqual(rt["test2"].value, 2)
+
+            packtimes.append(time.time())
+            snooze()
+
+        for t in packtimes:
+            self._storage.pack(t, referencesf)
+            cn.sync()
+
+            # TODO:  Is _cache supposed to have a clear() method, or not?
+            # cn._cache.clear()
+
+            # The last undo set the value to 3 and pack should
+            # never change that.
+            self.assertEqual(rt["test"].value, 3)
+            self.assertEqual(rt["test2"].value, 2)
+            self._inter_pack_pause()
+
+    def _inter_pack_pause(self):
+        # DirectoryStorage needs a pause between packs,
+        # most other storages dont.
+        pass
+
     def checkTransactionalUndoIterator(self):
         # check that data_txn set in iterator makes sense
         if not hasattr(self._storage, "iterator"):
@@ -581,10 +655,10 @@ class TransactionalUndoStorage:
             base = i * OBJECTS + i
             for j in range(OBJECTS):
                 tid = info[base + j]['id']
-                s.transactionalUndo(tid, t)
+                s.undo(tid, t)
             s.tpc_vote(t)
             s.tpc_finish(t)
-        
+
         for i in range(BATCHES):
             undo(i)
 
@@ -601,14 +675,14 @@ class TransactionalUndoStorage:
         for i in range(BATCHES):
             txn = iter[offset]
             offset += 1
-            
+
             tid = p64(i + 1)
             eq(txn.tid, tid)
 
-            L1 = [(rec.oid, rec.serial, rec.data_txn) for rec in txn]
+            L1 = [(rec.oid, rec.tid, rec.data_txn) for rec in txn]
             L2 = [(oid, revid, None) for _tid, oid, revid in orig
                   if _tid == tid]
-            
+
             eq(L1, L2)
 
         for i in range(BATCHES * OBJECTS):
@@ -630,3 +704,83 @@ class TransactionalUndoStorage:
             eq(L1, L2)
 
         self.assertRaises(IndexError, iter.__getitem__, offset)
+
+    def checkUndoLogMetadata(self):
+        # test that the metadata is correct in the undo log
+        t = transaction.get()
+        t.note('t1')
+        t.setExtendedInfo('k2','this is transaction metadata')
+        t.setUser('u3',path='p3')
+        db = DB(self._storage)
+        conn = db.open()
+        root = conn.root()
+        o1 = C()
+        root['obj'] = o1
+        txn = transaction.get()
+        txn.commit()
+        l = self._storage.undoLog()
+        self.assertEqual(len(l),2)
+        d = l[0]
+        self.assertEqual(d['description'],'t1')
+        self.assertEqual(d['k2'],'this is transaction metadata')
+        self.assertEqual(d['user_name'],'p3 u3')
+
+    # A common test body for index tests on undoInfo and undoLog.  Before
+    # ZODB 3.4, they always returned a wrong number of results (one too
+    # few _or_ too many, depending on how they were called).
+    def _exercise_info_indices(self, method_name):
+        db = DB(self._storage)
+        info_func = getattr(db, method_name)
+        cn = db.open()
+        rt = cn.root()
+
+        # Do some transactions.
+        for key in "abcdefghijklmnopqrstuvwxyz":
+            rt[key] = ord(key)
+            transaction.commit()
+
+        # 26 letters = 26 transactions, + the hidden transaction to make
+        # the root object, == 27 expected.
+        allofem = info_func(0, 100000)
+        self.assertEqual(len(allofem), 27)
+
+        # Asking for no more than 100000 should do the same.
+        redundant = info_func(last=-1000000)
+        self.assertEqual(allofem, redundant)
+
+        # By default, we should get only 20 back.
+        default = info_func()
+        self.assertEqual(len(default), 20)
+        # And they should be the most recent 20.
+        self.assertEqual(default, allofem[:20])
+
+        # If we ask for only one, we should get only the most recent.
+        fresh = info_func(last=1)
+        self.assertEqual(len(fresh), 1)
+        self.assertEqual(fresh[0], allofem[0])
+
+        # Another way of asking for only the most recent.
+        redundant = info_func(last=-1)
+        self.assertEqual(fresh, redundant)
+
+        # Try a slice that doesn't start at 0.
+        oddball = info_func(first=11, last=17)
+        self.assertEqual(len(oddball), 17-11)
+        self.assertEqual(oddball, allofem[11 : 11+len(oddball)])
+
+        # And another way to spell the same thing.
+        redundant = info_func(first=11, last=-6)
+        self.assertEqual(oddball, redundant)
+
+        cn.close()
+        # Caution:  don't close db; the framework does that.  If you close
+        # it here, the ZODB tests still work, but the ZRS RecoveryStorageTests
+        # fail (closing the DB here in those tests closes the ZRS primary
+        # before a ZRS secondary even starts, and then the latter can't
+        # find a server to recover from).
+
+    def checkIndicesInUndoInfo(self):
+        self._exercise_info_indices("undoInfo")
+
+    def checkIndicesInUndoLog(self):
+        self._exercise_info_indices("undoLog")
