@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,15 +12,21 @@
 ##############################################################################
 """ Basic ZPublisher request management.
 
-$Id: BaseRequest.py,v 1.51.2.2 2003/11/17 22:34:19 tseaver Exp $
+$Id: BaseRequest.py 77299 2007-07-02 10:42:14Z shh $
 """
-
-from urllib import quote
+from urllib import quote as urllib_quote
 import xmlrpc
 from zExceptions import Forbidden
 
+from zope.event import notify
+from zope.app.publication.interfaces import EndRequestEvent
+
 UNSPECIFIED_ROLES=''
 
+def quote(text):
+    # quote url path segments, but leave + and @ intact
+    return urllib_quote(text, '/+@')
+
 try:
     from ExtensionClass import Base
     class RequestContainer(Base):
@@ -31,12 +37,19 @@ try:
         def manage_property_types(self):
             return type_converters.keys()
 
-except:
+except ImportError:
     class RequestContainer:
         __roles__=None
         def __init__(self,**kw):
             for k,v in kw.items(): self.__dict__[k]=v
 
+try:
+    from AccessControl.ZopeSecurityPolicy import getRoles
+except ImportError:
+    def getRoles(container, name, value, default):
+        return getattr(value, '__roles__', default)
+
+
 _marker=[]
 class BaseRequest:
     """Provide basic ZPublisher request management
@@ -76,6 +89,7 @@ class BaseRequest:
     def close(self):
         self.other.clear()
         self._held=None
+        notify(EndRequestEvent(None, self))
 
     def processInputs(self):
         """Do any input processing that could raise errors
@@ -143,6 +157,9 @@ class BaseRequest:
     def has_key(self,key):
         return self.get(key, _marker) is not _marker
 
+    def __contains__(self, key):
+        return self.has_key(key)
+    
     def keys(self):
         keys = {}
         keys.update(self.common)
@@ -182,6 +199,9 @@ class BaseRequest:
         if response is None: response=self.response
         debug_mode=response.debug_mode
 
+        # remember path for later use
+        browser_path = path
+
         # Cleanup the path list
         if path[:1]=='/':  path=path[1:]
         if path[-1:]=='/': path=path[:-1]
@@ -219,14 +239,14 @@ class BaseRequest:
         object=parents[-1]
         del parents[:]
 
-        roles = getattr(object, '__roles__', UNSPECIFIED_ROLES)
+        roles = getRoles(None, None, object, UNSPECIFIED_ROLES)
 
         # if the top object has a __bobo_traverse__ method, then use it
         # to possibly traverse to an alternate top-level object.
         if hasattr(object,'__bobo_traverse__'):
             try:
                 object=object.__bobo_traverse__(request)
-                roles =getattr(object, '__roles__', UNSPECIFIED_ROLES)
+                roles = getRoles(None, None, object, UNSPECIFIED_ROLES)
             except: pass
 
         if not path and not method:
@@ -244,6 +264,10 @@ class BaseRequest:
         path.reverse()
 
         request['TraversalRequestNameStack'] = request.path = path
+        request['ACTUAL_URL'] = request['URL'] + quote(browser_path)
+
+        # Set the posttraverse for duration of the traversal here
+        self._post_traverse = post_traverse = []
 
         entry_name = ''
         try:
@@ -281,9 +305,9 @@ class BaseRequest:
                     entry_name = method
                     method = 'index_html'
                 else:
-                    if (hasattr(object, '__call__') and
-                        hasattr(object.__call__,'__roles__')):
-                        roles=object.__call__.__roles__
+                    if (hasattr(object, '__call__')):
+                        roles = getRoles(object, '__call__', object.__call__,
+                                         roles)
                     if request._hacked_path:
                         i=URL.rfind('/')
                         if i > 0: response.setBase(URL[:i])
@@ -299,14 +323,20 @@ class BaseRequest:
                     else: return response.forbiddenError(entry_name)
 
                 if hasattr(object,'__bobo_traverse__'):
-                    subobject=object.__bobo_traverse__(request,entry_name)
-                    if type(subobject) is type(()) and len(subobject) > 1:
-                        # Add additional parents into the path
-                        parents[-1:] = list(subobject[:-1])
-                        object, subobject = subobject[-2:]
+                    try:
+                        subobject=object.__bobo_traverse__(request,entry_name)
+                        if type(subobject) is type(()) and len(subobject) > 1:
+                            # Add additional parents into the path
+                            parents[-1:] = list(subobject[:-1])
+                            object, subobject = subobject[-2:]
+                    except (AttributeError, KeyError):
+                        if debug_mode:
+                            return response.debugError(
+                                "Cannot locate object at: %s" % URL)
+                        else:
+                            return response.notFoundError(URL)
                 else:
                     try:
-
                         # Note - no_acquire_flag is necessary to support
                         # things like DAV.  We have to make sure
                         # that the target object is not acquired
@@ -352,19 +382,15 @@ class BaseRequest:
                 # certain mutable types (dicts, lists) to become publishable
                 # when they shouldn't be. The following check makes sure that
                 # the right thing happens in both 2.2.2+ and earlier versions.
-                
+
                 if not typeCheck(subobject):
                     return response.debugError(
                         "The object at %s is not publishable." % URL
                         )
 
-                r = getattr(subobject, '__roles__', UNSPECIFIED_ROLES)
-                if r is not UNSPECIFIED_ROLES:
-                    roles = r
-                elif not got:
-                    # We got the subobject as an attribute, not an item,
-                    # so we should check "next to it" for __roles__.
-                    roles = getattr(object, entry_name+'__roles__', roles)
+                roles = getRoles(
+                    object, (not got) and entry_name or None, subobject,
+                    roles)
 
                 # Promote subobject to object
                 object=subobject
@@ -373,6 +399,9 @@ class BaseRequest:
                 steps.append(entry_name)
         finally:
             parents.reverse()
+ 
+        # After traversal post traversal hooks aren't available anymore
+        del self._post_traverse
 
         request['PUBLISHED'] = parents.pop(0)
 
@@ -446,18 +475,47 @@ class BaseRequest:
         # Remove http request method from the URL.
         request['URL']=URL
 
+        # Run post traversal hooks
+        if post_traverse:
+            result = exec_callables(post_traverse)
+            if result is not None:
+                object = result
+
         return object
 
+    def post_traverse(self, f, args=()):
+        """Add a callable object and argument tuple to be post-traversed.
+        
+        If traversal and authentication succeed, each post-traversal
+        pair is processed in the order in which they were added.
+        Each argument tuple is passed to its callable.  If a callable
+        returns a value other than None, no more pairs are processed,
+        and the return value replaces the traversal result.
+        """
+        try:
+            pairs = self._post_traverse
+        except AttributeError:
+            raise RuntimeError, ('post_traverse() may only be called '
+                                 'during publishing traversal.')
+        else:
+            pairs.append((f, tuple(args)))
+
     retry_count=0
     def supports_retry(self): return 0
 
     def _hold(self, object):
         """Hold a reference to an object to delay it's destruction until mine
         """
-        self._held=self._held+(object,)
-
-
+        if self._held is not None:
+            self._held=self._held+(object,)
 
+def exec_callables(callables):
+    result = None
+    for (f, args) in callables:
+        # Don't catch exceptions here. And don't hide them anyway.
+        result = f(*args)
+        if result is not None:
+            return result
 
 def old_validation(groups, request, auth,
                    roles=UNSPECIFIED_ROLES):
@@ -521,10 +579,15 @@ itypes = {}
 for name in ('NoneType', 'IntType', 'LongType', 'FloatType', 'StringType',
              'BufferType', 'TupleType', 'ListType', 'DictType', 'XRangeType',
              'SliceType', 'EllipsisType', 'UnicodeType', 'CodeType',
-             'TracebackType', 'FrameType', 'DictProxyType'):
+             'TracebackType', 'FrameType', 'DictProxyType', 'BooleanType',
+             'ComplexType'):
     if hasattr(types, name):
         itypes[getattr(types, name)] = 0
 
+# Python 2.4 no longer maintains the types module.
+itypes[set] = 0
+itypes[frozenset] = 0
+
 def typeCheck(obj, deny=itypes):
     # Return true if its ok to publish the type, false otherwise.
     return deny.get(type(obj), 1)
