@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,13 +12,10 @@
 ##############################################################################
 '''CGI Response Output formatter
 
-$Id: BaseResponse.py,v 1.17.64.1 2003/11/17 22:34:19 tseaver Exp $'''
-__version__ = '$Revision: 1.17.64.1 $'[11:-2]
+$Id: BaseResponse.py 40222 2005-11-18 15:46:28Z andreasjung $'''
+__version__ = '$Revision: 1.19 $'[11:-2]
 
-import  types, sys
-from types import StringType, InstanceType
-from zExceptions import Unauthorized, Forbidden
-from zExceptions import NotFound, BadRequest
+from zExceptions import Unauthorized, Forbidden, NotFound, BadRequest
 
 class BaseResponse:
     """Base Response Class
