@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -15,10 +15,10 @@ __version__='$Revision: 1.96 $'[11:-2]
 
 import re, sys, os, time, random, codecs
 from types import StringType, UnicodeType
-from BaseRequest import BaseRequest
+from BaseRequest import BaseRequest, quote
 from HTTPResponse import HTTPResponse
 from cgi import FieldStorage, escape
-from urllib import quote, unquote, splittype, splitport
+from urllib import unquote, splittype, splitport
 from copy import deepcopy
 from Converters import get_converter
 from TaintedString import TaintedString
@@ -269,7 +269,15 @@ class HTTPRequest(BaseRequest):
             if environ.has_key('HTTP_X_FORWARDED_FOR') and self._client_addr in trusted_proxies:
                 # REMOTE_ADDR is one of our trusted local proxies. Not really very remote at all.
                 # The proxy can tell us the IP of the real remote client in the forwarded-for header
-                self._client_addr = environ['HTTP_X_FORWARDED_FOR'].split(',')[-1].strip()
+                # Skip the proxy-address itself though
+                forwarded_for = [
+                    e.strip()
+                    for e in environ['HTTP_X_FORWARDED_FOR'].split(',')]
+                forwarded_for.reverse()
+                for entry in forwarded_for:
+                    if entry not in trusted_proxies:
+                        self._client_addr = entry
+                        break
         else:
             self._client_addr = ''
 
@@ -1039,7 +1047,7 @@ class HTTPRequest(BaseRequest):
         except: rsp.exception()
         if object is None:
             req.close()
-            raise sys.exc_info()[0], rsp.errmsg
+            raise rsp.errmsg, sys.exc_info()[1]
 
         # The traversal machinery may return a "default object"
         # like an index_html document. This is not appropriate
@@ -1293,7 +1301,7 @@ class HTTPRequest(BaseRequest):
         return result+"</table>"
 
     def __repr__(self):
-        return "<%s, URL=%s>" % (self.__class__.__name__, self['URL'])
+        return "<%s, URL=%s>" % (self.__class__.__name__, self.get('URL'))
 
     def text(self):
         result="FORM\n\n"
@@ -1339,16 +1347,6 @@ class HTTPRequest(BaseRequest):
     def taintWrapper(self, enabled=TAINTING_ENABLED):
         return enabled and TaintRequestWrapper(self) or self
 
-    # Used by Five
-
-    def getPresentationSkin(self):
-        """see zope.component.interfaces.IPresentationRequest"""
-        return getattr(self, '_presentation_skin', None)
-
-    def setPresentationSkin(self, skin):
-        """see zope.publisher.interfaces.IPublicationRequest"""
-        self._presentation_skin = skin
-
     def shiftNameToApplication(self):
         """see zope.publisher.interfaces.http.IVirtualHostRequest"""
         # this is needed for ++skin++
@@ -1429,7 +1427,8 @@ class FileUpload:
         if hasattr(file, '__methods__'): methods=file.__methods__
         else: methods= ['close', 'fileno', 'flush', 'isatty',
                         'read', 'readline', 'readlines', 'seek',
-                        'tell', 'truncate', 'write', 'writelines']
+                        'tell', 'truncate', 'write', 'writelines',
+                        '__iter__','next'] # see Collector 1837
 
         d=self.__dict__
         for m in methods:
@@ -1449,6 +1448,8 @@ class FileUpload:
         """
         return not not self.filename
 
+    def xreadlines(self):
+        return self
 
 parse_cookie_lock=allocate_lock()
 def parse_cookie(text,
@@ -1456,7 +1457,7 @@ def parse_cookie(text,
                  qparmre=re.compile(
                     '([\x00- ]*([^\x00- ;,="]+)="([^"]*)"([\x00- ]*[;,])?[\x00- ]*)'),
                  parmre=re.compile(
-                    '([\x00- ]*([^\x00- ;,="]+)=([^\x00- ;,"]*)([\x00- ]*[;,])?[\x00- ]*)'),
+                    '([\x00- ]*([^\x00- ;,="]+)=([^;,"]*)([\x00- ]*[;,])?[\x00- ]*)'),
                  paramlessre=re.compile(
                     '([\x00- ]*([^\x00- ;,="]+)[\x00- ]*[;,][\x00- ]*)'),
 
@@ -1573,4 +1574,4 @@ def _filterPasswordFields(items):
 # if any trusted-proxies are defined in the configuration file.
 
 trusted_proxies = []
-    
+
