@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -72,15 +72,16 @@ class ZServerHTTPResponse(HTTPResponse):
                 self.status == 200:
             self.setStatus('nocontent')
 
-        # add content length if not streaming
-        if not headers.has_key('content-length') and \
-                not self._streaming:
-            self.setHeader('content-length',len(body))
-
-
-        content_length= headers.get('content-length', None)
-        if content_length>0 :
-            self.setHeader('content-length', content_length)
+        if self.status in (100, 101, 102, 204, 304):
+            # These responses should not have any body or Content-Length.
+            # See RFC 2616 4.4 "Message Length".
+            body = ''
+            if 'content-length' in headers:
+                del headers['content-length']
+            if 'content-type' in headers:
+                del headers['content-type']
+        elif not headers.has_key('content-length') and not self._streaming:
+            self.setHeader('content-length', len(body))
 
         headersl=[]
         append=headersl.append
@@ -97,8 +98,7 @@ class ZServerHTTPResponse(HTTPResponse):
         append('Date: %s' % build_http_date(time.time()))
 
         if self._http_version=='1.0':
-            if self._http_connection=='keep-alive' and \
-                    self.headers.has_key('content-length'):
+            if self._http_connection=='keep-alive':
                 self.setHeader('Connection','Keep-Alive')
             else:
                 self.setHeader('Connection','close')
@@ -108,14 +108,19 @@ class ZServerHTTPResponse(HTTPResponse):
         if self._http_version=='1.1':
             if self._http_connection=='close':
                 self.setHeader('Connection','close')
-            elif not self.headers.has_key('content-length'):
-                if self.http_chunk and self._streaming:
-                    self.setHeader('Transfer-Encoding','chunked')
-                    self._chunking=1
-                else:
-                    self.setHeader('Connection','close')
+            elif (not self.headers.has_key('content-length') and 
+                  self.http_chunk and self._streaming):
+                self.setHeader('Transfer-Encoding','chunked')
+                self._chunking=1
+                    
+        headers = headers.items()
+        for line in self.accumulated_headers.splitlines():
+            if line[0] == '\t':
+                headers[-1][1] += '\n' + line
+                continue
+            headers.append(line.split(': ', 1))
 
-        for key, val in headers.items():
+        for key, val in headers:
             if key.lower()==key:
                 # only change non-literal header names
                 key="%s%s" % (key[:1].upper(), key[1:])
@@ -125,10 +130,13 @@ class ZServerHTTPResponse(HTTPResponse):
                     key="%s-%s%s" % (key[:l],key[l+1:l+2].upper(),key[l+2:])
                     start=l+1
                     l=key.find('-',start)
+                val = val.replace('\n\t', '\r\n\t')
             append("%s: %s" % (key, val))
         if self.cookies:
-            headersl=headersl+self._cookie_list()
-        headersl[len(headersl):]=[self.accumulated_headers, body]
+            headersl.extend(self._cookie_list())
+            
+        append('')
+        append(body)
         return "\r\n".join(headersl)
 
     _tempfile=None
@@ -151,6 +159,7 @@ class ZServerHTTPResponse(HTTPResponse):
 
         """
 
+
         if type(data) != type(''):
             raise TypeError('Value must be a string')
 
