@@ -1,20 +1,19 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
 #
 ##############################################################################
-__doc__='''Simple Tree classes
-
-$Id: SimpleTree.py,v 1.4.42.1 2004/01/08 23:34:03 tseaver Exp $'''
-__version__='$Revision: 1.4.42.1 $'[11:-2]
+"""Simple Tree classes
 
+$Id: SimpleTree.py 40222 2005-11-18 15:46:28Z andreasjung $
+"""
 from Tree import TreeMaker, TreeNode, b2a
 from cgi import escape
 
