@@ -1,20 +1,19 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
 #
 ##############################################################################
-__doc__='''Zope-specific versions of ZTUTils classes
-
-$Id: Zope.py,v 1.11.42.6 2004/01/27 15:45:35 fdrake Exp $'''
-__version__='$Revision: 1.11.42.6 $'[11:-2]
+"""Zope-specific versions of ZTUTils classes
 
+$Id: Zope.py 69796 2006-08-27 14:14:52Z shh $
+"""
 import sys, cgi, urllib, cgi
 from Tree import encodeExpansion, decodeExpansion, TreeMaker
 from SimpleTree import SimpleTreeMaker
@@ -201,7 +200,7 @@ def make_hidden_input(*args, **kwargs):
         d.update(arg)
     d.update(kwargs)
 
-    hq = cgi.escape
+    hq = lambda x:cgi.escape(x, quote=True)
     qlist = complex_marshal(d.items())
     for i in range(len(qlist)):
         k, m, v = qlist[i]
@@ -236,8 +235,14 @@ def complex_marshal(pairs):
         elif hasattr(v, 'items'):
             sublist = []
             for sk, sv in v.items():
-                sm = simple_marshal(sv)
-                sublist.append(('%s.%s' % (k, sk), '%s:record' % sm,  sv))
+                if isinstance(sv, list):
+                    for ssv in sv:
+                        sm = simple_marshal(ssv)
+                        sublist.append(('%s.%s' % (k, sk), 
+                                            '%s:list:record' % sm, ssv))
+                else:
+                    sm = simple_marshal(sv)
+                    sublist.append(('%s.%s' % (k, sk), '%s:record' % sm,  sv))
         elif isinstance(v, list):
             sublist = []
             for sv in v:
