@@ -1,26 +1,29 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """WebDAV support - lockable item.
 
-$Id: Lockable.py 40281 2005-11-21 00:08:00Z efge $
+$Id: Lockable.py 40279 2005-11-20 23:50:04Z efge $
 """
 
+import Acquisition
 from AccessControl import ClassSecurityInfo
 from Globals import InitializeClass
 from Globals import PersistentMapping
-import Acquisition
+from zope.interface import implements
 
 from EtagSupport import EtagSupport
+from interfaces import ILockItem
+from interfaces import IWriteLock
 from WriteLockInterface import LockItemInterface
 from WriteLockInterface import WriteLockInterface
 
@@ -28,15 +31,15 @@ from WriteLockInterface import WriteLockInterface
 class ResourceLockedError(Exception): pass
 
 class LockableItem(EtagSupport):
-    """\
-    Implements the WriteLockInterface, and is inherited by Resource which
-    is then inherited by the majority of Zope objects.  For an object to
-    be lockable, however, it should have the WriteLockInterface in its
-    __implements__ list, ie:
 
-    __implements__ = (WriteLockInterface,)
+    """Implements the WriteLock interface.
+
+    This class is inherited by Resource which is then inherited by the
+    majority of Zope objects.
     """
 
+    implements(IWriteLock)
+
     # Protect methods using declarative security
     security = ClassSecurityInfo()
     security.declarePrivate('wl_lockmapping')
@@ -102,7 +105,8 @@ class LockableItem(EtagSupport):
 
     def wl_setLock(self, locktoken, lock):
         locks = self.wl_lockmapping(create=1)
-        if LockItemInterface.isImplementedBy(lock):
+        if ILockItem.providedBy(lock) or \
+                LockItemInterface.isImplementedBy(lock):
             if locktoken == lock.getLockToken():
                 locks[locktoken] = lock
             else:
@@ -128,8 +132,10 @@ class LockableItem(EtagSupport):
         except:
             # The locks may be totally messed up, so we'll just delete
             # and replace.
-            if hasattr(self, '_dav_writelocks'): del self._dav_writelocks
-            if WriteLockInterface.isImplementedBy(self):
+            if hasattr(self, '_dav_writelocks'):
+                del self._dav_writelocks
+            if IWriteLock.providedBy(self) or \
+                    WriteLockInterface.isImplementedBy(self):
                 self._dav_writelocks = PersistentMapping()
 
         # Call into a special hook used by LockNullResources to delete
@@ -146,4 +152,5 @@ InitializeClass(LockableItem)
 def wl_isLocked(ob):
     """ Returns true if the object is locked, returns 0 if the object
     is not locked or does not implement the WriteLockInterface """
-    return WriteLockInterface.isImplementedBy(ob) and ob.wl_isLocked()
+    return (IWriteLock.providedBy(ob) or
+            WriteLockInterface.isImplementedBy(ob)) and ob.wl_isLocked()
