@@ -1,36 +1,43 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
+"""WebDAV support - null resource objects.
 
-"""WebDAV support - null resource objects."""
+$Id: NullResource.py 41329 2006-01-16 17:35:51Z tseaver $
+"""
 
-__version__='$Revision: 1.39.62.4 $'[11:-2]
+import sys
 
-import sys, os,  mimetypes, Globals, davcmds
-import Acquisition, OFS.content_types
-from common import absattr, aq_base, urlfix, tokenFinder, IfParser
-from AccessControl.Permission import Permission
-from AccessControl import getSecurityManager
-from Resource import Resource
-from Globals import Persistent, DTMLFile
-from WriteLockInterface import WriteLockInterface
+import Acquisition
+import Globals
 import OFS.SimpleItem
-from zExceptions import Unauthorized, NotFound, Forbidden, BadRequest
+from AccessControl import getSecurityManager
+from Globals import Persistent, DTMLFile
+from OFS.CopySupport import CopyError
 from zExceptions import MethodNotAllowed
+from zExceptions import Unauthorized, NotFound, Forbidden, BadRequest
+from zope.app.content_types import guess_content_type
+
+import davcmds
+from common import aq_base, tokenFinder, IfParser
 from common import isDavCollection
 from common import Locked, Conflict, PreconditionFailed, UnsupportedMediaType
-from OFS.CopySupport import CopyError
+from interfaces import IWriteLock
+from Resource import Resource
+from WriteLockInterface import WriteLockInterface
+
 
 class NullResource(Persistent, Acquisition.Implicit, Resource):
+
     """Null resources are used to handle HTTP method calls on
     objects which do not yet exist in the url namespace."""
 
@@ -65,6 +72,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
 
     # Most methods return 404 (Not Found) for null resources.
     DELETE=TRACE=PROPFIND=PROPPATCH=COPY=MOVE=HEAD
+    index_html = HEAD
 
     def _default_PUT_factory( self, name, typ, body ):
         # Return DTMLDoc/PageTemplate/Image/File, based on sniffing.
@@ -94,7 +102,9 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
         parent = self.__parent__
 
         ifhdr = REQUEST.get_header('If', '')
-        if WriteLockInterface.isImplementedBy(parent) and parent.wl_isLocked():
+        if (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             if ifhdr:
                 parent.dav__simpleifhandler(REQUEST, RESPONSE, col=1)
             else:
@@ -132,7 +142,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
 
         typ=REQUEST.get_header('content-type', None)
         if typ is None:
-            typ, enc=OFS.content_types.guess_content_type(name, body)
+            typ, enc = guess_content_type(name, body)
 
         factory = getattr(parent, 'PUT_factory', self._default_PUT_factory )
         ob = factory(name, typ, body)
@@ -172,7 +182,9 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
             raise Forbidden, 'Cannot create collection at this location.'
 
         ifhdr = REQUEST.get_header('If', '')
-        if WriteLockInterface.isImplementedBy(parent) and parent.wl_isLocked():
+        if (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             if ifhdr:
                 parent.dav__simpleifhandler(REQUEST, RESPONSE, col=1)
             else:
@@ -202,7 +214,9 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
         name = self.__name__
         parent = self.__parent__
 
-        if WriteLockInterface.isImplementedBy(parent) and parent.wl_isLocked():
+        if (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             if ifhdr:
                 parent.dav__simpleifhandler(REQUEST, RESPONSE, col=1)
             else:
@@ -234,7 +248,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
         else:
             # The command was succesful
             lock = locknull.wl_getLock(token)
-            RESPONSE.setStatus(200)                         
+            RESPONSE.setStatus(200)
             RESPONSE.setHeader('Content-Type', 'text/xml; charset="utf-8"')
             RESPONSE.setHeader('Lock-Token', 'opaquelocktoken:' + token)
             RESPONSE.setBody(lock.asXML())
@@ -369,7 +383,9 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         # First we need to see if the parent of the locknull is locked, and
         # if the user owns that lock (checked by handling the information in
         # the If header).
-        if WriteLockInterface.isImplementedBy(parent) and parent.wl_isLocked():
+        if (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             itrue = parent.dav__simpleifhandler(REQUEST, RESPONSE, 'PUT',
                                                 col=1, url=parenturl,
                                                 refresh=1)
@@ -392,7 +408,7 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         body = REQUEST.get('BODY', '')
         typ = REQUEST.get_header('content-type', None)
         if typ is None:
-            typ, enc = OFS.content_types.guess_content_type(name, body)
+            typ, enc = guess_content_type(name, body)
 
         factory = getattr(parent, 'PUT_factory', self._default_PUT_factory)
         ob = (factory(name, typ, body) or
@@ -407,7 +423,8 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
             raise Forbidden, sys.exc_info()[1]
 
         # Put the locks on the new object
-        if not WriteLockInterface.isImplementedBy(ob):
+        if not (IWriteLock.providedBy(ob) or
+                WriteLockInterface.isImplementedBy(ob)):
             raise MethodNotAllowed, (
                 'The target object type cannot be locked')
         for token, lock in locks:
@@ -439,7 +456,9 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
 
         # If the parent object is locked, that information should be in the
         # if-header if the user owns a lock on the parent
-        if WriteLockInterface.isImplementedBy(parent) and parent.wl_isLocked():
+        if (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             itrue = parent.dav__simpleifhandler(REQUEST, RESPONSE, 'MKCOL',
                                                 col=1, url=parenturl,
                                                 refresh=1)
@@ -466,5 +485,4 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         RESPONSE.setBody('')
         return RESPONSE
 
-
 Globals.default__class_init__(LockNullResource)
