@@ -1,48 +1,63 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-
 """WebDAV support - resource objects.
 
-$Id: Resource.py 65606 2006-02-28 19:55:33Z sidnei $
+$Id: Resource.py 73501 2007-03-24 14:46:00Z shh $
 """
 
-import sys
 import mimetypes
+import sys
+import warnings
 from urllib import unquote
 
-import Globals
 import ExtensionClass
-from Acquisition import aq_base
+import Globals
 from AccessControl import getSecurityManager
+from Acquisition import aq_base, aq_inner, aq_parent
+from zExceptions import BadRequest, MethodNotAllowed
+from zExceptions import Unauthorized, Forbidden, NotFound
+from zope.interface import implements
 from ZPublisher.HTTPRangeSupport import HTTPRangeInterface
-from zExceptions import Unauthorized, Forbidden
-from zExceptions import BadRequest, MethodNotAllowed, NotFound
 
-from WriteLockInterface import WriteLockInterface
+import davcmds
 import Lockable
 from common import absattr, urlfix, rfc1123_date, tokenFinder, urlbase
 from common import IfParser
 from common import isDavCollection
 from common import Locked, Conflict, PreconditionFailed
-import davcmds
+from interfaces import IDAVResource
+from interfaces import IWriteLock
+from WriteLockInterface import WriteLockInterface
+
+from zope.event import notify
+from zope.app.event.objectevent import ObjectCopiedEvent
+from zope.app.container.contained import ObjectMovedEvent
+from zope.app.container.contained import notifyContainerModified
+from OFS.event import ObjectClonedEvent
+from OFS.event import ObjectWillBeMovedEvent
+import OFS.subscribers
+
 
 class Resource(ExtensionClass.Base, Lockable.LockableItem):
+
     """The Resource mixin class provides basic WebDAV support for
     non-collection objects. It provides default implementations
     for most supported WebDAV HTTP methods, however certain methods
     such as PUT should be overridden to ensure correct behavior in
     the context of the object type."""
 
+    implements(IDAVResource)
+
     __dav_resource__=1
 
     __http_methods__=('GET', 'HEAD', 'POST', 'PUT', 'DELETE', 'OPTIONS',
@@ -219,7 +234,7 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         ifhdr = REQUEST.get_header('If', '')
         url = urlfix(REQUEST['URL'], 'DELETE')
         name = unquote(filter(None, url.split( '/')[-1]))
-        parent = self.aq_parent
+        parent = aq_parent(aq_inner(self))
         # Lock checking
         if Lockable.wl_isLocked(self):
             if ifhdr:
@@ -228,8 +243,9 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
                 # We're locked, and no if header was passed in, so
                 # the client doesn't own a lock.
                 raise Locked, 'Resource is locked.'
-        elif WriteLockInterface.isImplementedBy(parent) and \
-             parent.wl_isLocked():
+        elif (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             if ifhdr:
                 parent.dav__simpleifhandler(REQUEST, RESPONSE, 'DELETE', col=1)
             else:
@@ -356,8 +372,9 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         if existing:
             # The destination itself exists, so we need to check its locks
             destob = aq_base(parent)._getOb(name)
-            if WriteLockInterface.isImplementedBy(destob) and \
-               destob.wl_isLocked():
+            if (IWriteLock.providedBy(destob) or
+                    WriteLockInterface.isImplementedBy(destob)) and \
+                    destob.wl_isLocked():
                 if ifhdr:
                     itrue = destob.dav__simpleifhandler(
                         REQUEST, RESPONSE, 'COPY', refresh=1)
@@ -365,8 +382,9 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
                         raise PreconditonFailed
                 else:
                     raise Locked, 'Destination is locked.'
-        elif WriteLockInterface.isImplementedBy(parent) and \
-             parent.wl_isLocked():
+        elif (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             if ifhdr:
                 parent.dav__simpleifhandler(REQUEST, RESPONSE, 'COPY',
                                             refresh=1)
@@ -380,14 +398,22 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         if depth=='0' and isDavCollection(ob):
             for id in ob.objectIds():
                 ob._delObject(id)
+
+        notify(ObjectCopiedEvent(ob, self))
+
         if existing:
             object=getattr(parent, name)
             self.dav__validate(object, 'DELETE', REQUEST)
             parent._delObject(name)
+
         parent._setObject(name, ob)
         ob = parent._getOb(name)
         ob._postCopy(parent, op=0)
-        ob.manage_afterClone(ob)
+
+        OFS.subscribers.compatibilityCall('manage_afterClone', ob, ob)
+
+        notify(ObjectClonedEvent(ob))
+
         # We remove any locks from the copied object because webdav clients
         # don't track the lock status and the lock token for copied resources
         ob.wl_clearLocks()
@@ -451,8 +477,9 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         if existing:
             # The destination itself exists, so we need to check its locks
             destob = aq_base(parent)._getOb(name)
-            if WriteLockInterface.isImplementedBy(destob) and \
-               destob.wl_isLocked():
+            if (IWriteLock.providedBy(destob) or
+                    WriteLockInterface.isImplementedBy(destob)) and \
+                    destob.wl_isLocked():
                 if ifhdr:
                     itrue = destob.dav__simpleifhandler(
                         REQUEST, RESPONSE, 'MOVE', url=dest, refresh=1)
@@ -460,8 +487,9 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
                         raise PreconditionFailed
                 else:
                     raise Locked, 'Destination is locked.'
-        elif WriteLockInterface.isImplementedBy(parent) and \
-             parent.wl_isLocked():
+        elif (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             # There's no existing object in the destination folder, so
             # we need to check the folders locks since we're changing its
             # member list
@@ -483,20 +511,52 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
                 raise PreconditionFailed, 'Source is locked and no '\
                       'condition was passed in.'
 
+        orig_container = aq_parent(aq_inner(self))
+        orig_id = self.getId()
+
+        self._notifyOfCopyTo(parent, op=1)
+
+        notify(ObjectWillBeMovedEvent(self, orig_container, orig_id,
+                                      parent, name))
+
         # try to make ownership explicit so that it gets carried
         # along to the new location if needed.
         self.manage_changeOwnershipType(explicit=1)
 
-        self._notifyOfCopyTo(parent, op=1)
-        ob = aq_base(self._getCopy(parent))
-        self.aq_parent._delObject(absattr(self.id))
+        ob = self._getCopy(parent)
         ob._setId(name)
+
+        try:
+            orig_container._delObject(orig_id, suppress_events=True)
+        except TypeError:
+            # BBB: removed in Zope 2.11
+            orig_container._delObject(orig_id)
+            warnings.warn(
+                "%s._delObject without suppress_events is deprecated "
+                "and will be removed in Zope 2.11." %
+                orig_container.__class__.__name__, DeprecationWarning)
+
         if existing:
             object=getattr(parent, name)
             self.dav__validate(object, 'DELETE', REQUEST)
             parent._delObject(name)
-        parent._setObject(name, ob)
+
+        try:
+            parent._setObject(name, ob, set_owner=0, suppress_events=True)
+        except TypeError:
+            # BBB: removed in Zope 2.11
+            parent._setObject(name, ob, set_owner=0)
+            warnings.warn(
+                "%s._setObject without suppress_events is deprecated "
+                "and will be removed in Zope 2.11." %
+                parent.__class__.__name__, DeprecationWarning)
         ob = parent._getOb(name)
+
+        notify(ObjectMovedEvent(ob, orig_container, orig_id, parent, name))
+        notifyContainerModified(orig_container)
+        if aq_base(orig_container) is not aq_base(parent):
+            notifyContainerModified(parent)
+
         ob._postCopy(parent, op=1)
 
         # try to make ownership implicit if possible
@@ -598,5 +658,4 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
     def listDAVObjects(self):
         return []
 
-        
 Globals.default__class_init__(Resource)
