@@ -11,9 +11,9 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""Classes to support implenting `IContained`
+"""Classes to support implementing `IContained`
 
-$Id: contained.py 27409 2004-09-02 07:05:38Z pruggera $
+$Id: contained.py 40368 2005-11-25 15:09:45Z efge $
 """
 __docformat__ = 'restructuredtext'
 
@@ -27,21 +27,23 @@ from zope.interface import providedBy
 
 from zope.app import zapi
 from zope.app.exception.interfaces import UserError
-from zope.app.event.objectevent import ObjectEvent, modified
+from zope.app.event.objectevent import ObjectEvent
+from zope.app.event.objectevent import ObjectModifiedEvent
 from zope.event import notify
-from zope.app.i18n import ZopeMessageIDFactory as _
-from zope.app.container.interfaces import IContained, IReadContainer
+from zope.app.i18n import ZopeMessageFactory as _
+from zope.app.container.interfaces import IContained
 from zope.app.container.interfaces import INameChooser
 from zope.app.container.interfaces import IObjectAddedEvent
 from zope.app.container.interfaces import IObjectMovedEvent
 from zope.app.container.interfaces import IObjectRemovedEvent
+from zope.app.container.interfaces import IContainerModifiedEvent
 from zope.app.location.interfaces import ILocation, ISublocations
 from zope.app.container._zope_app_container_contained import ContainedProxyBase
 from zope.app.container._zope_app_container_contained import getProxiedObject
+from zope.app.broken.broken import IBroken
 
 class Contained(object):
-    """Stupid mix-in that defines `__parent__` and `__name__` attributes
-    """
+    """Stupid mix-in that defines `__parent__` and `__name__` attributes"""
 
     zope.interface.implements(IContained)
 
@@ -83,17 +85,22 @@ class ObjectRemovedEvent(ObjectMovedEvent):
             oldName = object.__name__
         ObjectMovedEvent.__init__(self, object, oldParent, oldName, None, None)
 
+class ContainerModifiedEvent(ObjectModifiedEvent):
+    """The container has been modified."""
+
+    zope.interface.implements(IContainerModifiedEvent)
+
+
 def dispatchToSublocations(object, event):
     """Dispatch an event to sublocations of a given object
 
-       When a move event happens for an object, it's important to
-       notify subobjects as well.
+       When a move event happens for an object, it's important to notify
+       subobjects as well.
 
        We do this based on locations.
 
        Suppose, for example, that we define some location objects.
 
-
          >>> class L(object):
          ...     zope.interface.implements(ILocation)
          ...     def __init__(self, name):
@@ -126,7 +133,7 @@ def dispatchToSublocations(object, event):
          ...         ),
          ...       L(13),
          ...       )
-         
+
        Now, if we call the dispatcher, it should call event handlers
        for all of the objects.
 
@@ -141,12 +148,13 @@ def dispatchToSublocations(object, event):
 
        Now we'll register it:
 
-         >>> from zope.app.tests import ztapi
-         >>> ztapi.handle([None, IObjectMovedEvent], handler)
+         >>> from zope.app.testing import ztapi
+         >>> ztapi.subscribe([None, IObjectMovedEvent], None, handler)
 
        We also register our dispatcher:
-       
-         >>> ztapi.handle([None, IObjectMovedEvent], dispatchToSublocations)
+
+         >>> ztapi.subscribe([None, IObjectMovedEvent], None,
+         ...                 dispatchToSublocations)
 
        We can then call the dispatcher for the root object:
 
@@ -168,7 +176,7 @@ def dispatchToSublocations(object, event):
        This suggests that location event handlers need to be aware that
        the objects they are called on and the event objects could be
        different.
-       
+
        """
     subs = ISublocations(object, None)
     if subs is not None:
@@ -195,7 +203,7 @@ class ContainerSublocations(object):
          >>> sublocations.sort()
          >>> sublocations
          [1, 2, 42]
-      
+
        """
 
     def __init__(self, container):
@@ -249,7 +257,7 @@ def containedEvent(object, container, name=None):
         True
         >>> item.__name__
         u'foo'
-    
+
     We don't get a new added event:
 
         >>> event
@@ -301,11 +309,27 @@ def containedEvent(object, container, name=None):
         True
         >>> x.__name__
         'foo'
+
+    Make sure we don't lose existing directly provided interfaces.
+
+        >>> from zope.interface import Interface, directlyProvides
+        >>> class IOther(Interface):
+        ...     pass
+        >>> from zope.app.location import Location
+        >>> item = Location()
+        >>> directlyProvides(item, IOther)
+        >>> IOther.providedBy(item)
+        True
+        >>> x, event = containedEvent(item, container, 'foo')
+        >>> IOther.providedBy(item)
+        True
     """
 
     if not IContained.providedBy(object):
         if ILocation.providedBy(object):
-            zope.interface.directlyProvides(object, IContained)
+            zope.interface.directlyProvides(
+                object, IContained,
+                zope.interface.directlyProvidedBy(object))
         else:
             object = ContainedProxy(object)
 
@@ -338,13 +362,17 @@ def contained(object, container, name=None):
     """
     return containedEvent(object, container, name)[0]
 
+def notifyContainerModified(object, *descriptions):
+    """Notify that the container was modified."""
+    notify(ContainerModifiedEvent(object, *descriptions))
+
 def setitem(container, setitemf, name, object):
     """Helper function to set an item and generate needed events
 
     This helper is needed, in part, because the events need to get
     published after the `object` has been added to the `container`.
 
-    If the item implements `IContained`, simply set it's `__parent__`
+    If the item implements `IContained`, simply set its `__parent__`
     and `__name__` attributes:
 
     >>> class IItem(zope.interface.Interface):
@@ -358,14 +386,14 @@ def setitem(container, setitemf, name, object):
 
     >>> from zope.app.container.interfaces import IObjectAddedEvent
     >>> from zope.app.container.interfaces import IObjectMovedEvent
-    >>> from zope.app.tests import ztapi
+    >>> from zope.app.testing import ztapi
 
-    >>> ztapi.handle([IItem, IObjectAddedEvent],
-    ...              lambda obj, event: obj.setAdded(event))
+    >>> ztapi.subscribe([IItem, IObjectAddedEvent], None,
+    ...                 lambda obj, event: obj.setAdded(event))
+
+    >>> ztapi.subscribe([IItem, IObjectMovedEvent], None,
+    ...                 lambda obj, event: obj.setMoved(event))
 
-    >>> ztapi.handle([IItem, IObjectMovedEvent],
-    ...              lambda obj, event: obj.setMoved(event))
-    
     >>> item = Item()
 
     >>> container = {}
@@ -543,7 +571,6 @@ def setitem(container, setitemf, name, object):
     ValueError: empty names are not allowed
 
     """
-
     # Do basic name check:
     if isinstance(name, str):
         try:
@@ -566,12 +593,12 @@ def setitem(container, setitemf, name, object):
     setitemf(name, object)
     if event:
         notify(event)
-        modified(container)
+        notifyContainerModified(container)
 
 fixing_up = False
 def uncontained(object, container, name=None):
     """Clear the containment relationship between the `object` and
-    the `container'
+    the `container`.
 
     If we run this using the testing framework, we'll use `getEvents` to
     track the events generated:
@@ -631,9 +658,9 @@ def uncontained(object, container, name=None):
     >>> len(getEvents(IObjectModifiedEvent))
     1
 
-    But, if either the name or parent are not ``None`` and they are not
-    the container and the old name, we'll get a modified event but not
-    a removed event.
+    But, if either the name or parent are not ``None`` and they are not the
+    container and the old name, we'll get a modified event but not a removed
+    event.
 
     >>> item.__parent__, item.__name__ = container, None
     >>> uncontained(item, container, u'foo')
@@ -663,15 +690,16 @@ def uncontained(object, container, name=None):
 
     if oldparent is not container or oldname != name:
         if oldparent is not None or oldname is not None:
-            modified(container)
+            notifyContainerModified(container)
         return
 
     event = ObjectRemovedEvent(object, oldparent, oldname)
     notify(event)
 
-    object.__parent__ = None
-    object.__name__ = None
-    modified(container)
+    if not IBroken.providedBy(object):
+        object.__parent__ = None
+        object.__name__ = None
+    notifyContainerModified(container)
 
 class NameChooser(object):
 
@@ -728,7 +756,7 @@ class NameChooser(object):
             i += 1
             n = name + u'-' + unicode(i) + suffix
 
-        # Make sure tha name is valid.  We may have started with something bad.
+        # Make sure the name is valid.  We may have started with something bad.
         self.checkName(n, object)
 
         return n
@@ -766,9 +794,8 @@ class DecoratorSpecificationDescriptor(
     >>> [interface.getName() for interface in list(providedBy(x))]
     ['I4', 'I3']
 
-    When we decorate objects, what order should the interfaces come
-    in?  One could argue that decorators are less specific, so they
-    should come last.
+    When we decorate objects, what order should the interfaces come in? One
+    could argue that decorators are less specific, so they should come last.
 
     >>> [interface.getName() for interface in list(providedBy(D1(x)))]
     ['I4', 'I3', 'I1', 'IContained', 'IPersistent']
@@ -819,6 +846,9 @@ class ContainedProxyClassProvides(zope.interface.declarations.ClassProvides):
 
 class ContainedProxy(ContainedProxyBase):
 
+    # Prevent proxies from having their own instance dictionaries:
+    __slots__ = ()
+
     __safe_for_unpickling__ = True
 
     zope.interface.implements(IContained)
