@@ -11,12 +11,13 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""Unautorized Exception View Class
+"""Unauthorized Exception View Class
 
-$Id: unauthorized.py 25177 2004-06-02 13:17:31Z jim $
+$Id: unauthorized.py 71681 2007-01-02 13:52:00Z adamg $
 """
+__docformat__ = 'restructuredtext'
+
 from zope.app import zapi
-from zope.app.security.interfaces import IAuthenticationService
 
 
 class Unauthorized(object):
@@ -25,12 +26,12 @@ class Unauthorized(object):
         # Set the error status to 403 (Forbidden) in the case when we don't
         # challenge the user
         self.request.response.setStatus(403)
+
+        # make sure that squid does not keep the response in the cache
+        self.request.response.setHeader('Expires', 'Mon, 26 Jul 1997 05:00:00 GMT')
+        self.request.response.setHeader('Cache-Control', 'no-store, no-cache, must-revalidate')
+        self.request.response.setHeader('Pragma', 'no-cache')
+        
         principal = self.request.principal
-        prinreg = zapi.getParent(principal)
-        if not IAuthenticationService.providedBy(prinreg):
-            # With PluggableAuthenticationService, principals are
-            # contained in the PrincipalSource, which is contained in
-            # the service.
-            prinreg = zapi.getParent(prinreg)
-        assert IAuthenticationService.providedBy(prinreg)
-        prinreg.unauthorized(principal.id, self.request)
+        auth = zapi.principals()
+        auth.unauthorized(principal.id, self.request)
