@@ -13,8 +13,12 @@
 ##############################################################################
 """Browser widgets for items
 
-$Id: itemswidgets.py 27726 2004-09-30 23:00:04Z jim $
+$Id: itemswidgets.py 74950 2007-05-01 03:42:46Z lra $
 """
+__docformat__ = 'restructuredtext'
+import sets
+from xml.sax.saxutils import escape
+
 from zope.interface import implements
 from zope.i18n import translate
 from zope.schema.interfaces import ValidationError, InvalidValue
@@ -24,38 +28,43 @@ from zope.app import zapi
 from zope.app.form.browser.widget import SimpleInputWidget, renderElement
 from zope.app.form.interfaces import IInputWidget, IDisplayWidget
 from zope.app.form.interfaces import ConversionError
+from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
 
-from zope.app.i18n import ZopeMessageIDFactory as _
+from zope.app.i18n import ZopeMessageFactory as _
 
 
 # For choices, we want to make the widget a view of the field and vocabulary.
 
 def ChoiceDisplayWidget(field, request):
-    return zapi.getMultiView((field, field.vocabulary), request, IDisplayWidget)
+    return zapi.getMultiAdapter((field, field.vocabulary, request),
+                                IDisplayWidget)
 
 def ChoiceInputWidget(field, request):
-    return zapi.getMultiView((field, field.vocabulary), request, IInputWidget)
+    return zapi.getMultiAdapter((field, field.vocabulary, request),
+                                IInputWidget)
 
-# for collections, we want to make the widget a view of the field and the 
+# for collections, we want to make the widget a view of the field and the
 # value_type.  If the value_type is None we may fall over.  We may
 # not be able to do any better than that.
 
 def CollectionDisplayWidget(field, request):
-    return zapi.getMultiView((field, field.value_type), request, IDisplayWidget)
+    return zapi.getMultiAdapter((field, field.value_type, request),
+                                IDisplayWidget)
 
 def CollectionInputWidget(field, request):
-    return zapi.getMultiView((field, field.value_type), request, IInputWidget)
+    return zapi.getMultiAdapter((field, field.value_type, request),
+                                IInputWidget)
 
-# for collections of choices, we want to make the widget a view of the field, 
+# for collections of choices, we want to make the widget a view of the field,
 # the value type, and the vocabulary.
 
 def ChoiceCollectionDisplayWidget(field, value_type, request):
-    return zapi.getMultiView(
-        (field, value_type.vocabulary), request, IDisplayWidget)
+    return zapi.getMultiAdapter((field, value_type.vocabulary, request),
+                                IDisplayWidget)
 
 def ChoiceCollectionInputWidget(field, value_type, request):
-    return zapi.getMultiView(
-        (field, value_type.vocabulary), request, IInputWidget)
+    return zapi.getMultiAdapter((field, value_type.vocabulary, request),
+                                IInputWidget)
 
 class TranslationHook(object):
     """A mixin class that provides the translation capabilities."""
@@ -63,12 +72,6 @@ class TranslationHook(object):
     def translate(self, msgid):
         return translate(msgid, context=self.request, default=msgid)
 
-def message(msgid, default):
-    """Add a default value to a i18n message id."""
-    msgid.default = default
-    return msgid
-
-
 class ItemsWidgetBase(TranslationHook, SimpleInputWidget):
     """Convenience base class for widgets displaying items/choices."""
 
@@ -91,33 +94,35 @@ class ItemsWidgetBase(TranslationHook, SimpleInputWidget):
     def __call__(self):
         """Render the widget to HTML."""
         raise NotImplementedError(
-            "__call__() must be implemented by a subclass; use _getFormValue()")
+            "__call__() must be implemented by a subclass; use _getFormValue()"
+            )
 
     def textForValue(self, term):
-        """Extract a string from the term.
+        """Extract a string from the `term`.
 
-        The term must be a vocabulary tokenized term. 
+        The `term` must be a vocabulary tokenized term.
 
-        This can be overridden to support more complex term objects. The token
-        is returned here since it's the only thing known to be a string, or
-        str()able."""
+        This can be overridden to support more complex `term`
+        objects. The token is returned here since it's the only thing
+        known to be a string, or str()able.
+
+        """
         if ITitledTokenizedTerm.providedBy(term):
-            return term.title
+            return self.translate(term.title)
         return term.token
 
     def convertTokensToValues(self, tokens):
         """Convert term tokens to the terms themselves.
 
         Tokens are used in the HTML form to represent terms. This method takes
-        the form tokens and converts them back to terms. 
+        the form tokens and converts them back to terms.
         """
         values = []
         for token in tokens:
             try:
                 term = self.vocabulary.getTermByToken(token)
             except LookupError, error:
-                raise InvalidValue, \
-                      "token %r not found in vocabulary" %token
+                raise InvalidValue("token %r not found in vocabulary" % token)
             else:
                 values.append(term.value)
         return values
@@ -133,7 +138,7 @@ class ItemsWidgetBase(TranslationHook, SimpleInputWidget):
                 self.empty_marker_name in self.request.form)
 
     def _toFieldValue(self, input):
-        """See SimpleInputWidget"""
+        """See `SimpleInputWidget`"""
         raise NotImplementedError(
             "_toFieldValue(input) must be implemented by a subclass\n"
             "It may be inherited from the mix-in classes SingleDataHelper\n"
@@ -143,7 +148,7 @@ class ItemsWidgetBase(TranslationHook, SimpleInputWidget):
 class SingleDataHelper(object):
     """Mix-in helper class for getting the term from the HTML form.
 
-    This is used when we expect a single input, i.e. the Choice field. 
+    This is used when we expect a single input, i.e. the Choice field.
     """
 
     def _toFieldValue(self, input):
@@ -156,12 +161,21 @@ class SingleDataHelper(object):
             return self.context.missing_value
 
     def hidden(self):
+        #XXX: _getFormValue() should return a string value that can be
+        #     used in a HTML form, but it doesn't. When
+        #     http://www.zope.org/Collectors/Zope3-dev/584 gets fixed
+        #     this hack should be reverted.
+        #     -- Bjorn Tillenius, 2006-04-12
+        value = self._getFormValue()
+        if value == self._missing:
+            form_value = ''
+        else:
+            form_value = self.vocabulary.getTerm(value).token
         return renderElement(u'input',
                              type='hidden',
                              name=self.name,
                              id=self.name,
-                             value=self.vocabulary.getTerm(
-                                self._getFormValue()).token,
+                             value=form_value,
                              cssClass=self.cssClass,
                              extra=self.extra)
 
@@ -176,14 +190,23 @@ class MultiDataHelper(object):
     def _toFieldValue(self, input):
         """See SimpleInputWidget"""
         if input is None:
-            return []
-        if not isinstance(input, list):
+            input = []
+        elif not isinstance(input, list):
             input = [input]
         try:
-            return self.convertTokensToValues(input)
+            values = self.convertTokensToValues(input)
         except InvalidValue, e:
             raise ConversionError("Invalid value", e)
 
+        # All AbstractCollection fields have a `_type` attribute specifying
+        # the type of collection. Use it to generate the correct type,
+        # otherwise return a list. 
+        if hasattr(self.context, '_type'):
+            return self.context._type(values)
+        else:
+            return values
+
+
     def _getDefault(self):
         # Return the default value for this widget;
         # may be overridden by subclasses.
@@ -198,7 +221,11 @@ class MultiDataHelper(object):
 class ItemDisplayWidget(SingleDataHelper, ItemsWidgetBase):
     """Simple single-selection display that can be used in many cases."""
 
-    _messageNoValue = message(_("item-missing-single-value-for-display"), "")
+    def __init__(self, *args, **kw):
+        ItemsWidgetBase.__init__(self, *args, **kw)
+        self.required = False
+
+    _messageNoValue = _("item-missing-single-value-for-display", "")
 
     def __call__(self):
         """See IBrowserWidget."""
@@ -213,8 +240,11 @@ class ItemDisplayWidget(SingleDataHelper, ItemsWidgetBase):
 class ItemsMultiDisplayWidget(MultiDataHelper, ItemsWidgetBase):
     """Displays a sequence of items."""
 
-    _messageNoValue = message(
-        _("vocabulary-missing-multiple-value-for-display"), "")
+    def __init__(self, *args, **kw):
+        ItemsWidgetBase.__init__(self, *args, **kw)
+        self.required = False
+
+    _messageNoValue = _("vocabulary-missing-multiple-value-for-display", "")
 
     itemTag = 'li'
     tag = 'ol'
@@ -225,8 +255,6 @@ class ItemsMultiDisplayWidget(MultiDataHelper, ItemsWidgetBase):
         if value:
             rendered_items = self.renderItems(value)
             return renderElement(self.tag,
-                                 type=self.type,
-                                 name=self.name,
                                  id=self.name,
                                  cssClass=self.cssClass,
                                  contents="\n".join(rendered_items),
@@ -243,9 +271,10 @@ class ItemsMultiDisplayWidget(MultiDataHelper, ItemsWidgetBase):
         tag = self.itemTag
         for item in value:
             term = self.vocabulary.getTerm(item)
-            items.append(renderElement(tag,
-                                       cssClass=cssClass,
-                                       contents=self.textForValue(term)))
+            items.append(renderElement(
+                tag,
+                cssClass=cssClass,
+                contents=escape(self.textForValue(term))))
         return items
 
 class ListDisplayWidget(ItemsMultiDisplayWidget):
@@ -255,7 +284,6 @@ class ListDisplayWidget(ItemsMultiDisplayWidget):
     """
     tag = 'ol'
 
-
 class SetDisplayWidget(ItemsMultiDisplayWidget):
     """Display widget for unordered multi-selection fields.
 
@@ -273,10 +301,11 @@ class ItemsEditWidgetBase(SingleDataHelper, ItemsWidgetBase):
     as value_type.
     """
     implements(IInputWidget)
-    
+
     size = 5
     tag = 'select'
-    firstItem = False
+
+    _displayItemForMissingValue = True
 
     def __init__(self, field, vocabulary, request):
         """Initialize the widget."""
@@ -300,8 +329,7 @@ class ItemsEditWidgetBase(SingleDataHelper, ItemsWidgetBase):
         contents.append(self._div('value', self.renderValue(value)))
         contents.append(self._emptyMarker())
 
-        return self._div(self.cssClass, "\n".join(contents),
-                         id=self.name)
+        return self._div(self.cssClass, "\n".join(contents))
 
 
     def _div(self, cssClass, contents, **kw):
@@ -316,7 +344,7 @@ class ItemsEditWidgetBase(SingleDataHelper, ItemsWidgetBase):
 
     def renderItemsWithValues(self, values):
         """Render the list of possible values, with those found in
-        'values' being marked as selected."""
+        `values` being marked as selected."""
 
         cssClass = self.cssClass
 
@@ -324,21 +352,38 @@ class ItemsEditWidgetBase(SingleDataHelper, ItemsWidgetBase):
         # vocabulary, so that need not be considered here
         rendered_items = []
         count = 0
+
+        # Handle case of missing value
+        missing = self._toFormValue(self.context.missing_value)
+
+        if self._displayItemForMissingValue and not self.context.required:
+            if missing in values:
+                render = self.renderSelectedItem
+            else:
+                render = self.renderItem
+
+            missing_item = render(count,
+                self.translate(self._messageNoValue),
+                missing,
+                self.name,
+                cssClass)
+            rendered_items.append(missing_item)
+            count += 1
+
+        # Render normal values
         for term in self.vocabulary:
             item_text = self.textForValue(term)
 
             if term.value in values:
-                rendered_item = self.renderSelectedItem(count,
-                                                        item_text,
-                                                        term.token,
-                                                        self.name,
-                                                        cssClass)
+                render = self.renderSelectedItem
             else:
-                rendered_item = self.renderItem(count,
-                                                item_text,
-                                                term.token,
-                                                self.name,
-                                                cssClass)
+                render = self.renderItem
+
+            rendered_item = render(count,
+                item_text,
+                term.token,
+                self.name,
+                cssClass)
 
             rendered_items.append(rendered_item)
             count += 1
@@ -346,16 +391,16 @@ class ItemsEditWidgetBase(SingleDataHelper, ItemsWidgetBase):
         return rendered_items
 
     def renderItem(self, index, text, value, name, cssClass):
-        """Render an item for a particular value."""
+        """Render an item for a particular `value`."""
         return renderElement('option',
-                             contents=text,
+                             contents=escape(text),
                              value=value,
                              cssClass=cssClass)
 
     def renderSelectedItem(self, index, text, value, name, cssClass):
-        """Render an item for a particular value that is selected."""
+        """Render an item for a particular `value` that is selected."""
         return renderElement('option',
-                             contents=text,
+                             contents=escape(text),
                              value=value,
                              cssClass=cssClass,
                              selected='selected')
@@ -364,35 +409,21 @@ class ItemsEditWidgetBase(SingleDataHelper, ItemsWidgetBase):
 class SelectWidget(ItemsEditWidgetBase):
     """Provide a selection list for the item."""
 
-    _messageNoValue = message(_("vocabulary-missing-single-value-for-edit"),
-                              "(no value)")
+    _messageNoValue = _("vocabulary-missing-single-value-for-edit",
+                        "(no value)")
 
     def renderValue(self, value):
         rendered_items = self.renderItems(value)
         contents = "\n%s\n" %"\n".join(rendered_items)
         return renderElement('select',
                              name=self.name,
+                             id=self.name,
                              contents=contents,
                              size=self.size,
                              extra=self.extra)
 
     def renderItems(self, value):
-        # check if we want to select first item
-        if (value == self.context.missing_value
-            and getattr(self, 'firstItem', False)
-            and len(self.vocabulary) > 0):
-            # Grab the first item from the iterator:
-            values = [iter(self.vocabulary).next().value]
-        elif value != self.context.missing_value:
-            values = [value]
-        else:
-            values = []
-        items = self.renderItemsWithValues(values)
-        if not self.context.required:
-            option = ('<option value="">%s</option>'
-                      %(self.translate(self._messageNoValue)))
-            items.insert(0, option)
-        return items
+        return self.renderItemsWithValues([value])
 
 
 class DropdownWidget(SelectWidget):
@@ -400,7 +431,7 @@ class DropdownWidget(SelectWidget):
     size = 1
 
 
-class RadioWidget(ItemsEditWidgetBase):
+class RadioWidget(SelectWidget):
     """Radio widget for single item choices.
 
     This widget can be used when the number of selections is going
@@ -408,10 +439,10 @@ class RadioWidget(ItemsEditWidgetBase):
     """
     orientation = "vertical"
 
-    _messageNoValue = message(_("vocabulary-missing-single-value-for-edit"),
-                              "(no value)")
+    _messageNoValue = _("vocabulary-missing-single-value-for-edit",
+                        "(no value)")
 
-    _joinButtonToMessageTemplate = u"%s&nbsp;%s"
+    _joinButtonToMessageTemplate = u"<label>%s&nbsp;%s</label>"
 
     def renderItem(self, index, text, value, name, cssClass):
         """Render an item of the list."""
@@ -422,7 +453,7 @@ class RadioWidget(ItemsEditWidgetBase):
                              id=id,
                              cssClass=cssClass,
                              type='radio')
-        return self._joinButtonToMessageTemplate %(elem, text)
+        return self._joinButtonToMessageTemplate % (elem, text)
 
     def renderSelectedItem(self, index, text, value, name, cssClass):
         """Render a selected item of the list."""
@@ -432,43 +463,9 @@ class RadioWidget(ItemsEditWidgetBase):
                              name=name,
                              id=id,
                              cssClass=cssClass,
-                             checked=None,
+                             checked="checked",
                              type='radio')
         return self._joinButtonToMessageTemplate %(elem, text)
-    
-    def renderItems(self, value):
-        # check if we want to select first item, the previously selected item
-        # or the "no value" item.
-        no_value = None
-        if (value == self.context.missing_value
-            and getattr(self, 'firstItem', False)
-            and len(self.vocabulary) > 0):
-            if self.context.required:
-                # Grab the first item from the iterator:
-                values = [iter(self.vocabulary).next().value]
-            else:
-                # the "no value" option will be checked
-                no_value = 'checked'
-        elif value != self.context.missing_value:
-            values = [value]
-        else:
-            values = []
-
-        items = self.renderItemsWithValues(values)
-        if not self.context.required:
-            kwargs = {
-                'value': '',
-                'name': self.name,
-                'cssClass': self.cssClass,
-                'type': 'radio'}
-            if no_value:
-                kwargs['checked']=no_value
-            option = renderElement('input', **kwargs)
-            option = self._joinButtonToMessageTemplate %(
-                option, self.translate(self._messageNoValue))
-            items.insert(0, option)
-
-        return items
 
     def renderValue(self, value):
         rendered_items = self.renderItems(value)
@@ -481,8 +478,9 @@ class RadioWidget(ItemsEditWidgetBase):
 class ItemsMultiEditWidgetBase(MultiDataHelper, ItemsEditWidgetBase):
     """Items widget supporting multiple selections."""
 
-    _messageNoValue = message(
-        _("vocabulary-missing-multiple-value-for-edit"), "(no values)")
+    _messageNoValue = _("vocabulary-missing-multiple-value-for-edit",
+                        "(no values)")
+    _displayItemForMissingValue = False
 
     def renderItems(self, value):
         if value == self.context.missing_value:
@@ -497,11 +495,12 @@ class ItemsMultiEditWidgetBase(MultiDataHelper, ItemsEditWidgetBase):
         rendered_items = self.renderItems(value)
         return renderElement(self.tag,
                              name=self.name + ':list',
-                             multiple=None,
+                             id=self.name,
+                             multiple='multiple',
                              size=self.size,
                              contents="\n".join(rendered_items),
                              extra=self.extra)
-    
+
     def hidden(self):
         items = []
         for item in self._getFormValue():
@@ -514,12 +513,58 @@ class ItemsMultiEditWidgetBase(MultiDataHelper, ItemsEditWidgetBase):
                               cssClass=self.cssClass,
                               extra=self.extra))
         return '\n'.join(items)
-         
+
 
 class MultiSelectWidget(ItemsMultiEditWidgetBase):
     """Provide a selection list for the list to be selected."""
 
 
+class MultiSelectSetWidget(MultiSelectWidget):
+    """Provide a selection list for the set to be selected."""
+
+    def _toFieldValue(self, input):
+        value = super(MultiSelectSetWidget, self)._toFieldValue(input)
+        if isinstance(value, list):
+            value = sets.Set(value)
+        return value
+
+
+class OrderedMultiSelectWidget(ItemsMultiEditWidgetBase):
+    """A multi-selection widget with ordering support."""
+
+    template = ViewPageTemplateFile('orderedSelectionList.pt')
+
+    def choices(self):
+        """Return a set of tuples (text, value) that are available."""
+        # Not all content objects must necessarily support the attributes
+        if hasattr(self.context.context, self.context.__name__):
+            available_values = self.context.get(self.context.context)
+        else:
+            available_values = []
+        return [{'text': self.textForValue(term), 'value': term.token}
+                for term in self.vocabulary
+                if term.value not in available_values]
+
+    def selected(self):
+        """Return a list of tuples (text, value) that are selected."""
+        # Get form values
+        values = self._getFormValue()
+        # Not all content objects must necessarily support the attributes
+        if hasattr(self.context.context, self.context.__name__):
+            # merge in values from content 
+            for value in self.context.get(self.context.context):
+                if value not in values:
+                    values.append(value)
+
+        terms = [self.vocabulary.getTerm(value)
+                 for value in values]
+        return [{'text': self.textForValue(term), 'value': term.token}
+                for term in terms]
+
+    def __call__(self):
+        return self.template()
+
+
 class MultiCheckBoxWidget(ItemsMultiEditWidgetBase):
     """Provide a list of checkboxes that provide the choice for the list."""
 
@@ -552,5 +597,5 @@ class MultiCheckBoxWidget(ItemsMultiEditWidgetBase):
                              name=name,
                              id=id,
                              value=value,
-                             checked=None)
+                             checked="checked")
         return self._joinButtonToMessageTemplate %(elem, text)
