@@ -13,12 +13,13 @@
 ##############################################################################
 """Select Widget Tests
 
-$Id: test_itemswidget.py 27000 2004-08-11 16:08:41Z garrett $
+$Id: test_itemswidget.py 74950 2007-05-01 03:42:46Z lra $
 """
+import sets
 import unittest
 
 from zope.interface import Interface, implements
-from zope.schema import Choice, List
+from zope.schema import Choice, List, Set, TextLine
 from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
 from zope.publisher.browser import TestRequest
 
@@ -33,9 +34,10 @@ from zope.app.form.browser.itemswidgets import SelectWidget, DropdownWidget
 from zope.app.form.browser.itemswidgets import RadioWidget
 from zope.app.form.browser.itemswidgets import ItemsMultiEditWidgetBase
 from zope.app.form.browser.itemswidgets import MultiSelectWidget
+from zope.app.form.browser.itemswidgets import OrderedMultiSelectWidget
 from zope.app.form.browser.itemswidgets import MultiCheckBoxWidget
 from zope.app.form.browser.tests.support import VerifyResults
-from zope.app.tests.placelesssetup import PlacelessSetup 
+from zope.app.testing.placelesssetup import PlacelessSetup 
 
 vocab = SimpleVocabulary(
     [SimpleTerm(value, token, title) for value, token, title in (
@@ -57,6 +59,12 @@ class ICollector(Interface):
         value_type=choice,
         required=False)
 
+    letters = Set(
+        title=u"Letters",
+        description=u"The Letters",
+        value_type=choice,
+        required=False)
+
 
 class Collector(object):
     implements(ICollector)
@@ -91,6 +99,12 @@ class ItemsWidgetBaseTest(VerifyResults, PlacelessSetup, unittest.TestCase):
         self.assertEqual(widget.name, 'foo.%s' %name)
         self.assertEqual(widget.empty_marker_name,
                          'foo.%s-empty-marker' %name)
+        # Declaring empty prefix
+        widget.setPrefix('')
+        self.assertEqual(widget._prefix, '')
+        self.assertEqual(widget.name, name)
+        self.assertEqual(widget.empty_marker_name,
+                         '%s-empty-marker' %name)
 
     def test_convertTokensToValues(self):
         widget = self._makeWidget()
@@ -113,6 +127,8 @@ class ItemDisplayWidgetTest(ItemsWidgetBaseTest):
         widget = self._makeWidget(form={'field.choice': 'token1'})
         self.assertEqual(widget(), 'One')
 
+    def test_not_required(self):
+        self.failIf(self._makeWidget().required)
 
 class ItemsMultiDisplayWidgetTest(ItemsWidgetBaseTest):
     
@@ -127,13 +143,10 @@ class ItemsMultiDisplayWidgetTest(ItemsWidgetBaseTest):
         widget = self._makeWidget(form={'field.numbers': ['token1', 'token2']})
         self.assertEqual(
             widget(),
-            '<%s class="textType" '
-            'id="field.numbers" '
-            'name="field.numbers" '
-            'type="text" >'
+            '<%s id="field.numbers" >'
             '<li>One</li>\n<li>Two</li>'
             '</%s>' %(self._tag, self._tag))
-        
+
     def test_renderItems(self):
         widget = self._makeWidget()
         self.assertEqual(
@@ -143,6 +156,15 @@ class ItemsMultiDisplayWidgetTest(ItemsWidgetBaseTest):
         self.assertRaises(TypeError, widget.renderItems, 1)
 
 
+    def test_not_required(self):
+        numbers = List(value_type=ICollector['choice']).bind(Collector(None))
+        request = TestRequest()
+        widget = self._widget(numbers, self._vocabulary, request) 
+        self.failIf(widget.required)
+
+
+        
+
 class ListDisplayWidgetTest(ItemsMultiDisplayWidgetTest):
     _widget = ListDisplayWidget
     _tag = 'ol'
@@ -197,11 +219,13 @@ class ItemsEditWidgetBaseTest(ItemsWidgetBaseTest):
              u'<option value="token2">Two</option>',
              u'<option value="token3">Three</option>'])             
 
-    def test_error(self):
-        widget = self._makeWidget(form={'field.choice': 'ten'})
-        widget.setPrefix('field.')
-        widget._getFormValue()
-        self.assert_(isinstance(widget._error, ConversionError))
+# This test is disabled because it tests for the presense of a missfeature,
+# which has been removed.  Did someone actually *want* this?
+##     def test_error(self):
+##         widget = self._makeWidget(form={'field.choice': 'ten'})
+##         widget.setPrefix('field.')
+##         widget._getFormValue()
+##         self.assert_(isinstance(widget._error, ConversionError))
     
     def test_hidden(self):
         widget = self._makeWidget(form={'field.choice': 'token2'})
@@ -223,9 +247,9 @@ class SelectWidgetTest(ItemsEditWidgetBaseTest):
         widget.context.required = False
         self.assertEqual(
             widget(),
-            '<div id="field.choice">\n'
+            '<div>\n'
             '<div class="value">\n'
-            '<select name="field.choice" size="%i" >\n'
+            '<select id="field.choice" name="field.choice" size="%i" >\n'
             '<option value="">(no value)</option>\n'
             '<option selected="selected" value="token1">One</option>\n'
             '<option value="token2">Two</option>\n'
@@ -239,7 +263,7 @@ class SelectWidgetTest(ItemsEditWidgetBaseTest):
         widget.setPrefix('field.')
         self.assertEqual(
             widget.renderValue('one'),
-            '<select name="field.choice" size="%i" >\n'
+            '<select id="field.choice" name="field.choice" size="%i" >\n'
             '<option selected="selected" value="token1">One</option>\n'
             '<option value="token2">Two</option>\n'
             '<option value="token3">Three</option>\n'
@@ -280,8 +304,8 @@ class SelectWidgetTest(ItemsEditWidgetBaseTest):
         widget.setPrefix('field.')
         widget.firstItem = True
         self.assertEqual(
-            widget.renderItems(None),
-            [u'<option selected="selected" value="token1">One</option>',
+            widget.renderItems(widget._toFormValue(widget.context.missing_value)),
+            [u'<option value="token1">One</option>',
              u'<option value="token2">Two</option>',
              u'<option value="token3">Three</option>'])
 
@@ -300,7 +324,7 @@ class RadioWidgetTest(ItemsEditWidgetBaseTest):
         widget = self._makeWidget()
         self.verifyResult(
             widget.renderItem('', 'Foo', 'foo', 'bar', None),
-            ['<input', 'type="radio"', 'name="bar"', 'value="foo"',
+            ['<label', '<input', 'type="radio"', 'name="bar"', 'value="foo"',
              'class="radioType"', '>&nbsp;Foo'])
         self.verifyResult(
             widget.renderItem('bar', 'Foo', 'foo', 'bar', 'klass'),
@@ -311,11 +335,11 @@ class RadioWidgetTest(ItemsEditWidgetBaseTest):
         widget = self._makeWidget()
         self.verifyResult(
             widget.renderSelectedItem('', 'Foo', 'foo', 'bar', 'klass'),
-            ['<input', 'type="radio"', 'name="bar"', 'value="foo"',
+            ['<label', '<input', 'type="radio"', 'name="bar"', 'value="foo"',
              'checked="checked"', '>&nbsp;Foo'])
         self.verifyResult(
             widget.renderSelectedItem('', 'Foo', 'foo', 'bar', 'klass'),
-            ['<input', 'type="radio"', 'name="bar"', 'value="foo"',
+            ['<label', '<input', 'type="radio"', 'name="bar"', 'value="foo"',
              'class="klass radioType"', 'checked="checked"', '>&nbsp;Foo'])
 
     def test_renderItemsWithValues(self):
@@ -325,7 +349,7 @@ class RadioWidgetTest(ItemsEditWidgetBaseTest):
         for index, item in enumerate(items):
             self.verifyResult(
                 item,
-                ['<input', 'class="radioType"', 'name="field.choice"',
+                ['<label', '<input', 'class="radioType"', 'name="field.choice"',
                  'id="field.choice.%i' %index, 'type="radio"',
                  'value="%s"' %values[index][0], 
                  '&nbsp;%s' %values[index][1]])
@@ -338,7 +362,7 @@ class RadioWidgetTest(ItemsEditWidgetBaseTest):
         for index, item in enumerate(items):
             self.verifyResult(
                 item,
-                ['<input', 'class="radioType"', 'name="field.choice"',
+                ['<label', '<input', 'class="radioType"', 'name="field.choice"',
                  'id="field.choice.%i' %index, 'type="radio"',
                  'value="%s"' %values[index][0], '&nbsp;%s' %values[index][1]])
         self.verifyResult(items[0], ['checked="checked"'])
@@ -354,28 +378,28 @@ class RadioWidgetTest(ItemsEditWidgetBaseTest):
         for index, item in enumerate(items):
             self.verifyResult(
                 item,
-                ['<input', 'class="radioType"', 'name="field.choice"',
+                ['<label', '<input', 'class="radioType"', 'name="field.choice"',
                  'type="radio"', 
                  'value="%s"' %values[index][0], '&nbsp;%s' %values[index][1]])
 
     def test_renderItems_firstItem(self):
         widget = self._makeWidget()
-        widget.firstItem = True
-        items = widget.renderItems(None)
+        items = widget.renderItems(
+                widget._toFormValue(widget.context.missing_value))
         values = [('token1','One'), ('token2','Two'), ('token3','Three')]
         for index, item in enumerate(items):
             self.verifyResult(
                 item,
-                ['<input', 'class="radioType"', 'name="field.choice"',
+                ['<label', '<input', 'class="radioType"', 'name="field.choice"',
                  'id="field.choice.%i"' %index, 'type="radio"',
                  'value="%s"' %values[index][0], '&nbsp;%s' %values[index][1]])
-        self.verifyResult(items[0], ['checked="checked"'])
 
     def test_renderValue(self):
         widget = self._makeWidget()
-        self.verifyResult(widget.renderValue(None), ['<br /><input'])
-        widget.orientation='horizontal'
-        self.verifyResult(widget.renderValue(None), ['&nbsp;&nbsp;<input'])
+        self.verifyResult(widget.renderValue(None), ['<br /><label><input'])
+        widget.orientation = 'horizontal'
+        self.verifyResult(widget.renderValue(None),
+                          ['&nbsp;&nbsp;<label><input'])
 
 
 class ItemsMultiEditWidgetBaseTest(ItemsEditWidgetBaseTest):
@@ -393,11 +417,26 @@ class ItemsMultiEditWidgetBaseTest(ItemsEditWidgetBaseTest):
              '>One</option>\n', 'value="token2"', '>Two</option>\n',
              'value="token3"', '>Three</option>', '</select>'])
 
-    def test_error(self):
-        widget = self._makeWidget(form={'field.numbers': ['ten']})
-        widget.setPrefix('field.')
-        widget._getFormValue()
-        self.assert_(isinstance(widget._error, ConversionError))
+    def test_renderItemsWithValues(self):
+        widget = self._makeWidget()
+        self.assertEqual(
+            widget.renderItemsWithValues(['one', 'two']),
+            [u'<option selected="selected" value="token1">One</option>',
+             u'<option selected="selected" value="token2">Two</option>',
+             u'<option value="token3">Three</option>'])
+        self.assertEqual(
+            widget.renderItemsWithValues([]),
+            [u'<option value="token1">One</option>',
+             u'<option value="token2">Two</option>',
+             u'<option value="token3">Three</option>'])
+
+# This test is disabled because it tests for the presense of a missfeature,
+# which has been removed.  Did someone actually *want* this?
+##     def test_error(self):
+##         widget = self._makeWidget(form={'field.numbers': ['ten']})
+##         widget.setPrefix('field.')
+##         widget._getFormValue()
+##         self.assert_(isinstance(widget._error, ConversionError))
     
     def test_hidden(self):
         widget = self._makeWidget(
@@ -409,12 +448,49 @@ class ItemsMultiEditWidgetBaseTest(ItemsEditWidgetBaseTest):
             ['<input', 'type="hidden"', 'value="token2"', 'id="field.numbers"', 
              'name="field.numbers:list"', 'value="token3"'])
 
+    def test_getInputValue(self):
+        widget = self._makeWidget(form={'field.numbers': ['token2','token3']})
+        widget.setPrefix('field.')
+        self.assertEqual(widget.getInputValue(), ['two', 'three'])
+
+        self._field = ICollector.get('letters')
+        widget = self._makeWidget(form={'field.letters-empty-marker': '1'})
+        widget.setPrefix('field.')
+        self.assertEqual(widget.getInputValue(), sets.Set())
+        widget = self._makeWidget(form={'field.letters': ['token2','token3']})
+        widget.setPrefix('field.')
+        self.assertEqual(widget.getInputValue(), sets.Set(['two', 'three']))
+        self._field = ICollector.get('numbers')
+
+
 
 class MultiSelectWidgetTest(ItemsMultiEditWidgetBaseTest):
 
     _widget = MultiSelectWidget
 
 
+class OrderedMultiSelectWidgetTest(ItemsMultiEditWidgetBaseTest):
+
+    _widget = OrderedMultiSelectWidget
+
+    def test_choices(self):
+        widget = self._makeWidget()
+        choices = [choice['text'] for choice in widget.choices()]
+        choices.sort()
+        self.assertEqual(choices, ['One', 'Three', 'Two'])
+
+    def test_selected(self):
+        widget = self._makeWidget(nums=['one'])
+        widget._data = ['two']
+        selected = [select['text'] for select in widget.selected()]
+        selected.sort()
+        self.assertEqual(selected, ['One', 'Two'])
+        widget._data = ['one']
+        selected = [select['text'] for select in widget.selected()]
+        selected.sort()
+        self.assertEqual(selected, ['One'])
+    
+
 class MultiCheckBoxWidgetTest(ItemsMultiEditWidgetBaseTest):
 
     _widget = MultiCheckBoxWidget
@@ -473,6 +549,7 @@ def test_suite():
     suite.addTest(unittest.makeSuite(RadioWidgetTest))
     suite.addTest(unittest.makeSuite(ItemsMultiEditWidgetBaseTest))
     suite.addTest(unittest.makeSuite(MultiSelectWidgetTest))
+    suite.addTest(unittest.makeSuite(OrderedMultiSelectWidgetTest))
     suite.addTest(unittest.makeSuite(MultiCheckBoxWidgetTest))
     return suite
 
