@@ -13,9 +13,9 @@
 ##############################################################################
 """Unit test logic for setting up and tearing down basic infrastructure
 
-$Id: placelesssetup.py 27079 2004-08-12 19:45:55Z srichter $
+$Id: placelesssetup.py 29143 2005-02-14 22:43:16Z srichter $
 """
-from zope.app.tests import ztapi
+from zope.app.testing import ztapi
 from zope.i18n.interfaces import IUserPreferredCharsets
 from zope.i18n.interfaces import IUserPreferredLanguages
 from zope.publisher.browser import BrowserLanguages
