@@ -16,10 +16,27 @@
 These exceptions are so general purpose that they don't belong in Zope
 application-specific packages.
 
-$Id: __init__.py 26796 2004-07-27 15:23:30Z jim $
+$Id: __init__.py 37531 2005-07-28 14:02:49Z anguenot $
 """
 
+import zope.deprecation
+
+zope.deprecation.deprecated('INotFoundError',
+                            'This interface has been deprecated. '
+                            'Use standard interface instead '
+                            'The reference will be gone in 3.3')
+
+zope.deprecation.deprecated('NotFoundError',
+                            'This class has been deprecated. '
+                            'Use standard exceptions instead '
+                            'The reference will be gone in 3.3')
+
+# Turn of deprecation warning here for the above import that are here for BBB
+# The depreaction above and within the _notfounderror module will do the job.
+zope.deprecation.__show__.off()
 from zope.exceptions._notfounderror import NotFoundError, INotFoundError
+zope.deprecation.__show__.on()
+
 from zope.exceptions._duplicate import DuplicationError, IDuplicationError
 
 # Importing these interfaces from here is deprecated!
@@ -28,9 +45,10 @@ from zope.exceptions._duplicate import DuplicationError, IDuplicationError
 try:
     import zope.security
 except ImportError, v:
-    if not str(v).endswith(' zope.security'):
+    # "ImportError: No module named security"
+    if not str(v).endswith(' security'):
         raise
-else:    
+else:
     from zope.security.interfaces import IUnauthorized, Unauthorized
     from zope.security.interfaces import IForbidden, IForbiddenAttribute
     from zope.security.interfaces import Forbidden, ForbiddenAttribute
