@@ -13,13 +13,17 @@
 ##############################################################################
 """Message ID tests.
 
-$Id: tests.py 25177 2004-06-02 13:17:31Z jim $
+$Id: tests.py 29137 2005-02-14 11:35:28Z tiran $
 """
 import unittest
-from zope.testing.doctestunit import DocTestSuite
+from zope.testing.doctestunit import DocTestSuite, DocFileSuite
 
 def test_suite():
-    return DocTestSuite('zope.i18nmessageid.messageid')
+    return unittest.TestSuite((
+	    DocTestSuite('zope.i18nmessageid.messageid'),
+	    DocTestSuite('zope.i18nmessageid.message'),
+	    DocFileSuite('messages.txt', package='zope.i18nmessageid'),
+	    ))
 
 if __name__ == '__main__':
     unittest.main(defaultTest="test_suite")
